/***************************************************************************
                          CdboActionFactory.cpp  -  description
                             -------------------
    begin                : Sat Aug 17 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/ 
#include <klocale.h>
#include <kdebug.h>

#include "CdboActionFactory.h"
#include "CdboActionFactory.moc"
#include "CdboAction.h"


#include "CdboEraseAction.h"
#include "CdboUtilActions.h"
#include "CdboFixateAction.h"
#include "CdboRipAction.h"
#include "CdboCopyCdAction.h"
#include "CdboReadDataCdAction.h"
#include "CdboCdrecordAction.h"
#include "CdboCdrdaoAction.h"
#include "CdboCreateDataCdAction.h"
#include "CdboCreateAudioCdAction.h"
#include "CdboMkisofsAction.h"
#include "CdboOgg123Action.h"
#include "CdboMpg123Action.h"
#include "CdboCdda2wavAction.h"

#include "CdboVersion.h"

K_EXPORT_COMPONENT_FACTORY( libcdboactions, CdboActionFactory)

KInstance *CdboActionFactory::mInstance = 0;
KAboutData *CdboActionFactory::mAboutData = 0;

CdboActionFactory::CdboActionFactory(QObject* parent, const char* name):
	KLibFactory (parent, name){
	actionDict.setAutoDelete(true);
}
//======================================
CdboActionFactory::~CdboActionFactory(){
	delete mAboutData;
	mAboutData = 0;
	delete mInstance;
	mInstance = 0;
	actionDict.clear();
}
//======================================
QObject* CdboActionFactory::createObject(QObject* parent, const char* name,
		const char* classname, const QStringList &) {

	if(!classname){
		kdError() << "CdboActionFactory::Missing classname for action" << endl;
		return 0;
	}
	if(!parent)
		kdDebug() << "CdboActionFactory::Missing parent for action" << endl;

	if(!name)
		return createAction(classname);
	else
		return createAction(classname, name);
}
//==============================
CdboAction* CdboActionFactory::createAction(QString classname, QString name){
	if(name.isNull()){
		kdDebug() << "CdboActionFactory::Will use classname as name: " << classname << endl;
		name = classname;
	}

	CdboAction* action = actionDict[classname];
	if(action)
		return action;

	if(QString(classname) == "CdboEraseAction")
		action = new CdboEraseAction (this, parent(), name.latin1());
	
	else if(QString(classname) == "CdboEraseProcAction")
		action = new CdboEraseProcAction(parent(), name.latin1());
		
	else if(QString(classname) == "CdboCheckDriveAction")
		action = new CdboCheckDriveAction(parent(), name.latin1());
	
	else if(QString(classname) == "CdboFixateAction")
		action = new CdboFixateAction(this, parent(), name.latin1());
	
	else if(QString(classname) == "CdboFixateProcAction")
		action = new CdboFixateProcAction(parent(), name.latin1());
		
	else if(QString(classname) == "CdboRipAction")
		action = new CdboRipAction(parent(), name.latin1());
	
	else if(QString(classname) == "CdboScanAudioTracksAction")
		action = new CdboScanAudioTracksAction(parent(), name.latin1());

	else if(QString(classname) == "CdboCopyCdAction")
		action = new CdboCopyCdAction(this, parent(), name.latin1());

	else if(QString(classname) == "CdboCdSizeAction")
		action = new CdboCdSizeAction(parent(), name.latin1());

	else if(QString(classname) == "CdboCheckTmpAction")
		action = new CdboCheckTmpAction(parent(), name.latin1());

	else if(QString(classname) == "CdboReadDataCdAction")
		action = new CdboReadDataCdAction(parent(), name.latin1());

	else if(QString(classname) == "CdboCdrecordAction")
		action = new CdboCdrecordAction(parent(), name.latin1());

	else if(QString(classname) == "CdboCdrdaoAction")
		action = new CdboCdrdaoAction(parent(), name.latin1());

	else if(QString(classname) == "CdboCreateDataCdAction")
		action = new CdboCreateDataCdAction(this, parent(), name.latin1());

	else if(QString(classname) == "CdboCreateAudioCdAction")
		action = new CdboCreateAudioCdAction(this, parent(), name.latin1());
	
	else if(QString(classname) == "CdboOgg123Action")
		action = new CdboOgg123Action(parent(), name.latin1());
	
	else if(QString(classname) == "CdboMpg123Action")
		action = new CdboMpg123Action(parent(), name.latin1());

	else if(QString(classname) == "CdboCreateDataOTFAction")
		action = new CdboCreateDataOTFAction(parent(), name.latin1());

	else if(QString(classname) == "CdboMkisofsAction")
		action = new CdboMkisofsAction(parent(), name.latin1());
	
	else if(QString(classname) == "CdboCdda2wavAction")
		action = new CdboCdda2wavAction(parent(), name.latin1());

	if(action){
		actionDict.insert(classname, action);
		connect(action, SIGNAL(actionDone(CdboAction*)),
			this, SLOT(actionDone(CdboAction*)));
	}
	return action;
}
//==================================
void CdboActionFactory::actionDone(CdboAction* action){
	kdDebug() << "CdboActionFactory::removing: " << action->className() << endl;
	actionDict.remove(action->className());
}
//======================================
KInstance *CdboActionFactory::instance()
{
	if(!mInstance)
		mInstance = new KInstance(aboutData());
	return mInstance;
}
//======================================
const KAboutData *CdboActionFactory::aboutData()
{
	if(!mAboutData)
	{
		mAboutData = new KAboutData( "cdbakeoven", I18N_NOOP("CD Bake Oven"),
			CDBO_VERSION, I18N_NOOP("Thank you for using CD Bake Oven - CD Burning facility for KDE"),
				KAboutData::License_GPL, "(c) 2001, Alexy Podolsky",
			I18N_NOOP("\nThis is free software with ABSOLUTELY NO WARRANTY."
		          "\nUse at your own risk! See --license for more details."),
		          "http://cdbakeoven.sourceforge.net");
		          mAboutData->addAuthor("Alexy Podolsky",
		          I18N_NOOP("Author and maintainer"),
		          "alexpod@users.sourceforge.net");
		    mAboutData->addAuthor("Ryan Cumming",
		     I18N_NOOP("UI tweaks and code cleanup"),
		          "ryan@completely.kicks-ass.org");
	}
	return mAboutData;
}


