/***************************************************************************
                          CdboCdda2wavAction.cpp  -  description
                             -------------------
    begin                : Fri Aug 23 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

 ***************************************************************************/
#include <qtimer.h>
 
#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <krun.h>
#include <kdebug.h>

#include "CdboCdda2wavAction.h"
#include "CdboCdda2wavAction.moc"

#include "CdboConstants.h"

CdboCdda2wavAction::CdboCdda2wavAction( QObject* d, const char* name) :
	CdboAction(d, name){
	
}
//---------------------------------------------------------------------------------------------
CdboCdda2wavAction::~CdboCdda2wavAction(){

}
//---------------------------------------------------------------------------------------------
void CdboCdda2wavAction::initProcess(KProcess *proc, CdboParamsDict* /*params*/){
	ignoreOutput = true;
	tracksDone = 1;
/////////////////////////////////////////////////////////////////////////////////////////////
	QString device = paramVal("SOURCE_DEVICE");
	if(device.isEmpty())
		return;
		
	QStringList tracksList;
	if(!listParamVal("PICKED_TRACKS", tracksList))
		return;

	QStringList outFiles;
	if(!listParamVal("WAV_CONVERTED_NAMES", outFiles))
		return;
	
	totalTracks = tracksList.count();
	
/////////////////////////////////////////////////////////////////////////////////////////////
	
	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdda2wav", "cdda2wav");
	KRun::shellQuote(exe);
	QString baseCommand = exe + " -g";
	    
	config->setGroup("Default Settings");
	if(config->readNumEntry("Level Of Output", Cdbo::StandartOutput) == Cdbo::CustomOutput){
		switch(config->readNumEntry("Custom Level Of Output", 1)){
			case 0: baseCommand += " --quiet -Q";break;
			case 1: break;
			case 2: baseCommand += " -v 2"; break;
			case 3: baseCommand += " -v 64"; break;
		}
	
		if(config->readBoolEntry("Show Debug Info", false))
			baseCommand += " -V";
	}

	config->setGroup("Audio Read Options");
	
	baseCommand += " -D " + device;
	
	int iBuff = config->readNumEntry("CD-Rom Speed", -1);
	if(iBuff > -1)
		baseCommand += " -S " + QString::number(iBuff);
		
	iBuff = config->readNumEntry("Transfer Size", -1);
	if(iBuff > -1)
		baseCommand += " -n " + QString::number(iBuff);
		
	if(config->readBoolEntry("Paranoia Mode", false))
		baseCommand += " -paranoia";
	
	switch(config->readNumEntry("CD Type", 0)){
		case 0: baseCommand += " -x"; break;
		case 1: baseCommand += " -c 2"; break;
		case 2: baseCommand += " -c 1"; break;
		case 3: baseCommand += " -c s"; break;
	}		
	if(config->readBoolEntry("Echo", false))
		baseCommand += " -e";
	
	if(config->readBoolEntry("Perform CDDB Lookup", true))
		baseCommand += " -L 1 -d 0";

	if(config->readBoolEntry("Split Output", true))
		baseCommand += " -B";
	
	baseCommand += " -Owav" ;
	
	switch(config->readNumEntry("CD-Rom Indianess", 0)){
		case 1: baseCommand += " -C big"; break;
		case 2: baseCommand += " -C little"; break;
	}
	switch(config->readNumEntry("Output Indianess", 0)){
		case 1: baseCommand += " -E big"; break;
		case 2: baseCommand += " -E little"; break;
	}

	
	/////////////////////////////////////////////////
	QString command = "";
	for(uint i = 0; i < tracksList.count(); i++)
		command += baseCommand + " -t " + tracksList[i] + "+" + tracksList[i] + " " + outFiles[i] + "; ";

	*proc << command;
}
//---------------------------------------------------------------------------------------------
bool CdboCdda2wavAction::filterOutput(int &type, QString &message){
	
	if(message.find("cdda2wav:") != -1 && tracksDone == 1)
		return true;

	if((message.startsWith("CDINDEX") || message.startsWith("CDDB") || 
			message.startsWith("CD-Text") || message.startsWith("CD-Extra") || 
			message.startsWith("using") || message.startsWith("cddb")) &&  tracksDone == 1){
		type = Cdbo::InfoMessage;
		return true;
	}
	
	if(message.find("overlap:min/max/cur") != -1 || message.find("percent_done") != -1){
		emitPercentInfo("CdboCdda2wavAction", 
			i18n("Reading track %1 of %2, percent done:").arg(QString::number(tracksDone)).arg(QString::number(totalTracks)));
		return false;
	}
	
	int spot = message.find("%");
	if(spot !=-1){
		message = message.left(spot);
		message = (message.right(message.length() - message.findRev(" "))).stripWhiteSpace();
		int m = message.toInt();

		if(m == 100 && tracksDone == totalTracks){
			emitPercent("CdboCdda2wavAction", m);
			ignoreOutput = false;
			return false;
		}

		if(m == 100 && tracksDone < totalTracks){
			tracksDone++;
			emitPercentInfo("CdboCdda2wavAction", 
				i18n("Reading track %1 of %2, percent done:").arg(QString::number(tracksDone)).arg(QString::number(totalTracks)));
			return false;
		}

		emitPercent("CdboCdda2wavAction", m);
		return false;
	}

	if(ignoreOutput)
		return false;
	
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboCdda2wavAction::processLaunched(){
	emitOutput(Cdbo::StatusOkMessage, i18n("Starting to read audio tracks."));
	emitStatus(i18n("Reading audio tracks from CD..."));
	config->setGroup("Audio Read Options");
	if(config->readBoolEntry("Perform CDDB Lookup", true))
		emitOutput(Cdbo::StatusOkMessage, i18n("Connecting to cddb server..."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboCdda2wavAction::processFailedToLaunch(){
	emitOutput(Cdbo::ErrorMessage,
		i18n("Read audio CD process failed to start. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboCdda2wavAction::processSucceded(){
	emitOutput(Cdbo::StatusOkMessage,
		i18n("Read audio CD process complited successfully."));
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboCdda2wavAction::processFailed(){
	emitOutput(Cdbo::StatusNotOkMessage,
		i18n("Failed to read tracks of this audio CD."));
	CdboAction::processFailed();
}

