/***************************************************************************
                          CdboCdrdaoAction.cpp  -  description
                             -------------------
    begin                : Fri Aug 23 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

 ***************************************************************************/
#include <qtimer.h>
 
#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <krun.h>
#include <kdebug.h>

#include "CdboCdrdaoAction.h"
#include "CdboCdrdaoAction.moc"

#include "CdboConstants.h"

CdboCdrdaoAction::CdboCdrdaoAction( QObject* d, const char* name) :
	CdboAction(d, name){

	instanceCount = 0;
}
//---------------------------------------------------------------------------------------------
CdboCdrdaoAction::~CdboCdrdaoAction(){

}
//---------------------------------------------------------------------------------------------
void CdboCdrdaoAction::initProcess(KProcess *proc, CdboParamsDict* /*params*/){
	lastMess = "";
	instanceCount++;
	firstTrack = true;
	askedToIncert = false;
	triedToCloseTray = false;
	readPercent = 101;
	incertRequests = 0;

/////////////////////////////////////////////////////////////////////////////////////////////
	//can be: read-cd or write or copy
	mode = paramVal("CDRDAO_MODE");
	if(mode.isEmpty())
		return;

	QString device = paramVal("TARGET_DEVICE");
	if(device.isEmpty())
		return;

	actionType = paramVal("ACTION_TYPE");
	if(actionType.isEmpty())
		return;

	QString speed = paramVal("SPEED");
	if(speed.isEmpty())
		return;
		
	QString outputFile = paramVal("IMAGE_NAME", false);

	bool audio = true;
	boolParamVal("IS_AUDIO", audio, false);
	
	bool onTheFly = false;
	boolParamVal("IS_ON_THE_FLY", onTheFly, false);
	
	
	//depend on mode
	bool required = false;
	if(mode == "copy" || mode == "read-cd")
		required = true;
	QString sourceDevice = paramVal("SOURCE_DEVICE", required);
	if(required && sourceDevice.isEmpty())
		return;
	sameDevice = sourceDevice == device;
	
	required = false;
	if(mode == "write" || mode == "read-cd" || (mode == "copy" && !onTheFly))
		required = true;
	QString tocName = paramVal("TOC_NAME", required);
	if(required && tocName.isEmpty())
		return;

	if(mode == "copy" && !onTheFly){
		tocToDelete = tocName;
		tocName = "";
	}
	else
		tocToDelete = "";
		
	if(mode == "copy" || mode == "read-cd")
		emitStatus(i18n("Preparing for read..."));
	else if(mode == "write")
		emitStatus(i18n("Preparing for write..."));
	
/////////////////////////////////////////////////////////////////////////////////////////////

	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdrdao", "cdrdao");
	KRun::shellQuote(exe);
	*proc << exe;
	
	*proc << mode;
	
	config->setGroup("Default Settings");
	if(config->readNumEntry("Level Of Output", Cdbo::StandartOutput) == Cdbo::CustomOutput){
		switch(config->readNumEntry("Custom Level Of Output", 1)){
			case 0: break;
			case 1: *proc  << "-v 1"; break;
			case 2: *proc  << "-v 2"; break;
			case 3: *proc  << "-v 3"; break;
		}
		if(config->readBoolEntry("Show Debug Info", false))
			*proc  << "-v 4";
	}

	if(onTheFly)
		*proc  << "--on-the-fly";
	
	if(actionType == "dummy"){
		emitStatus(i18n("Writing (dummy)..."));
		*proc  << "--simulate";
	}

	config->setGroup("Recording Options");
	if(config->readBoolEntry("Eject Media At End", true))
		*proc  << "--eject";

	*proc  << "--speed " + speed;

	if(config->readBoolEntry("Overburn CD", false))
		*proc  << "--overburn";

	if(config->readBoolEntry("Use Custom cdrdao Options", false))
		addCustomOpt(proc);

	if(!outputFile.isEmpty()){
		KRun::shellQuote(outputFile);
		*proc  << "--datafile " + outputFile + " --keepimage";
	}
	
	if(audio && tocName.isEmpty())
		addAudioOpt(proc);
	
	*proc  << "--device " + device;
	if(!sourceDevice.isEmpty() && !sameDevice)
		*proc  << "--source-device " + sourceDevice;
	
	config->setGroup("cdrdao Driver Mappings");
	*proc << "--driver" <<  config->readEntry("Driver for " + device, "generic-mmc");
	if(!sourceDevice.isEmpty() && !sameDevice)
		*proc << "--source-driver" <<  config->readEntry("Driver for " + sourceDevice, "generic-mmc");
		
	if(!tocName.isEmpty()){
		KRun::shellQuote(tocName);
		*proc << tocName;
	}
}
//---------------------------------------------------------------------------------------------
void CdboCdrdaoAction::addCustomOpt(KProcess *proc){
	config->setGroup("Recording Options");
	if(config->readBoolEntry("Force cdrdao", false))
		*proc  << "--force";
	if(config->readBoolEntry("Reload cdrdao", false))
		*proc  << "--reload";
	*proc  << "--buffers " + QString::number(config->readNumEntry("Number of Buffers for cdrdao", 32));

}
//---------------------------------------------------------------------------------------------
void CdboCdrdaoAction::addAudioOpt(KProcess *proc){
	config->setGroup("Audio Read Options");
	if(config->readBoolEntry("Perform CDDB Lookup", true))
		*proc  << "--with-cddb";
	if(config->readBoolEntry("Read Raw Toc", false))
		*proc  << "--read-raw";
	if(config->readBoolEntry("Fast Toc Lookup", true))
		*proc  << "--fast-toc";
	if(config->readBoolEntry("Tao Source", false))
		*proc  << "--tao-source";

	*proc  << "--paranoia-mode " + QString::number(config->readNumEntry("Correction Mode", 3));

}
//---------------------------------------------------------------------------------------------
bool CdboCdrdaoAction::filterOutput(int &type, QString &message){

	if(message.find("Please insert a recordable medium") != -1 || 
			(sameDevice && message.find("Medium in recorder device is not empty") != -1)){
		if(askedToIncert)
			return false;
		
		askedToIncert = true;
		requestCDR(true, paramVal("SOURCE_DEVICE_NAME", false));
		askedToIncert = false;
		return false;
	}
	if(message.find("hit return to reload") != -1){
		requestReload();
		return false;
	}
	
	if(message.find("hit enter to try again") != -1/* || message.find("Unit not ready, still") != -1*/){
		if(!triedToCloseTray){
			triedToCloseTray = true;
			closeTray(paramVal("SOURCE_DEVICE_NAME", false));
		}
		if(incertRequests == 20){
			triedToCloseTray = false;
			requestReload();
			incertRequests = 0;
		}
		incertRequests++;
		sendSignalToProcess();
		
		return false;
	}
	
	if(message.startsWith("Leadout")){
		totalTime = message.mid(message.find("(") - 8, 8);
		oneReadPecent = totalTime.section(":", 0, 0).toFloat() * 60 + totalTime.section(":", 1, 1).toFloat();
		oneReadPecent = oneReadPecent / 100;
		if(oneReadPecent == 0)
			oneReadPecent = 0.1;
	}

	if(message.find("Track") != -1 && message.endsWith("...")){ 
		message = message.right(message.length() - message.findRev(" ")).stripWhiteSpace();
		message = message.left(message.length() - 3);
		emitPercent("CdboCdrdaoAction-1", 0);
		triedToCloseTray = false;
		emitStatus(i18n("Reading audio tracks from CD..."));
		emitPercentInfo("CdboCdrdaoAction-1", i18n("Reading track %1. Total percent done:").arg(message) );
		return false;
	}
	
	if(message.startsWith("Copying data track")){ 
		message = message.section(" ", 3,3);
		
		emitPercent("CdboCdrdaoAction-1", 0);
		emitStatus(i18n("Reading data tracks from CD..."));
		emitPercentInfo("CdboCdrdaoAction-1", i18n("Reading track %1. Total percent done:").arg(message) );
		return false;
	}
	if(message.startsWith("Corresponding toc-file is written")){
		emitPercent("CdboCdrdaoAction-1", 100, i18n("This shows time remaining as in '00:01:00 of 00:03:34' so at least 'of' needs to be translated", 
			", %1 of %1").arg(totalTime).arg(totalTime));
		tocToDelete += message.section("\"", 1,1);
		return false;
	}
	if(message.stripWhiteSpace().length() == 8 && message.contains(":") == 2){
		if((int)readPercent == 100)
			return false;
		readPercent = message.section(":", 0, 0).toFloat() * 60 + message.section(":", 1, 1).toFloat();
		readPercent = readPercent / oneReadPecent;
		if((int)readPercent >= 100){
			emitOutput(Cdbo::ProcessMessage, i18n("Postprocessing..."));
			message = totalTime;
		}
		emitPercent("CdboCdrdaoAction-1", (int)readPercent, i18n("This shows time remaining as in '00:01:00 of 00:03:34' so at least 'of' needs to be translated", 
			", %1 of %1").arg(message.stripWhiteSpace()).arg(totalTime));
		return false;
	}
	
	 if(message.contains(":") > 1 && (int)readPercent < 100)
	 	return false;
		
	if(message.find("Found pre-gap:") != -1)
		return false;

	if(message.startsWith("Pausing 10 seconds")){
		secCount = 9;
		QTimer::singleShot(1000, this, SLOT(updateCounter()));
		type = Cdbo::StatusOkMessage;
		if(actionType == "dummy")
			message =  i18n("Last chance to quit, starting dummy write in %1 seconds").arg(QString::number(secCount));
		else
			message =  i18n("Last chance to quit, starting real write in %1 seconds").arg(QString::number(secCount));
	}
	if(message.startsWith("Wrote")){
		float done = message.section(" ", 1, 1).toFloat();
		float totalToDo = message.section(" ", 3, 3).toFloat();

		done = done / (totalToDo / 100);
		if(done > 100)
			done = 100;
		emitPercent("CdboCdrdaoAction" + QString::number(instanceCount), (int)done, i18n(", %1").arg(message.section(" ", 1)));
		return false;
	}

	if(message.startsWith("Writing track")){

		message = message.section(" ", 0, 2).stripWhiteSpace();
		message += i18n(". Total percent done:");

		if(firstTrack){
			firstTrack = false;
			emitStatus(i18n("Writing..."));
			emitPercent("CdboCdrdaoAction" + QString::number(instanceCount), 0);
			emitPercentInfo("CdboCdrdaoAction" + QString::number(instanceCount), message );
		}
		else
			emitPercentInfo("CdboCdrdaoAction" + QString::number(instanceCount), message );
		return false;
	}

	if(message.startsWith("Found") && lastMess.startsWith("Analyzing"))
		return false;
	
	if(message.startsWith("Writing finished") || message.startsWith("CD copying") || message.startsWith("Keeping created") || message.startsWith("Process can be aborted"))
		return false;
		
	if(message.startsWith("Analyzing") && lastMess.startsWith("Analyzing")){
		type = Cdbo::OverwritePrevious;
		lastMess = message;
	}
	if(lastMess.startsWith("Analyzing")){
		if(!message.isEmpty()){
			bool isNum = false;
			message.section(":", 0, 0).toInt(&isNum);
			if(isNum){
				type = Cdbo::OverwritePrevious;
				lastMess = message;
			}
		}
	}

	return true;
}
//---------------------------------------------------------------------------------------------
void CdboCdrdaoAction::updateCounter(){
	if(secCount == 0)
		return;
	secCount--;
	QString buff;
	if(actionType == "dummy")
		buff =  i18n("Last chance to quit, starting dummy write in %1 seconds").arg(QString::number(secCount));
	else
		buff =  i18n("Last chance to quit, starting real write in %1 seconds").arg(QString::number(secCount));

	emitOutput(Cdbo::OverwritePrevious, buff);
	QTimer::singleShot(1000, this, SLOT(updateCounter()));
}
//---------------------------------------------------------------------------------------------
void CdboCdrdaoAction::processLaunched(){
	if(actionType == "dummy")
		emitOutput(Cdbo::StatusOkMessage, i18n("Starting burn process in test mode."));
	else
		emitOutput(Cdbo::StatusOkMessage, i18n("Starting burn process."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboCdrdaoAction::processFailedToLaunch(){
	if(actionType == "dummy")
		emitOutput(Cdbo::ErrorMessage, i18n("Test failed to start. Make sure you have enough privileges."));
	else
		emitOutput(Cdbo::ErrorMessage, i18n("Burn process failed to start. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboCdrdaoAction::processSucceded(){
	if(actionType == "dummy")
		emitOutput(Cdbo::StatusOkMessage, i18n("Test completed successfully."));
	else
		emitOutput(Cdbo::StatusOkMessage, i18n("Burn completed successfully."));
	emitPercent("CdboCdrdaoAction" + QString::number(instanceCount), 100);
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboCdrdaoAction::processFailed(){
	if(actionType == "dummy")
		emitOutput(Cdbo::StatusNotOkMessage, i18n("Test burn failed."));
	else
		emitOutput(Cdbo::StatusNotOkMessage, i18n("Burn failed."));
	CdboAction::processFailed();
}
//---------------------------------------------------------------------------------------------
void CdboCdrdaoAction::resetSettings(){
	if(!tocToDelete.isEmpty())
		removeOneFile(tocToDelete, false);
	CdboAction::resetSettings();
}

