/***************************************************************************
                          CdboCopyCdAction.cpp  -  description
                             -------------------
    begin                : Sun Sep 1 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtimer.h>
#include <qregexp.h>

#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <krun.h>
#include <kdebug.h>

#include "CdboCopyCdAction.h"
#include "CdboCopyCdAction.moc"

#include "CdboConstants.h"

CdboCopyCdAction::CdboCopyCdAction(KLibFactory* factory, QObject* d, const char* name) :
	CdboActionHandler(factory, d, name){

}
//---------------------------------------------------------------------------------------------
CdboCopyCdAction::~CdboCopyCdAction(){

}
//---------------------------------------------------------------------------------------------
void CdboCopyCdAction::initProcess(KProcess *, CdboParamsDict* params){
	readyToLaunch = false;
	this->params = params;

/////////////////////////////////////////////////////////////////////////////////////////////

	bool audio = false;
	boolParamVal("IS_AUDIO", audio, false);
	
	bool daoMode = false;
	boolParamVal("IS_DAO", daoMode, false);
	
	bool onTheFly = false;
	boolParamVal("IS_ON_THE_FLY", onTheFly, false);
	
	bool multyStepAudio = false;
	if(!boolParamVal("IS_MULTY_STEP_AUDIO", multyStepAudio, audio && !onTheFly) && audio && !onTheFly)
		return;
	if(!multyStepAudio && audio )
		daoMode = true;
	
	QString tmpDir = paramVal("TMP_DIRECTORY", !onTheFly);
	if(tmpDir.isEmpty() && !onTheFly)
		return;
	
	isoImageName = "";
	if(onTheFly && !audio && !daoMode){//if data on-the-fly
		isoImageName = paramVal("IMAGE_NAME");
		if(isoImageName.isEmpty())
			return;
	}
	else if(!onTheFly && !audio && !daoMode){//if data
		config->setGroup("Default Settings");
		isoImageName = config->readEntry("Custom ISO Name", "cdboImage_<DATE>.iso");
		if(isoImageName.contains("<DATE>"))
			isoImageName.replace(QRegExp("<DATE>"), timeStamp());
		isoImageName = tmpDir + isoImageName;
		setParamVal("IMAGE_NAME", &isoImageName);
		filesToCleanup.append(isoImageName);
	}
	else if(multyStepAudio){ //if audio track-by-track  give tmp files names
		daoMode = false;
	
		QStringList pickedTracksList;
		if(!listParamVal("PICKED_TRACKS", pickedTracksList, "\n", false)){
			QString device = paramVal("SOURCE_DEVICE");
			if(device.isEmpty())
				return;
				
			totalTracks = 0;
			waitingForProcess = true;
			countAudioTracks(device);
			while(waitingForProcess)
				qApp->processEvents(1);
			
			for(int i = 1; i <= totalTracks; i++)
				pickedTracksList.append(QString::number(i));
		
			pickedTracksListHolder = pickedTracksList.join("\n");
			setParamVal("PICKED_TRACKS", &pickedTracksListHolder);
		}
		
		QString baseName = tmpDir + "cdbo_audio_" + timeStamp() + "_";
		QString buff;
		convertedNamesHolder = "";
		for(uint i = 0; i < pickedTracksList.count(); i++){
			buff = baseName + pickedTracksList[i] + ".wav";
			filesToCleanup.append(buff);
			
			KRun::shellQuote(buff);
			isoImageName += buff + " ";
			convertedNamesHolder += buff + "\n";
			
		}
		setParamVal("WAV_CONVERTED_NAMES", &convertedNamesHolder);
		setParamVal("IMAGE_NAME", &isoImageName);
	}
	else if(!onTheFly && daoMode){//if dao
		QString baseName = tmpDir + "cdbo_audio_" + timeStamp();
		
		isoImageName = baseName + ".bin";
		setParamVal("IMAGE_NAME", &isoImageName);
		filesToCleanup.append(isoImageName);
		
		tocName = tmpDir;
		setParamVal("TOC_NAME", &tocName);
	}
	
	if(daoMode){
		cdrdaoMode = "copy";
		setParamVal("CDRDAO_MODE", &cdrdaoMode);
	}
	
	withCleanUp = true;
	boolParamVal("IS_WITH_CLEAN_UP", withCleanUp, false);
	if(onTheFly)
		withCleanUp = false;

/////////////////////////////////////////////////////////////////////////////////////////////


	if(!audio && !daoMode){
		if(!createAction("CdboCdSizeAction")){
			processFailed();
			return;
		}
	}
	config->setGroup("Default Settings");
	if(!onTheFly && config->readBoolEntry("Check Tmp Size", true)){
		if(!createAction("CdboCheckTmpAction")){
			processFailed();
			return;
		}
	}
	config->setGroup("Recording Options");
	if(config->readBoolEntry("Check Drive First", false)){
		if(!createAction("CdboCheckDriveAction")){
			processFailed();
			return;
		}
	}


	if(!onTheFly && !daoMode && !audio){
		if(!createAction("CdboReadDataCdAction")){
			processFailed();
			return;
		}
	}
	
	if(multyStepAudio){
		if(!createAction("CdboCdda2wavAction")){
			processFailed();
			return;
		}
	}
	
	if(daoMode){
		if(!createAction("CdboCdrdaoAction")){
			processFailed();
			return;
		}
	}
	else{
		if(!createAction("CdboCdrecordAction")){
			processFailed();
			return;
		}
	}
	
	QTimer::singleShot(1, this, SLOT(begin()));
}
//---------------------------------------------------------------------------------------------
void CdboCopyCdAction::countAudioTracks(const QString &device){
	gotAnswer = false;
	emitStatus(i18n("Counting audio tracks..."));
	emitOutput(Cdbo::InfoMessage, i18n("Counting audio tracks..."));

	
	KShellProcess *cdboProc = new KShellProcess();
	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdda2wav", "cdda2wav");
	KRun::shellQuote(exe);
	*cdboProc << exe << "-J" << "-g" << "-D" << device;
	
	debugProcess(cdboProc);
	QApplication::connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(curProcDoneSlot(KProcess *)));
 	QApplication::connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedCountAudioMessageSlot(KProcess *, char *, int)));
	QApplication::connect(cdboProc, SIGNAL(receivedStderr (KProcess *, char *, int)),
		this, SLOT(receivedCountAudioMessageSlot(KProcess *, char *, int)));

	if(!cdboProc->start(KProcess::NotifyOnExit, KProcess::AllOutput)){
		emitOutput(Cdbo::ErrorMessage, i18n("Failed to count number of tracks. "\
				"Make sure you have enough privileges."));
		delete cdboProc;
		processFailed();
	}
}
//---------------------------------------------------------------------------------------------
void CdboCopyCdAction::receivedCountAudioMessageSlot(KProcess *proc, char *buffer, int buflen){	
	if(gotAnswer)
		return;
	if(buflen > 0){
		QString buff(buffer);
		if(buff.contains("Permission denied")) {
			gotAnswer = true;
			emitOutput(Cdbo::ErrorMessage, i18n("Failed to count number of tracks. "\
				"Make sure you have enough privileges."));
			processFailed();
			return;
		}
		if(buff.contains("Inquiry command failed") || buff.contains("error") || buff.contains("load cdrom please") ||
				buff.contains("Invalid argument") || buff.contains("No medium found")) {
			gotAnswer = true;
			emitOutput(Cdbo::ErrorMessage, i18n("Failed to count number of tracks. "\
				"This device does not seem to contain a valid audio CD."));
			processFailed();
			return;
		}
	
		buff = buff.left(buflen);
		buff = buff.replace(QRegExp("\n")," ");
		int pos = buff.find("Tracks:");
		if(pos != -1){
			gotAnswer = true;
			pos += 7;
			buff = buff.right(buff.length() - pos);
			buff = buff.left(buff.find("."));
			
			QString total = buff.left(buff.find(" ")).stripWhiteSpace();
			QString totalTime = buff.right(buff.length() - buff.find(" "));
			emitOutput(Cdbo::InfoMessage, i18n("Reading all tracks from Audio CD"));
			emitOutput(Cdbo::InfoMessage, i18n("Total Tracks: %1,  Total Time: %2").arg(total).arg(totalTime));
			
			totalTracks =  total.toInt();
			proc->kill();
			waitingForProcess = false;
		}
 	}
}
//---------------------------------------------------------------------------------------------
void CdboCopyCdAction::curProcDoneSlot(KProcess *proc){
	//waitingForProcess = false;
	delete proc;
}
//---------------------------------------------------------------------------------------------
void CdboCopyCdAction::processSucceded(){
	emitOutput(Cdbo::StatusOkMessage, i18n("CD successfully copied."));
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboCopyCdAction::processFailed(){
	emitOutput(Cdbo::StatusNotOkMessage, i18n("Copy process failed."));
	CdboAction::processFailed();
}
//---------------------------------------------------------------------------------------------
void CdboCopyCdAction::resetSettings(){
	CdboActionHandler::resetSettings();
	
	if(withCleanUp && filesToCleanup.count() > 0){
		for(uint i = 0; i < filesToCleanup.count(); i++)
			removeOneFile(filesToCleanup[i], false);
	}
}
