/***************************************************************************
                          CdboFixateAction.cpp  -  description
                             -------------------
    begin                : Fri Aug 16 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/
#include <qtimer.h>
#include <qregexp.h>

#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <krun.h>

#include "CdboFixateAction.h"
#include "CdboFixateAction.moc"

#include "CdboConstants.h"


CdboFixateAction::CdboFixateAction(KLibFactory* factory, QObject* d, const char* name) :
	CdboActionHandler(factory, d, name){
}
//---------------------------------------------------------------------------------------------
CdboFixateAction::~CdboFixateAction(){

}
//---------------------------------------------------------------------------------------------
void CdboFixateAction::initProcess(KProcess *, CdboParamsDict* params){
	readyToLaunch = false;
	this->params = params;

	config->setGroup("Recording Options");
	if(config->readBoolEntry("Check Drive First", false)){
		if(!createAction("CdboCheckDriveAction")){
			processFailed();
			return;
		}
	}
	if(!createAction("CdboFixateProcAction")){
		processFailed();
		return;
	}
	QTimer::singleShot(1, this, SLOT(begin()));
}
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

CdboFixateProcAction::CdboFixateProcAction( QObject* d, const char* name) :
	CdboAction(d, name){

}
//---------------------------------------------------------------------------------------------
CdboFixateProcAction::~CdboFixateProcAction(){

}
//---------------------------------------------------------------------------------------------
void CdboFixateProcAction::initProcess(KProcess *proc, CdboParamsDict* /*params*/){
	secCount = 100;
/////////////////////////////////////////////////////////////////////////////////////////////

	QString device = paramVal("TARGET_DEVICE");
	if(device.isEmpty())
		return;

	actionType = paramVal("ACTION_TYPE");
	if(actionType.isEmpty())
		return;

	QString speed = paramVal("SPEED");
	if(speed.isEmpty())
		return;
		
	bool ejectAtEnd = false;
	boolParamVal("EJECT_AFTER_FIXATE", ejectAtEnd, false);
		
////////////////////////////////////////////////////////////////////////////////////////////

	emitStatus(i18n("Fixating..."));
	////////////////////////////////////////////////////////////

	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdrecord", "cdrecord");
	KRun::shellQuote(exe);
	*proc << exe;

	config->setGroup("Default Settings");
	if(config->readNumEntry("Level Of Output", Cdbo::StandartOutput) == Cdbo::CustomOutput){
		switch(config->readNumEntry("Custom Level Of Output", 1)){
			case 0: *proc  << "-s"; break;
			case 1: *proc  << "-v"; break;
			case 2: *proc  << "-V"; break;
			case 3: *proc  << "-VV"; break;
		}
		if(config->readBoolEntry("Show Debug Info", false))
			*proc  << "-d";
	}
	else
		*proc  << "-v";

	config->setGroup("Recording Options");

	if(ejectAtEnd)
		*proc  << "-eject";

	*proc  << "speed=" + speed;

	if(actionType == "dummy"){
		emitStatus(i18n("Fixating (Test Mode)..."));
		*proc  << "-dummy";
	}
	//////////// fixate
	*proc  << "-fix";
	if(config->readBoolEntry("Burn Proof", false))
		*proc  << "driveropts=burnproof";

	*proc  << "dev=" + device;
}
//---------------------------------------------------------------------------------------------
bool CdboFixateProcAction::filterOutput(int &type, QString &message){
	QString buff;
	if(secCount < 9){
		//tere is always some garbich in front >{
		message = message.remove(0,message.find(QRegExp("[0-9]{1,1}")));
		if(actionType == "dummy")
			buff =  i18n("Last chance to quit, starting dummy fixate in %1").arg(message);
		else
			buff =  i18n("Last chance to quit, starting real fixate in %1").arg(message);
		
		type = Cdbo::OverwritePrevious;
		emitOutput(type, buff);
		secCount++;
		return false;
	}
	if(message.find("Last chance to quit") != -1 && secCount == 100){
		type = Cdbo::StatusOkMessage;
		secCount = 0;
	}

	if(message.find("Re-load disk and hit") != -1){
		requestReload();
		return false;
	}
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboFixateProcAction::processLaunched(){
	emitOutput(Cdbo::StatusOkMessage, i18n("Starting fixate CD process."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboFixateProcAction::processFailedToLaunch(){
	emitOutput(Cdbo::ErrorMessage,
		i18n("Fixate CD process failed to start. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboFixateProcAction::processSucceded(){
	emitOutput(Cdbo::StatusOkMessage, i18n("CD fixated successfully."));
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboFixateProcAction::processFailed(){
	emitOutput(Cdbo::StatusNotOkMessage, i18n("Failed to fixate this CD."));
	CdboAction::processFailed();
}
