 /***************************************************************************
                          CdboMkisofsAction.cpp  -  description
                             -------------------
    begin                : Thu Aug 15 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdir.h>
#include <qregexp.h>

#include <kconfig.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <krun.h>
#include <kio/job.h>
#include <kdebug.h>

#include "CdboMkisofsAction.h"
#include "CdboMkisofsAction.moc"
#include "CdboConstants.h"

#include "SessionTypeDialog.h"


CdboMkisofsAction::CdboMkisofsAction( QObject* d, const char* name) :
	CdboAction(d, name){

	//sessionType = Cdbo::InvalidSession;
	shownPerInfo = false;
	ignoreOut = false;
}

CdboMkisofsAction::~CdboMkisofsAction(){

}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::initProcess(KProcess *proc, CdboParamsDict* params){

//////////////////////////////////////////////////////////////////////////////

	if(!boolParamVal("IS_ON_THE_FLY_CREATE_DATA", creatingCdOnTheFly))
		return;
		
	sessionType = Cdbo::InvalidSession;
	if(!intParamVal("SESSION_TYPE", sessionType, false) ||
				(sessionType < Cdbo::FirstSession || sessionType > Cdbo::FinalSession))
		sessionType = Cdbo::InvalidSession;

	scsiDevice = paramVal("TARGET_DEVICE");
	if(scsiDevice.isEmpty())
		return;


	QString *sessionTypeHolder = dynamic_cast<QString *>((*params)["SESSION_TYPE"]);//take it as a string now

	QString isoImageName = paramVal("IMAGE_NAME");
	if(isoImageName.isEmpty()){
		internalError(i18n("Failed to get '%1' for: %2.").arg("IMAGE_NAME").arg(name()));
		return;
	}
	
	bool mappingsReady;
	if(!boolParamVal("IS_MAPPINGS_READY", mappingsReady))
		return;
		
	QString pathspecName = "", hidejoiName = "", hidehfsName = "", hiderrName = "";
	bool topLevel = false;
	QStringList fileList;
	if(!mappingsReady){
		QString fileListString = paramVal("FILE_LIST");
		if(fileListString.isEmpty())
			return;
		fileList = QStringList::split("\n", fileListString, false);
		if(!boolParamVal("IS_TOP_LEVEL", topLevel))
		 	return;
	}
	else{
		
		pathspecName = paramVal("PATH_SPEC_NAME");
		if(pathspecName.isEmpty())
			return;
		
		hidejoiName = paramVal("HIDE_JOI_NAME", false);
		hidehfsName = paramVal("HIDE_HFS_NAME", false);
		hiderrName = paramVal("HIDE_RR_NAME", false);
	}
///////////////////////////////////////////////////////////////////////////////

	config->setGroup("ISO Settings");
	if(config->readBoolEntry("Bootable", false)){
		if(!prepareBootFiles()){
			readyToLaunch = false;
			return;
		}
	}
	
	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to mkisofs", "mkisofs");
	KRun::shellQuote(exe);
	*proc << exe;

	////////////////////////////
	config->setGroup("Default Settings");
	if(config->readNumEntry("Level Of Output", Cdbo::StandartOutput) == Cdbo::CustomOutput){
		switch(config->readNumEntry("Custom Level Of Output", 1)){
			case 0: *proc  << "-quiet"; break;
			case 1: break;
			case 2: *proc  << "-v"; break;
			case 3: *proc  << "-v -v"; break;
		}
	}
	else
		*proc  << "-v";
		
	*proc  << "-gui";


	config->setGroup("ISO Settings");
	if(config->readEntry("Session Type", "Single") == "Multi-Session"  &&
		sessionType == Cdbo::InvalidSession){

		SessionTypeDialog *std = new SessionTypeDialog(&sessionType, 0, "std", true);
		std->exec();
		delete std;

		if(sessionTypeHolder != NULL)
			*sessionTypeHolder = QString::number(sessionType);

		switch(sessionType){
			case Cdbo::FirstSession:
				break;

			case Cdbo::MiddleSession:
			case Cdbo::FinalSession:{
				
				waitingForProcess = config->readBoolEntry("Check old sessions", false);
				if(waitingForProcess)
					checkLastSessions();

				while(waitingForProcess)
					qApp->processEvents(1);

				if(!readyToLaunch)
					return;

				waitingForProcess = true;
				fetchLastSession();

				while(waitingForProcess)
					qApp->processEvents(1);

				if(!readyToLaunch)
					return;
					
				addSessionOpt(proc);
				break;
			}
			default:{
				processCancelledByUser();
				return;
			}
		}
	}
	addStdOpt(proc);

	config->setGroup("ISO Settings");
	if(!creatingCdOnTheFly){//if ont-the-fly - pipe it
		KRun::shellQuote(isoImageName);
		*proc << "-o " + isoImageName;
	}
	
//BEGIN: FILE LIST ===========
	QString parsedFList = "";
	if(topLevel)
		parsedFList += " " + createTopLevelString(fileList);
	
	else if(mappingsReady){
		if(!hidejoiName.isEmpty()){
			KRun::shellQuote(hidejoiName);
			parsedFList += " -hide-joliet-list " + hidejoiName;
		}
		if(!hidehfsName.isEmpty()){
			KRun::shellQuote(hidehfsName);
			parsedFList += " -hide-hfs-list " + hidehfsName;
		}
		if(!hiderrName.isEmpty()){
			KRun::shellQuote(hiderrName);
			parsedFList += " -hide-list " + hiderrName;
		}
		
		KRun::shellQuote(pathspecName);
		parsedFList += " -graft-points -path-list " + pathspecName;
	}
	else
		parsedFList += " -graft-points " + createFileListString(fileList);
	
	
	if(config->readBoolEntry("Bootable", false)){
		QString boot = bootDir;
		KRun::shellQuote(boot);
		parsedFList += " " + boot;
	}
//END: FILE LIST ===========
	
//BEGIN: T-SIZE ===========
//FIXME:  add this option to Iso Image Settings Dialog
	waitingForProcess = creatingCdOnTheFly && config->readBoolEntry("Drive requires t-size", true);
	if(waitingForProcess)
		fetchTSize(parsedFList);

	while(waitingForProcess)
		qApp->processEvents(1);

	if(!readyToLaunch)
		return;
					
//END: T-SIZE ===========
	
	*proc << parsedFList;
	
	if(creatingCdOnTheFly)
		*proc << "|";
	
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::addStdOpt(KProcess *proc){
	config->setGroup("ISO Settings");
	int type = config->readNumEntry("ISO Type", Cdbo::LinWinISO);

	switch(type){
		case Cdbo::LinuxISO:{
			*proc << "-r -L -l -allow-multidot -allow-lowercase";
			break;
		}
		case Cdbo::LinWinISO:{
			*proc << "-r -J -D";
			break;
		}
		default:{
			addCustomOpt(proc);
			break;
		}
	}
	if(config->readBoolEntry("Bootable", false))
		addBootOpt(proc);

	if(config->readBoolEntry("Hybrid", false))
		addHybridOpt(proc);

	if(config->readBoolEntry("Multivolumed Set", false)){
		*proc << "-volset-size " + config->readEntry("Volume Set Size", "0");
		*proc << "-volset-seqno " + config->readEntry("Volume Set Sequence", "0");
	}

	if(config->readBoolEntry("Follow Sym Links", false))
		*proc << "-f";
	if(config->readBoolEntry("Generate Tbl", false))
		*proc << " -T";
	if(config->readBoolEntry("Omit Version", false))
		*proc << "-N";
	if(config->readBoolEntry("Hide RR", false))
		*proc << "-hide-rr-moved";

	/////////////// labels
	QString strBuff  = config->readEntry("Application ID");
	if(strBuff != ""){
		KRun::shellQuote(strBuff);
		*proc << "-A " + strBuff;
	}
	strBuff  = config->readEntry("Copyright");
	if(strBuff != ""){
		KRun::shellQuote(strBuff);
		*proc << "-copyright " + strBuff;
	}
	
	strBuff  = config->readEntry("Volume ID");
	if(strBuff != ""){
		KRun::shellQuote(strBuff);
		*proc << "-V " + strBuff;
	}
	
	strBuff  = config->readEntry("Volume Set Name");
	if(strBuff != ""){
		KRun::shellQuote(strBuff);
		*proc << "-volset " + strBuff;
	}
	
	strBuff  = config->readEntry("Publisher ID");
	if(strBuff != ""){
		KRun::shellQuote(strBuff);
		*proc << "-P " + strBuff;
	}
	
	strBuff  = config->readEntry("Preparer ID");
	if(strBuff != ""){
		KRun::shellQuote(strBuff);
		*proc << "-p " + strBuff;
	}
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::addCustomOpt(KProcess *proc){
	config->setGroup("ISO Settings");
	if(config->readBoolEntry("Rock Ridge", false))
		*proc  << "-r";
	if(config->readBoolEntry("Joliet", false))
		*proc  << "-J";
	// filenames
	if(config->readBoolEntry("Untrans File Names", false)){
		*proc  << "-U";
	}
	else if(config->readBoolEntry("Relaxed File Names", false)){
		*proc  << "-L -l -allow-multidot -allow-lowercase";
		if(!config->readBoolEntry("Do Not Omit Trailing Period", false))
			*proc  << "-d";
	}
	else{
		if(config->readBoolEntry("Begin Period File", false))
			*proc  << "-L";
		if(config->readBoolEntry("Long File Names", false))
			 *proc  << "-l";
		 if(config->readBoolEntry("Multidot File Names", false))
			*proc  << "-allow-multidot";
		if(config->readBoolEntry("Lower Case File Names", false))
			 *proc  << "-allow-lowercase";
		 if(!config->readBoolEntry("Do Not Omit Trailing Period", false))
			*proc  << "-d";
	}
	if(config->readBoolEntry("Deep Dir Relocation", false))
		*proc  << "-D";
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::addSessionOpt(KProcess *proc){
	*proc << "-C " + sessionInfo;
	*proc << "-M " + scsiDevice;

	config->setGroup("ISO Settings");
	if(config->readBoolEntry("Check old sessions names", false))
		*proc << "-check-oldnames ";
	if(config->readBoolEntry("Force Rock Ridge", false))
		*proc << "-force-rr -no-rr";
}
//==========================================
void CdboMkisofsAction::addBootOpt(KProcess *proc){
	config->setGroup("ISO Settings");
	*proc  << "-c Boot/boot.cat";
	*proc  << "-b Boot/cdrom.img";
	switch(config->readNumEntry("Boot Image Index", 0)){
		case 0: break;
		case 1: *proc  << "-hard-disk-boot"; break;
		case 2: *proc  << "-no-emul-boot"; break;
	}
}
//==========================================
void CdboMkisofsAction::addHybridOpt(KProcess *proc){
	config->setGroup("ISO Settings");
	if(config->readBoolEntry("Apple", false))
		*proc  << "-apple";

	if(config->readBoolEntry("HFS", false) && config->readBoolEntry("Probe", false))
		*proc  << "-hfs -probe";
}
//==========================================
QString CdboMkisofsAction::createTopLevelString(QStringList &flist){
	QString fnames = "", buff = "";
	for(int i = 0; i < (int)flist.count(); i++){
		KRun::shellQuote(flist[i]);
		fnames = fnames + flist[i] + " ";
	}
	return fnames;
}
//==========================================
QString CdboMkisofsAction::createFileListString(QStringList &flist){
	QString fnames = "", buff = "";
	QFileInfo *d;
	//qApp->processEvents();
	for(int i = 0; i < (int)flist.count(); i++){
		d = new QFileInfo( flist[i] );
		if ( !d->exists()){
			delete d;
 			continue;
 		}
 		if(d->isDir()){
			buff = "/" + d->dir().dirName() + "/";
			KRun::shellQuote(buff);
			fnames += " " + buff + "=";
			buff = flist[i] + "/";
			KRun::shellQuote(buff);
			fnames += buff;
 		}
		else{
			buff = "/" + d->fileName();
			KRun::shellQuote(buff);
			fnames += " " + buff + "=";
			buff = flist[i];
			KRun::shellQuote(buff);
			fnames += buff;
		}
		delete d;
	}
	return fnames;
}
//---------------------------------------------------------------------------------------------
bool CdboMkisofsAction::filterOutput(int &type, QString &message){

	if(message.startsWith("Using") && !ignoreOut){
		type = Cdbo::ProcessMessage;
		message = i18n("Performing file system scan...");
		ignoreOut = true;
		return true;
	}
	if(message.find("done, estimate finish") !=-1){
		if(ignoreOut){
			emitOutput(Cdbo::ProcessMessage, i18n("Done."));
			ignoreOut = false;
		}
		QString buff = message.left(message.find("."));
		message = (message.right(message.length() - message.findRev(","))).stripWhiteSpace();
		emitPercent("CdboMkisofsAction", buff.toInt(), message);

		if(!shownPerInfo){
			emitPercentInfo("CdboMkisofsAction", i18n("Creating ISO image, percent done:") );
			shownPerInfo = true;
		}
		return false;
	}

	if(message.startsWith("Writing:") || message.startsWith("Done with:"))
		return false;
		
	if(message.find("mkisofs:") !=-1 || message.find("cdrecord:") !=-1)
		ignoreOut = false;
		
	if(ignoreOut)
		return false;
	//if(message.startsWith("Excluded by match:") || message.startsWith("Using") || message.startsWith("Scanning"))
	//	return;

	if(message.find("Total translation") !=-1){
		emitPercent("CdboMkisofsAction", 100);
		return false;
	}
	return true;
}
//---------------------------------------------------------------------------------------------
bool CdboMkisofsAction::prepareBootFiles(){
	config->setGroup("ISO Settings");
	QString bootCatalog = config->readEntry("Boot Catalog", "");
	QString bootImage = config->readEntry("Boot Image", "");
	if(bootCatalog == ""){
		emitOutput(Cdbo::ErrorMessage,
			i18n("You did not specify the 'Boot Catalog' location, which is required to make a bootable CD."));
		emitOutput(Cdbo::ErrorMessage,
			i18n("If this is not an attempt to make a bootable CD, please unselect this option."));
		return false;
	}
	if( bootImage == ""){
		emitOutput(Cdbo::ErrorMessage,
			i18n("You did not specify the 'Boot Image' location, which is required to make a bootable CD."));
		emitOutput(Cdbo::ErrorMessage,
			i18n("If this is not an attempt to make a bootable CD, please unselect this option."));
		return false;
	}


	config->setGroup("Default Settings");
	bootDir = config->readEntry("Tmp Dir", "/tmp/")  + "cdbo_boot_tmp";
	waitingForProcess = true;
	curJob = KIO::mkdir(KURL(bootDir));
	connect( curJob, SIGNAL(result( KIO::Job * )), this, SLOT( curJobDone( KIO::Job *)) );
	while(waitingForProcess)
		qApp->processEvents(1);


	//bootDir = bootDir  + "/Boot";
	waitingForProcess = true;
	curJob = KIO::mkdir(KURL(bootDir + "/Boot"));
	connect( curJob, SIGNAL(result( KIO::Job * )), this, SLOT( curJobDone( KIO::Job *)) );
	while(waitingForProcess)
		qApp->processEvents(1);

	waitingForProcess = true;
	curJob = KIO::copy( KURL(bootCatalog), "file:" + bootDir + "/Boot/boot.cat", false );
	connect( curJob, SIGNAL(result( KIO::Job * )), this, SLOT( curJobDone( KIO::Job *)) );
	while(waitingForProcess)
		qApp->processEvents(1);

	waitingForProcess = true;
	curJob = KIO::copy( KURL(bootImage), "file:" + bootDir + "/Boot/cdrom.img", false );
	connect( curJob, SIGNAL(result( KIO::Job * )), this, SLOT( curJobDone( KIO::Job *)) );
	while(waitingForProcess)
		qApp->processEvents(1);

	return true;
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::removeBootFiles(){
	curJob = KIO::rmdir(KURL(bootDir));
	connect( curJob, SIGNAL(result( KIO::Job * )), this, SLOT( curJobDone( KIO::Job *)) );
	removeOneFile(bootDir + "/Boot/cdrom.img");
	removeOneFile(bootDir + "/Boot/boot.cat");
	removeOneDir(bootDir + "/Boot");
	removeOneDir(bootDir);
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::checkLastSessions(){

	gotAnswer = false;
	emitStatus(i18n("Checking previous sessions..."));

	KShellProcess *cdboProc = new KShellProcess();

	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to mkisofs", "mkisofs");
	KRun::shellQuote(exe);
	*cdboProc << exe;
	*cdboProc << "-check-session " + scsiDevice;

	debugProcess(cdboProc);
	QApplication::connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(curProcDoneSlot(KProcess *)));
 	QApplication::connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedCheckLastSessionsMessageSlot(KProcess *, char *, int)));
	QApplication::connect(cdboProc, SIGNAL(receivedStderr (KProcess *, char *, int)),
		this, SLOT(receivedCheckLastSessionsMessageSlot(KProcess *, char *, int)));

	if(!cdboProc->start(KProcess::NotifyOnExit, KProcess::AllOutput)){
		emitOutput(Cdbo::ErrorMessage, i18n("'Check previous sessions' process failed to start. "\
				"Make sure you have enough privileges."));
		delete cdboProc;
		processFailed();
	}
	else
		emitOutput(Cdbo::InfoMessage, i18n("Checking previous sessions, please wait."));
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::receivedCheckLastSessionsMessageSlot(KProcess *, char *buffer, int buflen){
	if(gotAnswer)
		return;
	if(buflen > 0){
		QString buff(buffer);

		if(buff.contains("Permission denied")) {
			emitOutput(Cdbo::ErrorMessage, i18n("Failed to fetch last session info. "\
				"Make sure you have enough privileges."));

			processFailed();
			return;
		}
		if(buff.contains("Usage:")) {
			emitOutput(Cdbo::ErrorMessage, i18n("The combination of options could not be interpreted."));
			emitOutput(Cdbo::ErrorMessage,
				i18n("Please submit this as a bug to the email address located in the Help->About dialog."));
			processFailed();
			return;
		}
		if(buff.contains("error")) {
			emitOutput(Cdbo::ErrorMessage, i18n("This CD does not seem to contain a valid session."));
			processFailed();
			return;
		}
		if(buff.contains("NO Rock Ridge present")) {
			emitOutput(Cdbo::StatusNotOkMessage,
						i18n("Neither Rock Ridge nor TRANS.TBL name translations were found on previous sessions."));
			int result = KMessageBox::warningYesNo(0,
					i18n("Neither Rock Ridge nor TRANS.TBL name translations were found on previous sessions. "\
						"Would you like to continue anyway?"),
        				i18n("No Rock Ridge present"),
        				i18n("&Yes"),
        				i18n("&Cancel"));

			switch(result){
				case KMessageBox::Yes:{
					waitingForProcess = false;
					return;
				}
       			case KMessageBox::No:{
					processCancelledByUser();
					return;
				}
			}
		}
		else{
			gotAnswer = true;
			waitingForProcess = false;
			emitOutput(Cdbo::StatusOkMessage, i18n("Previous sessions seem to be valid."));
		}
 	}
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::fetchLastSession(){

	emitStatus(i18n("Fetching session info..."));
	KShellProcess *cdboProc = new KShellProcess();

	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdrecord", "cdrecord");
	KRun::shellQuote(exe);
	*cdboProc << exe;

	*cdboProc << "-msinfo";
	*cdboProc <<  "dev=" + scsiDevice;

	debugProcess(cdboProc);

	QApplication::connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(curProcDoneSlot(KProcess *)));
 	QApplication::connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedFetchLastSessionMessageSlot(KProcess *, char *, int)));
	QApplication::connect(cdboProc, SIGNAL(receivedStderr (KProcess *, char *, int)),
		this, SLOT(receivedFetchLastSessionMessageSlot(KProcess *, char *, int)));

	if(!cdboProc->start(KProcess::NotifyOnExit, (KProcess::Communication)(KProcess::Stdin|KProcess::Stdout|KProcess::Stderr))){
		emitOutput(Cdbo::ErrorMessage, i18n("'Fetch last session info' process failed to start. "\
				"Make sure you have enough privileges."));
		delete cdboProc;
		processFailed();
	}
	else
		emitOutput(Cdbo::InfoMessage, i18n("Fetching last session info, please wait."));
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::receivedFetchLastSessionMessageSlot(KProcess *proc, char *buffer, int buflen){
	if(buflen > 0){

		QString buff(buffer);
		buff = buff.left(buflen);

		if(buff.contains("cdrecord: Drive needs to reload the media to return to proper status.")){
			emitOutput(Cdbo::ProcessMessage,
				i18n("Drive needs to reload the media to return to proper status."));
			return;
		}
		if(buff.contains("Permission denied")) {
			proc->kill();
			emitOutput(Cdbo::ErrorMessage,i18n( "Failed to fetch last session info! "\
				"Make sure you have enough privileges."));
			processFailed();
			return;
		}
		if(buff.find("Re-load disk and hit") != -1){
			requestReload(false);
			
			if(proc->writeStdin("\n", 1)){
				kdDebug() << name() << "Signal to cdrecord went through" << endl;
				return;
			}
			proc->kill();
			internalError(i18n( "Failed to communicate with the process."));
			return;
		}
		if(buff.contains("Usage:")) {
			proc->kill();
			internalError(i18n("The combination of options could not be interpreted."));
			return;
		}

		if(buff.contains("error")) {
			proc->kill();
			emitOutput(Cdbo::ErrorMessage, i18n("This CD does not seem to contain a valid session."));
			buff = buff.replace(QRegExp("\n"), " ");
			emitOutput(Cdbo::ProcessMessage, buff);
			emitOutput(Cdbo::ErrorMessage, i18n("Failed to fetch last session info."));
			processFailed();
			return;
		}

		sessionInfo = buff.replace(QRegExp("\n"), "");
		waitingForProcess = false;
	}
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::fetchTSize(const QString &parsedFList){

	gotAnswer = false;
	emitStatus(i18n("Calculating image size..."));

	KShellProcess *cdboProc = new KShellProcess();

	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to mkisofs", "mkisofs");
	KRun::shellQuote(exe);
	*cdboProc << exe;
	*cdboProc << "-q -R -J -print-size " << parsedFList;

	debugProcess(cdboProc);
	QApplication::connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(curProcDoneSlot(KProcess *)));
 	QApplication::connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedFetchTSizeMessageSlot(KProcess *, char *, int)));
	QApplication::connect(cdboProc, SIGNAL(receivedStderr (KProcess *, char *, int)),
		this, SLOT(receivedFetchTSizeMessageSlot(KProcess *, char *, int)));

	if(!cdboProc->start(KProcess::NotifyOnExit, KProcess::AllOutput)){
		emitOutput(Cdbo::ErrorMessage, i18n("'Calculate image size' process failed to start. "\
				"Make sure you have enough privileges."));
		delete cdboProc;
		processFailed();
	}
	else
		emitOutput(Cdbo::InfoMessage, i18n("Calculating image size, please wait."));
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::receivedFetchTSizeMessageSlot(KProcess *, char *buffer, int buflen){
	if(gotAnswer)
		return;
	if(buflen > 0){
		QString buff(buffer);

		if(buff.contains("Permission denied") || buff.contains("No such file or directory") || buff.contains("error")) {
			emitOutput(Cdbo::ErrorMessage, i18n("Failed to calculate image size. "\
				"Make sure you have enough privileges."));

			processFailed();
			return;
		} 
		if(buff.contains("Usage:")) {
			emitOutput(Cdbo::ErrorMessage, i18n("The combination of options could not be interpreted."));
			emitOutput(Cdbo::ErrorMessage,
				i18n("Please submit this as a bug to the email address located in the Help->About dialog."));
			processFailed();
			return;
		}
		buff = buff.stripWhiteSpace();
		
		QRegExp ex("^[0-9]+$");
		if(ex.exactMatch(buff)){
			tsize = buff;
			setParamVal("ON-THE-FLY_TSIZE", &tsize);
			gotAnswer = true;
			waitingForProcess = false;
		}
		
		
 	}
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::curJobDone( KIO::Job *){
	waitingForProcess = false;
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::curProcDoneSlot(KProcess *proc){
	//waitingForProcess = false;
	delete proc;
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::processLaunched(){
	emitOutput(Cdbo::StatusOkMessage, i18n("Starting ISO creation process."));
	emitStatus(i18n("Creating ISO Image..."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::processFailedToLaunch(){
	config->setGroup("ISO Settings");
	if(config->readBoolEntry("Bootable", false))
		removeBootFiles();
	emitOutput(Cdbo::ErrorMessage,
		i18n("ISO creation process failed to start. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::processSucceded(){
	config->setGroup("ISO Settings");
	if(config->readBoolEntry("Bootable", false))
		removeBootFiles();

	if(!creatingCdOnTheFly)
		emitOutput(Cdbo::StatusOkMessage,i18n("ISO image created successfully."));

	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::processFailed(){
	config->setGroup("ISO Settings");
	if(config->readBoolEntry("Bootable", false))
		removeBootFiles();
	emitOutput(Cdbo::StatusNotOkMessage,i18n("Failed to create ISO image."));

	CdboAction::processFailed();
}
//---------------------------------------------------------------------------------------------
void CdboMkisofsAction::resetSettings(){
	gotAnswer = true;
	CdboAction::resetSettings();
	waitingForProcess = false;
	sessionType = Cdbo::InvalidSession;
}




