/***************************************************************************
                          CdboOgg123Action.cpp  -  description
                             -------------------
    begin                : Fri Aug 23 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdir.h>
#include <qregexp.h>

#include <kapplication.h>
#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <krun.h>
#include <kdebug.h>

#include "CdboOgg123Action.h"
#include "CdboOgg123Action.moc"

#include "CdboConstants.h"

CdboOgg123Action::CdboOgg123Action( QObject* d, const char* name) :
	CdboAction(d, name){

		title = "";
}
//---------------------------------------------------------------------------------------------
CdboOgg123Action::~CdboOgg123Action(){

}
//---------------------------------------------------------------------------------------------
void CdboOgg123Action::initProcess(KProcess *proc, CdboParamsDict* /*params*/){

/////////////////////////////////////////////////////////////////////////////////////////////
	QStringList inFiles;
	if(!listParamVal("OGG_FILES_TO_CONVERT", inFiles))
		return;
	QStringList outFiles;
	if(!listParamVal("OGG_CONVERTED_NAMES", outFiles))
		return;
	processesDone = 1;
	intParamVal("OGG_START_COUNTER", processesDone, false);

	totalToProcess = paramVal("TOTAL_TO_CONVERT");
	if(totalToProcess.isEmpty())
		return;
/////////////////////////////////////////////////////////////////////////////////////////////

	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to ogg123", "ogg123");
	KRun::shellQuote(exe);
	QString baseCommand = exe + " ";
	
	config->setGroup("Default Settings");
	if(config->readNumEntry("Level Of Output", Cdbo::StandartOutput) == Cdbo::CustomOutput){
		switch(config->readNumEntry("Custom Level Of Output", 1)){
			case 0: baseCommand +=  " -quiet "; break;
			case 1: break;
			case 2: baseCommand +=  " -v "; break;
			case 3: baseCommand +=  " -v "; break;
		}
	}
	else
		baseCommand += " -v ";
	
	QString command = "";
	for(uint i = 0; i < inFiles.count(); i++){
		KRun::shellQuote(inFiles[i]);
		KRun::shellQuote(outFiles[i]);
		command += baseCommand + "-d wav -f " + outFiles[i] + " " + inFiles[i] + "; ";
	}

	*proc << command;
}
//---------------------------------------------------------------------------------------------
bool CdboOgg123Action::filterOutput(int &type, QString &message){
	if(message.find("Title") !=-1){
		if(message.find("Title:") !=-1){
			message = message.replace(QRegExp("Title:"), "").stripWhiteSpace();
		}
		else{
			message = message.right(message.length() - message.find(":") - 1);
			message = message.left(message.find("Artist")).stripWhiteSpace();
		}
		title = message;
		return false;
	}
	
	int pos = 0;
	if((pos = message.find("Time:")) !=-1){
		message = message.right(message.length() - pos - 5);
		message = message.left(message.find("]")).stripWhiteSpace();
		QString buff = message.left(message.find(".")).stripWhiteSpace();
		message = message.replace(QRegExp(buff), "").stripWhiteSpace();
		message = message.right(message.length() - message.find("[") - 1);
		message = message.left(message.find(".")).stripWhiteSpace();

		float done = buff.left(buff.find(":")).toFloat() * 60 + buff.right(buff.length() - buff.find(":") -1).toFloat();

		float left = message.left(message.find(":")).toFloat() * 60 + message.right(message.length() - message.find(":") -1).toFloat();

		int i = (int)( done / ((done + left) / 100));
		if(i == 1){
			emitPercentInfo("CdboOgg123Action" + QString::number(processesDone), 
				i18n("Decompressing '%1' (%2 of %3), percent done:").arg(title.isEmpty()? i18n("Unknown"): title).
					arg(QString::number(processesDone)).arg(totalToProcess));
		}
	
		emitPercent("CdboOgg123Action" + QString::number(processesDone), i);
		return false;
	}
	
	if((message.find("Decoding of") !=-1 && message.find("finished") !=-1) || message.find("Done") !=-1){
		emitPercent("CdboOgg123Action" + QString::number(processesDone), 100);
		processesDone++;
		title = "";
		return false;
	}
	if(message.length() < 3 || message.find("Output Buffer") !=-1)
		return false;
	
	type = Cdbo::InfoMessage;
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboOgg123Action::processLaunched(){
	if(processesDone == 1)
		emitOutput(Cdbo::StatusOkMessage, i18n("Starting to decompress MP3/Ogg Vorbis file(s)."));
	emitStatus(i18n("Decompressing Audio..."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboOgg123Action::processFailedToLaunch(){
	emitOutput(Cdbo::ErrorMessage,
		i18n("Ogg Vorbis decompression failed to start. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboOgg123Action::processSucceded(){
	emitOutput(Cdbo::StatusOkMessage,
		i18n("Ogg Vorbis file(s) decompressed successfully."));
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboOgg123Action::processFailed(){
	emitOutput(Cdbo::StatusNotOkMessage,
		i18n("Failed to decompress Ogg Vorbis files."));
	CdboAction::processFailed();
}
