/***************************************************************************
                          CdboUtilActions.cpp  -  description
                             -------------------
    begin                : Sat Aug 17 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdir.h>

#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <kdirlister.h>
#include <kurl.h>
#include <krun.h>
#include <kdebug.h>
#include <kdirsize.h>

#include "CdboUtilActions.h"
#include "CdboUtilActions.moc"

#include "CdboConstants.h"

 
CdboCheckDriveAction::CdboCheckDriveAction( QObject* d, const char* name) :
	CdboAction(d, name){
	unlockUser = false;
}
//---------------------------------------------------------------------------------------------
CdboCheckDriveAction::~CdboCheckDriveAction(){
}
//---------------------------------------------------------------------------------------------
void CdboCheckDriveAction::initProcess(KProcess *proc, CdboParamsDict* /*params*/){

////////////////////////////////////////////////////////////////////////////////////////////

	QString device = paramVal("TARGET_DEVICE");
	if(device.isEmpty())
		return;

/////////////////////////////////////////////////////////////////////////////////////////////
	
	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdrecord", "cdrecord");
	KRun::shellQuote(exe);
	*proc << exe;

	*proc  << "-v";
	*proc  << "-checkdrive";
	*proc  << "driveropts=help";

	*proc  << "dev=" + device;
}
//---------------------------------------------------------------------------------------------
void CdboCheckDriveAction::processLaunched(){
	emitOutput(Cdbo::StatusOkMessage, i18n("Checking recording device."));
	emitStatus(i18n("Checking device..."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboCheckDriveAction::processFailedToLaunch(){
	emitOutput(Cdbo::ErrorMessage,
		i18n("Device check failed. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboCheckDriveAction::processSucceded(){
	emitOutput(Cdbo::StatusOkMessage,
		i18n("Driver for the current drive is present and valid."));
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboCheckDriveAction::processFailed(){
	emitOutput(Cdbo::StatusNotOkMessage,
		i18n("Not a valid recording device."));
	CdboAction::processFailed();
}

/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
CdboCheckTmpAction::CdboCheckTmpAction( QObject* d, const char* name) :
	CdboAction(d, name){
		unlockUser = false;
}
//-----------------------------------------------
CdboCheckTmpAction::~CdboCheckTmpAction(){
}
//==============================
void CdboCheckTmpAction::initProcess(KProcess *proc, CdboParamsDict* /*params*/){

////////////////////////////////////////////////////////////////////////////////////////////

	tempDir = paramVal("TMP_DIRECTORY");
	if(tempDir.isEmpty())
		return;
	QDir dir(tempDir);
	if(!dir.exists()){
		processFailedToLaunch();
		return;
	}
	
	if(!intParamVal("IMAGE_SIZE", minTmpSize))
		return;

/////////////////////////////////////////////////////////////////////////////////////////////
	emitStatus(i18n("Checking temporary directory size..."));
	
	QString tmp = tempDir;
	KRun::shellQuote(tmp);
	*proc << "df" << "-m" << tmp;
}
//---------------------------------------------------------------------------------------------
void CdboCheckTmpAction::receivedMessageSlot(KProcess *, char *buffer, int buflen){
	if(buflen > 0){
		QString buff(buffer);
		buff = buff.left(buflen);

		QStringList comOutput(QStringList::split("\n",buff));
		comOutput = QStringList::split(" ", comOutput[1]);
		buff = comOutput[3].simplifyWhiteSpace();

		if(buff.toInt() > minTmpSize)
			hasSpase = true;
		else
			hasSpase = false;

		config->setGroup("Default Settings");
		config->writeEntry("tmp Size", buff.toInt());
		emitOutput(Cdbo::InfoMessage,
			i18n("Directory '%1' has %2 MB available.").arg(tempDir).arg(buff));
	}
}
//---------------------------------------------------------------------------------------------
void CdboCheckTmpAction::receivedErrMessageSlot(KProcess *proc, char *buffer, int buflen){
	this->receivedMessageSlot(proc, buffer, buflen);
}
//---------------------------------------------------------------------------------------------
void CdboCheckTmpAction::processFailedToLaunch(){
	emitOutput(Cdbo::ErrorMessage,
		i18n("You must choose a valid temporary directory."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboCheckTmpAction::processSucceded(){
	if(!hasSpase){
		emitOutput(Cdbo::ErrorMessage,
			i18n("The chosen directory does not have enough space available " \
				"for storing the disk image."));
		emitOutput(Cdbo::ErrorMessage,
			i18n("Please choose another location."));
		CdboAction::processFailed();
	}
	else
		CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboCheckTmpAction::processFailed(){
	this->processSucceded();
}



/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////


CdboScanAudioTracksAction::CdboScanAudioTracksAction( QObject* d, const char* name) :
	CdboJobBasedAction(d, name){
}
//-----------------------------------------------
CdboScanAudioTracksAction::~CdboScanAudioTracksAction(){

}
//==============================
void CdboScanAudioTracksAction::launchJob(CdboParamsDict* params){

	QString device = paramVal("SOURCE_DEVICE");
	if(device.isEmpty())
		return;
		
	tracks = dynamic_cast<QString *>((*params)["TRACKS_LIST"]);
	if(tracks == NULL){
		internalError(i18n("CdboScanAudioTracksAction failed to get 'tracks' for process."));
		return;
	}
	QString *type = dynamic_cast<QString *>((*params)["ACTION_TYPE"]);
	if(type == NULL){
		type = new QString("wav");
	}

	trackList.clear();
	config->setGroup("Audio Read Options");

	dirLister = new KDirLister();
	KLocale slave("kio_audiocd");

	emitStatus(i18n("Scanning audio tracks..."));
	getLength = false;
	if(*type == "ogg")
		*type = "Ogg Vorbis";
	else if(*type == "mp3")
		*type = "MP3";
	else{
		getLength = true;
		*type ="By Name";
	}
	connect( dirLister, SIGNAL( completed() ), SLOT( scanAudioDiskDoneSlot() ) );
	connect( dirLister, SIGNAL( canceled() ), SLOT( processCancelledByUser() ) );
	connect( dirLister, SIGNAL( newItems( const KFileItemList& ) ), SLOT( receivedAudioListSlot( const KFileItemList& ) ) );
	
	if(config->readBoolEntry("Perform CDDB Lookup", true))
		dirLister->openURL( "audiocd:/" + slave.translate( type->latin1() ) + "?device=" + device, false, true );
	else
		dirLister->openURL( "audiocd:/?device=" + device, false, true );
	enableTimeout();
}
//=========================================================
void CdboScanAudioTracksAction::receivedAudioListSlot( const KFileItemList& fileList ){

	// 1) track Number
	// 2) track Name
	// 3) track Size
	// 4) track URL
	// 5) track Length - only for wav & cda
	QString trackInfo;
	int count;
	QString name, size, number;
	bool ok = true;
	KFileItemListIterator it( fileList );
	
	if(config->readBoolEntry("Perform CDDB Lookup", true)){
		for( ; it.current(); ++it ){
			size = KIO::convertSize(it.current()->size());

			name =  it.current()->text();
			name = name.left( name.length() - 4 );

			count = name.left( 2 ).toInt( &ok );
			if(!ok){
				count = name.right( 2 ).toInt( &ok );
				name = "<Unknown>";
			}
			else
				name = name.mid(3, name.length() );

			if(count <= 9)
				number = " 0" + QString::number(count);
			else
				number = " " + QString::number(count);

			trackInfo = number + ";;;" + name + ";;;" + size + ";;;" + it.current()->url().prettyURL();
			if(getLength){
				size = ";;;" + timeString(it.current()->size() / 176400);
				trackInfo = trackInfo + size;
			}
			trackList.append(trackInfo);
		}
	}
	else{
		for(; it.current(); ++it){
			if ( ! it.current()->isDir() ){
				count = it.current()->text().mid( 5, 2 ).toInt( 0 );
				size = KIO::convertSize(it.current()->size());
			
				name = "<Unknown>";

				if(count <= 9)
					number = " 0" + QString::number(count);
				else
					number = " " + QString::number(count);
				

				trackInfo = number + ";;;" + name + ";;;" + size + ";;;" + it.current()->url().prettyURL();
				if(getLength){
					size = ";;;" + timeString(it.current()->size() / 176400);
					trackInfo = trackInfo + size;
				}
				trackList.append(trackInfo);
			}
		}
	}
}
//======================================================================
void CdboScanAudioTracksAction::killCurrentTask(){
	dirLister->stop();
	showCanceledMess = false;
}
//======================================================================
void CdboScanAudioTracksAction::scanAudioDiskDoneSlot(){
	*tracks = trackList.join("\n");
	processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboScanAudioTracksAction::processFailedToLaunch(){
	CdboAction::processFailedToLaunch();
	killCurrentTask();

	emitOutput(Cdbo::StatusNotOkMessage, i18n("Operation timed out."));
	emitOutput(Cdbo::ErrorMessage, i18n("No responce recieved from audiocd slave."));
	emitOutput(Cdbo::InfoMessage, i18n("Try to use another device."));
}
//---------------------------------------------------------------------------------------------
void CdboScanAudioTracksAction::resetSettings(){
	resetTimeout();
	CdboAction::resetSettings();
}
//---------------------------------------------------------------------------------------------
QString CdboScanAudioTracksAction::cancelMessage(){
	return i18n("This will abort track scan.");
}



/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////


CdboCdSizeAction::CdboCdSizeAction( QObject* d, const char* name) :
	CdboJobBasedAction(d, name){
}
//-----------------------------------------------
CdboCdSizeAction::~CdboCdSizeAction(){

}
//==============================
void CdboCdSizeAction::launchJob(CdboParamsDict* params){

	/////////////////////////////////////////////////////////////////////////////////////////////

	QString device = paramVal("SOURCE_DEVICE_NAME");
	if(device.isEmpty())
		return;
	
	min = dynamic_cast<QString *>((*params)["IMAGE_SIZE"]);
	if(min  == NULL){
		internalError(i18n("CdboCheckTmpAction failed to get 'minTmpSize' for process."));
		return;
	}
	
	emitOutput(Cdbo::StatusOkMessage,
			i18n("Culculating size of CD. Please wait...."));

	config->setGroup("Wizard Detected");
	QString mountPoint = config->readEntry("Mount point for " + device, QString::null);

	if(mountPoint.endsWith("*supermount")){
		mountPoint = mountPoint.section("*", 0, 0);
	}
	else if(mountPoint != KIO::findDeviceMountPoint(device)){//means not mounted
		
		curJob = KIO::mount(true, 0, device, mountPoint, false);
		curJob->setName(mountPoint.utf8());
		connect(curJob, SIGNAL(result ( KIO::Job *)),
			this, SLOT(gotCdSize(KIO::Job *) ));
		return;

	}
	int detSize = (int)(KDirSize::dirSize(KURL(mountPoint)) / 1024 / 1024);

	emitOutput(Cdbo::InfoMessage,
			i18n("Size of this CD: %1MB.").arg(QString::number(detSize)));

	*min = QString::number(detSize);
	processSucceded();
}
///=========================================================
void CdboCdSizeAction::gotCdSize( KIO::Job * j){

	if(j->error()){
		emitOutput(Cdbo::ErrorMessage,
			i18n("Failed to get size of this CD."));
		*min = "0";
		processSucceded();
		return;
	}

	int detSize = (int)(KDirSize::dirSize(KURL(j->name())) / 1024 / 1024);

	emitOutput(Cdbo::InfoMessage,
			i18n("Size of this CD: %1MB.").arg(QString::number(detSize)));

	*min = QString::number(detSize);
	KIO::unmount(j->name(), false);
	processSucceded();
}


