/***************************************************************************
                          CdboCopyCDBase.cpp  -  description
                             -------------------
    begin                : Sat May 5 2002
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include "CdboCopyCDBase.h"
#include "CdboCopyCDBase.moc"

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qheader.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtabbar.h>



#include "CdboDriveInfoPnl.h"
#include "CdboSpeedPnl.h"
#include "CdboDialogCntPnl.h"
#include "CdboCddaView.h"

#include <ktabctl.h>

//
CdboCopyCDBase::CdboCopyCDBase( QWidget* parent,  const char* name, bool modal, WFlags fl )
	 : CdboDialogBase( parent, name, modal, fl )
{
	if ( !name )
		setName( "CdboCopyCDBase" );
	setCaption( i18n( "Copy CD" ) );

	QGridLayout *topLayout = new QGridLayout(this);
	//topLayout->setMargin(KDialog::marginHint());
	// topLayout->setSpacing(KDialog::spacingHint());
	topLayout->setSpacing( 0 );
	topLayout->setMargin( 0 );
	topLayout->setResizeMode(QLayout::Minimum);
	
//BEGIN cntpnl ================
	dialogCntPnl = new CdboDialogCntPnl( this, "dialogCntPnl" );
	topLayout->addMultiCellWidget(dialogCntPnl, 1, 1, 0, 1);
//END cntpnl ================

//BEGIN drives ================
	baseFrm = new QWidget( this, "baseFrm" );
	baseFrmLayout = new QVBoxLayout( baseFrm ); 
	baseFrmLayout->setSpacing( 0 );
	baseFrmLayout->setMargin( 8 );

	srcDevicePnl = new CdboDriveInfoPnl( baseFrm, "srcDevicePnl" );
	baseFrmLayout->addWidget( srcDevicePnl );

	targetDevicePnl = new CdboDriveInfoPnl( baseFrm, "targetDevicePnl" );
	baseFrmLayout->addWidget( targetDevicePnl );

	topLayout->addWidget(baseFrm, 0, 0);
//END drives ================


	detailsTab = new CdboCopyCDBaseTabs( this, "detailsTab" );
	
//BEGIN details ================
	detailsFrm = new QWidget( detailsTab, "detailsFrm" );
	detailsFrmLayout = new QVBoxLayout( detailsFrm ); 
	detailsFrmLayout->setSpacing( 2 );
	detailsFrmLayout->setMargin( 5 );
	detailsFrmLayout->setResizeMode(QLayout::Minimum);

//BEGIN read ================
	readDetailsPnl = new QGroupBox( detailsFrm, "readDetailsPnl" );
    readDetailsPnl->setTitle( i18n( "General" ) );
	readDetailsPnl->setColumnLayout(0, Qt::Vertical );
	readDetailsPnl->layout()->setSpacing( 6 );
	readDetailsPnl->layout()->setMargin( 11 );
	readDetailsPnlLayout = new QHBoxLayout( readDetailsPnl->layout() );
	readDetailsPnlLayout->setAlignment( Qt::AlignTop );

	typeLbl = new QLabel( readDetailsPnl, "typeLbl" );
	typeLbl->setText( i18n( "&Type of CD:" ) );
	readDetailsPnlLayout->addWidget( typeLbl );

	typeCmb = new QComboBox( FALSE, readDetailsPnl, "typeCmb" );
	typeCmb->insertItem( i18n( "Audio" ) );
	typeCmb->insertItem( i18n( "Common" ) );
	typeCmb->insertItem( i18n( "Data Only" ) );
	QToolTip::add( typeCmb, i18n( "Select type of CD. <b></b>Use 'What's This' to get description about each available Mode.") );
	QString whatsThis = i18n("Choose type of Source CD.");
	
	whatsThis += i18n("<br><br><b>Audio:</b> Source is regular Audio or Mixed-Mode CD with audio tracks.");
	whatsThis += i18n("<br><br><b>Common:</b> Source is one of the most common none audio CD types "\
		"such as regular Data, VCD, Mixed-Mode, etc. This CD will be copied in Disk-At-Once mode.");
	whatsThis += i18n("<br><br><b>Data Only:</b> Source is regular Data CD. "\
		"Image of this CD will be copied. <b>Warning:</b> CDs which require Disk-At-Once mode like VCDs will not be copied correctly if Data Only is chosen.");
	whatsThis += i18n("<br><br><b>Please Note:</b> To be able to copy some CD types Source device must be recognized as SCSI device.");
	QWhatsThis::add(typeCmb, whatsThis);
	
	
	readDetailsPnlLayout->addWidget( typeCmb );
	QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	readDetailsPnlLayout->addItem( spacer );

	removeTmpChk = new QCheckBox( readDetailsPnl, "removeTmpChk" );
	removeTmpChk->setText( i18n( "C&lean up after recording" ) );
	QToolTip::add(  removeTmpChk, i18n( "Remove image files, tracks, etc. used during copy process.<b></b>"
		" If not removed, those files can be found in working directory and reused at later time." ) );
	readDetailsPnlLayout->addWidget( removeTmpChk );
	
	detailsFrmLayout->addWidget( readDetailsPnl);

//END read ================

//BEGIN record ================
	recordDetailesPnl = new QGroupBox( detailsFrm, "recordDetailesPnl" );
	recordDetailesPnl->setEnabled( TRUE );
	recordDetailesPnl->setTitle( i18n( "Recording Details" ) );
	QWhatsThis::add(  recordDetailesPnl, i18n( "Recording Details." ) );
	recordDetailesPnl->setColumnLayout(0, Qt::Vertical );
	recordDetailesPnl->layout()->setSpacing( 0 );
	recordDetailesPnl->layout()->setMargin( 0 );

	recordDetailesPnlLayout = new QHBoxLayout( recordDetailesPnl->layout() );
	recordDetailesPnlLayout->setAlignment( Qt::AlignTop );
	recordDetailesPnlLayout->setSpacing( 4 );
	recordDetailesPnlLayout->setMargin( KDialog::marginHint() );

//BEGIN action ================
	actionCmb = new QComboBox( FALSE, recordDetailesPnl, "actionCmb" );

	QToolTip::add( actionCmb, i18n( "Select the action to take. <b></b>Use 'What's This' to get description about each available Mode.") );
	recordDetailesPnlLayout->addWidget( actionCmb );
//END action ================
	QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	recordDetailesPnlLayout->addItem( spacer_2 );

	numOfCopiesLbl = new QLabel( recordDetailesPnl, "numOfCopiesLbl" );
	numOfCopiesLbl->setText( i18n( "&Number of copies: " ) );
	numOfCopiesLbl->setAlignment( int( QLabel::WordBreak | QLabel::AlignCenter ) );
	numOfCopiesLbl->setIndent( -1 );
	recordDetailesPnlLayout->addWidget( numOfCopiesLbl );

	numberSpn = new QSpinBox( recordDetailesPnl, "numberSpn" );
	numberSpn->setMinValue( 1 );
	numberSpn->setValue( 1 );
	QToolTip::add(  numberSpn, i18n( "Specify number of copies to make" ) );
	recordDetailesPnlLayout->addWidget( numberSpn );
	//QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	//recordDetailesPnlLayout->addItem( spacer_3 );

	detailsFrmLayout->addWidget( recordDetailesPnl);
//END record ================


//BEGIN speed ================
	speedPnl = new CdboSpeedPnl( detailsFrm, "speedPnl" );
	detailsFrmLayout->addWidget( speedPnl);
//END speed ================

	detailsTab->addTab( detailsFrm, i18n( "P&rocess Details" ) );
//END details ================
	
//BEGIN audio ================
	audioTb = new QWidget( detailsTab, "audioTb" );
	audioTbLayout = new QVBoxLayout( audioTb );
	audioTbLayout->setSpacing( 4 );
	audioTbLayout->setMargin( 4 );

	trackView = new CdboCddaView( audioTb, "trackViewInCopy" );
	audioTbLayout->addWidget( trackView );
	detailsTab->addTab( audioTb, i18n( "Audio Tracks Selectio&n" ) );
//END audio ================
	
	topLayout->addWidget(detailsTab, 0, 1);

	topLayout->setColStretch(0, 4);
	topLayout->setColStretch(1, 6);
	
	numOfCopiesLbl->setBuddy( numberSpn );
	typeLbl->setBuddy( typeCmb );

}

/*  
 *  Destroys the object and frees any allocated resources
 */
CdboCopyCDBase::~CdboCopyCDBase()
{
    // no need to delete child widgets, Qt does it all for us
}

//================================================
CdboCopyCDBaseTabs::CdboCopyCDBaseTabs( QWidget* parent, const char* name):KTabCtl(parent, name){
	setBorder(false);
	setShape(QTabBar::TriangularAbove);
}
