/***************************************************************************
                          CdboDataCDBase.cpp  -  description
                             -------------------
    begin                : Sun May 13 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <klocale.h>
#include "CdboDataCDBase.h"
#include "CdboDataCDBase.moc"

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qhbox.h>


#include "CdboDriveInfoPnl.h"
#include "CdboSpeedPnl.h"
#include "CdboDialogCntPnl.h"

//
CdboDataCDBase::CdboDataCDBase( QWidget* parent, const char* name, bool modal, WFlags fl )
    : CdboDialogBase( parent, name, modal, fl )
{
    if ( !name )
	setName( "CdboDataCDBase" );

	setCaption( i18n( "Create New CD - CD Bake Oven" ) );
	cdboDataCDBaseLayout = new QGridLayout( this ); 
	cdboDataCDBaseLayout->setSpacing( 0 );
	cdboDataCDBaseLayout->setMargin( 0 );
	cdboDataCDBaseLayout->setResizeMode(QLayout::Minimum);

	baseFrm = new QFrame( this, "baseFrm" );

	baseFrm->setMargin( 0 );
	baseFrm->setMidLineWidth( 0 );
	baseFrmLayout = new QVBoxLayout( baseFrm ); 
	baseFrmLayout->setSpacing( 6 );
	baseFrmLayout->setMargin( 11 );

	targetDevicePnl = new CdboDriveInfoPnl( baseFrm, "targetDevicePnl" );
	baseFrmLayout->addWidget( targetDevicePnl );

	speedPnl = new CdboSpeedPnl( baseFrm, "speedPnl" );
	baseFrmLayout->addWidget( speedPnl );

	cdboDataCDBaseLayout->addWidget( baseFrm, 0, 0 );



	dialogCntPnl = new CdboDialogCntPnl( this, "dialogCntPnl" );

	cdboDataCDBaseLayout->addMultiCellWidget( dialogCntPnl, 1, 1, 0, 1 );
	
	detailsPnl = new QFrame( this, "detailsPnl" );
	detailsPnlLayout = new QVBoxLayout( detailsPnl );
	detailsPnlLayout->setSpacing( 6 );
	detailsPnlLayout->setMargin( 11 );

//BEGIN: recordDetailesPnl <><><><><><>

	recordDetailesPnl = new QGroupBox( detailsPnl, "recordDetailesPnl" );
	recordDetailesPnl->setTitle( i18n( "Recording Details" ) );
	QWhatsThis::add(  recordDetailesPnl, i18n( "Recording Details." ) );

	recordDetailesPnl->setColumnLayout(0, Qt::Vertical );
	recordDetailesPnl->layout()->setSpacing( 0 );
	recordDetailesPnl->layout()->setMargin( 0 );

	recordDetailesPnlLayout = new QHBoxLayout( recordDetailesPnl->layout() );
	recordDetailesPnlLayout->setAlignment( Qt::AlignTop );
	recordDetailesPnlLayout->setMargin( KDialog::marginHint() );

	actionCmb = new QComboBox( FALSE, recordDetailesPnl, "actionCmb" );
	QToolTip::add( actionCmb, i18n( "Select the action to take. <b></b>Use 'What's This' to get description about each available Mode.") );
	recordDetailesPnlLayout->addWidget( actionCmb );

	QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	recordDetailesPnlLayout->addItem( spacer_3 );


	numOfCopiesLbl = new QLabel( recordDetailesPnl, "numOfCopiesLbl" );
	numOfCopiesLbl->setText( i18n( "&Number of copies: " ) );
	numOfCopiesLbl->setAlignment( int( QLabel::WordBreak | QLabel::AlignCenter ) );
	numOfCopiesLbl->setIndent( -1 );
	recordDetailesPnlLayout->addWidget( numOfCopiesLbl );


	numberSpn = new QSpinBox( recordDetailesPnl, "numberSpn" );
	numberSpn->setMinValue( 1 );
	numberSpn->setValue( 1 );
	QToolTip::add(  numberSpn, i18n( "Specify number of copies to make" ) );
	recordDetailesPnlLayout->addWidget( numberSpn );


	detailsPnlLayout->addWidget(recordDetailesPnl, 4);

//END: recordDetailesPnl <><><><><><>

//BEGIN: imagePnl <><><><><><>
	imagePnl = new QGroupBox( detailsPnl, "imagePnl" );
	imagePnl->setTitle( i18n( "ISO Image (if used)" ) );
	QWhatsThis::add(  imagePnl, i18n( "ISO image." ) );

	imagePnl->setColumnLayout(0, Qt::Vertical );
	imagePnl->layout()->setSpacing( 0 );
	imagePnl->layout()->setMargin( 0 );
	imagePnlLayout = new QGridLayout( imagePnl->layout() );
	imagePnlLayout->setAlignment( Qt::AlignTop );
	imagePnlLayout->setSpacing( 6 );
	imagePnlLayout->setMargin( 11 );


	QHBox *locationHBox = new QHBox(imagePnl);
	locationHBox->setSpacing(KDialog::spacingHint());

	imageLbl = new CdboSqueezedLabel( locationHBox, "imageLbl" );
	imageLbl->setFrameShape( QLabel::Box );
	imageLbl->setFrameShadow( QLabel::Raised );
	imageLbl->setText( QString::null );

	imageLbl->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred));

	imageBrowseBtn = new QPushButton( locationHBox, "imageBrowseBtn" );
	imageBrowseBtn->setText( i18n( "..." ) );
	QToolTip::add(  imageBrowseBtn, i18n( "Browse for an existing ISO image which will be used to create new CD.<b></b>" ) );
	
	
	imagePnlLayout->addMultiCellWidget( locationHBox, 0, 0, 0, 2 );


	removeIsoChk = new QCheckBox( imagePnl, "removeIsoChk" );
	removeIsoChk->setText( i18n( "Delete &image(s) after recording" ) );
	QToolTip::add(  removeIsoChk, i18n( "When finished recording, remove image file(s)." ) );
	imagePnlLayout->addWidget( removeIsoChk, 1, 0);

	QSpacerItem* spacer_7 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	imagePnlLayout->addItem( spacer_7, 1, 1);

	isoPreviewBtn = new QPushButton( imagePnl, "isoPreviewBtn" );
	isoPreviewBtn->setText( i18n( "&Preview" ) );
	isoPreviewBtn->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, isoPreviewBtn->sizePolicy().hasHeightForWidth() ) );
	QToolTip::add(  isoPreviewBtn, i18n( "Preview ISO image" ) );
	imagePnlLayout->addWidget( isoPreviewBtn, 1, 2);


	imagePnlLayout->setColStretch(0, 1);

	detailsPnlLayout->addWidget(imagePnl, 6);
//END: imagePnl <><><><><><>

	cdboDataCDBaseLayout->addWidget( detailsPnl, 0, 1 );
	cdboDataCDBaseLayout->setColStretch(0, 4);
	cdboDataCDBaseLayout->setColStretch(1, 6);

	cdboDataCDBaseLayout->setRowStretch(1, 1);

	numOfCopiesLbl->setBuddy( numberSpn );
}
CdboDataCDBase::~CdboDataCDBase(){

}

CdboSqueezedLabel::CdboSqueezedLabel( QWidget *parent, const char *name ) :
	KSqueezedTextLabel ( parent, name ) {
}
//=======================
void CdboSqueezedLabel::setTooltip(const QString &newTooltip){
	prevTooltip = newTooltip;
	QToolTip::remove(this);
	QToolTip::add(this, prevTooltip );
}
//=======================
void CdboSqueezedLabel::resizeEvent( QResizeEvent * e){ 
	KSqueezedTextLabel::resizeEvent(e);
	QToolTip::remove(this);
	QToolTip::add(this, prevTooltip );
}

