/***************************************************************************
                          CdboDialogFactory.cpp  -  description
                             -------------------
    begin                : Fri Aug 23 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtimer.h>

#include <kcmdlineargs.h>
#include <kapplication.h>
#include <klocale.h>
#include <kdebug.h>
#include <kdialog.h>

#include "CdboDialogFactory.h"
#include "CdboDialogFactory.moc"


#include "CdboDialogBase.h"

#include "CdboCopyCDDialog.h"
#include "CdboRipAudioDialog.h"
#include "CdboDataCDDialog.h"
#include "CdboEraseCDDialog.h"
#include "CdboAudioCDDialog.h"

#include "CdboVersion.h"


K_EXPORT_COMPONENT_FACTORY( libcdbodialogs, CdboDialogFactory)

KInstance *CdboDialogFactory::mInstance = 0;
KAboutData *CdboDialogFactory::mAboutData = 0;

CdboDialogFactory::CdboDialogFactory(QObject* parent, const char* name):
	KLibFactory (parent, name){
}
//==============================
CdboDialogFactory::~CdboDialogFactory(){
	delete mAboutData;
	mAboutData = 0;
	delete mInstance;
	mInstance = 0;
}
//======================================
QObject*  CdboDialogFactory::createObject(QObject* parent, const char* name,
		const char* classname, const QStringList &args) {

	if(!classname){
		kdError() << "Missing classname for dialog" << endl;
		return 0;
	}

	QWidget * parentWidget = 0;
	if ( parent && parent->inherits ( "QWidget" ) ){
		parentWidget = static_cast<QWidget* >(parent);
	}
		
	if(!parentWidget)
		kdDebug() << "No parentWidget" << endl;

		
		
	return createDialog(classname, parentWidget, name, args);
}
//==============================
KDialog* CdboDialogFactory::createDialog(const char* classname, QWidget* parentWidget,
						const char* name, const QStringList &/*args*/){

	KDialog *dialog = dialogDict[classname];
	if(dialog){//FIXME: handle this in a better way
		return dialog;
	}
	
	if(QString(classname) == "CdboCopyCDDialog")
		dialog = new CdboCopyCDDialog(parentWidget, name);

	if(QString(classname) == "CdboFixateCDDialog")
		dialog = new CdboFixateCDDialog(parentWidget, name);

	if(QString(classname) == "CdboEraseCDDialog")
		dialog = new CdboEraseCDDialog(parentWidget, name);

	if(QString(classname) == "CdboRipAudioDialog")
		dialog = new CdboRipAudioDialog(parentWidget, name);

	if(QString(classname) == "CdboDataCDDialog")
		dialog = new CdboDataCDDialog(parentWidget, name);
	
	if(QString(classname) == "CdboAudioCDDialog")
		dialog = new CdboAudioCDDialog(parentWidget, name);

	if(dialog){
		dialogDict.insert(classname, dialog);
		connect(dialog, SIGNAL(dialogDone(KDialog*)),
			this, SLOT(dialogDone(KDialog*)));
	}
	return dialog;
}
//==================================
void CdboDialogFactory::dialogDone(KDialog* dialog){
	kdDebug() << "CdboDialogFactory::removing: " << dialog->className() << endl;
	dialogDict.remove(dialog->className());
}
//=================================
void CdboDialogFactory::loadArgs(KCmdLineArgs *args){
	this->args = args;
	QTimer::singleShot(1, this, SLOT(processArgs()));
}
//=================================
void CdboDialogFactory::processArgs(){
	QString dialogType;
	if (args->isSet("dialog-type")){
		dialogType = QString::fromLocal8Bit(args->getOption("dialog-type"));
		kdDebug() << "Will try to create '" << dialogType.latin1() << "'" << endl;
	}
	else{
		kdError() << "cdbodialogloader: missing dialog-type argument" << endl;
		kdError() << "Try `cdbodialogloader --help' for more information." << endl;
		kapp->exit(-1);
		return;
	}
	
	
	KDialog* dialog = createDialog(dialogType.latin1(), 0, dialogType.latin1());
	if(dialog){
	
		if(processCdboDialogArgs(dialog)){
			processCdboDataCDDialogArgs(dialog);
			if(!processCdboAudioCDDialogArgs(dialog)){
				delete dialog;
				return;
			}
		}
	
		dialog->exec();
		kdDebug() << "cdbodialogloader:: DONE." << endl;
		kapp->exit(0);
		return;
	}
	else{
		kdError() << "cdbodialogloader: Invalid dialog type." << endl;
		kapp->exit(-1);
		return;
	}
}
//=================================
bool CdboDialogFactory::processCdboDialogArgs(KDialog* dialog){
	if(!dialog->inherits("CdboDialogBase"))
		return false;
		
	CdboDialogBase* cdboDialog = static_cast<CdboDialogBase*>(dialog);
	
	if (args->isSet("default-device")){
		QString defaultDevice = QString::fromLocal8Bit(args->getOption("default-device"));
		kdDebug() << "Default Device: '" << defaultDevice << "'" << endl;
		cdboDialog->setCurrentTarget(defaultDevice);
	}
		
	if (args->isSet("debug-output")){
		kdDebug() << "debug-output is enabled" << endl;
		cdboDialog->debugOutput();
	}
		
	if (args->isSet("debug-commands")){
		kdDebug() << "debug-commands is enabled" << endl;
		cdboDialog->debugCommands();
	}
	return true;
}
//=================================
void CdboDialogFactory::processCdboDataCDDialogArgs(KDialog* dialog){
	if(!dialog->isA("CdboDataCDDialog"))
		return;

	CdboDataCDDialog* ncdDialog = static_cast<CdboDataCDDialog*>(dialog);

		
	if (args->isSet("iso-image-only")){
		kdDebug() << "iso-image-only is enabled" << endl;
		ncdDialog->setImageOnly(true);
	}

	//the first found of the following params is used
	if (args->isSet("source-dirs")){
		QString sourceDirs = QString::fromLocal8Bit(args->getOption("source-dirs"));
		kdDebug() << "Source Dirs: " << sourceDirs << endl;
		ncdDialog->setSourceDirs(sourceDirs);
		return;
	}
		
	if(args->isSet("iso-image")){
		QString isoImage = QString::fromLocal8Bit(args->getOption("iso-image"));
		kdDebug() << "ISO Image: " << isoImage << endl;
		ncdDialog->setISOImage(isoImage);
		return;
	}
	
	if(args->isSet("from-url")){
		QString url = QString::fromLocal8Bit(args->getOption("from-url"));
		kdDebug() << "'From URL: " << url << endl;
		ncdDialog->setURL(url);
		return;
	}
	
	if (args->isSet("pathspec-list")){
		QString file = QString::fromLocal8Bit(args->getOption("pathspec-list"));
		kdDebug() << "pathspec-list: " << file << endl;
		ncdDialog->setPathspecName(file);

		if (args->isSet("hide-joliet-list")){
			file = QString::fromLocal8Bit(args->getOption("hide-joliet-list"));
			kdDebug() << "hide-joliet-list: " << file << endl;
			ncdDialog->setHidejoiName(file);
		}
		if (args->isSet("hide-hfs-list")){
			file = QString::fromLocal8Bit(args->getOption("hide-hfs-list"));
			kdDebug() << "hide-hfs-list: " << file << endl;
			ncdDialog->setHidehfsName(file);
		}
		if (args->isSet("hide-rr-list")){
			file = QString::fromLocal8Bit(args->getOption("hide-rr-list"));
			kdDebug() << "hide-rr-list: " << file << endl;
			ncdDialog->setHiderrName(file);
		}
		return;
	}
}
//=================================
bool CdboDialogFactory::processCdboAudioCDDialogArgs(KDialog* dialog){
	if(!dialog->isA("CdboAudioCDDialog"))
		return true;

	CdboAudioCDDialog* ncdDialog = static_cast<CdboAudioCDDialog*>(dialog);
	
	//the first found of the following params is used
	if (args->isSet("audio-list")){
		QString sourceList = QString::fromLocal8Bit(args->getOption("audio-list"));
		kdDebug() << "Audio List: " << sourceList << endl;
		ncdDialog->setTrackList(sourceList);
		return true;
	}
		
	if (args->isSet("toc-file")){
		QString tocFile = QString::fromLocal8Bit(args->getOption("toc-file"));
		kdDebug() << "Toc File: " << tocFile << endl;
		ncdDialog->setTocFile(tocFile);
		return true;
	}
	
	kdError() << "cdbodialogloader: missing argument. audio-list or toc-file must be provided to use CdboAudioCDDialog." << endl;
	kdError() << "Try `cdbodialogloader --help' for more information." << endl;
	kapp->exit(-1);
	return false;
}
//======================================
KInstance *CdboDialogFactory::instance()
{
	if(!mInstance)
		mInstance = new KInstance(aboutData());
	return mInstance;
}
//======================================
const KAboutData *CdboDialogFactory::aboutData()
{
	if(!mAboutData)
	{
		mAboutData = new KAboutData( "cdbakeoven", I18N_NOOP("CD Bake Oven"),
			CDBO_VERSION, I18N_NOOP("Thank you for using CD Bake Oven - CD Burning facility for KDE"),
				KAboutData::License_GPL, "(c) 2001, Alexy Podolsky",
			I18N_NOOP("\nThis is free software with ABSOLUTELY NO WARRANTY."
		          "\nUse at your own risk! See --license for more details."),
		          "http://cdbakeoven.sourceforge.net");
		          mAboutData->addAuthor("Alexy Podolsky",
		          I18N_NOOP("Author and maintainer"),
		          "alexpod@users.sourceforge.net");
		    mAboutData->addAuthor("Ryan Cumming",
		     I18N_NOOP("UI tweaks and code cleanup"),
		          "ryan@completely.kicks-ass.org");
	}
	return mAboutData;
}



