/***************************************************************************
                          CdboEraseCDDialog.cpp  -  description
                             -------------------
    begin                : Sat May 5 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/
/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qcheckbox.h>

#include <kconfig.h>
#include <kapplication.h>

#include "CdboEraseCDDialog.h"
#include "CdboEraseCDDialog.moc"
#include "CdboPreviewDialog.h"
#include "CdboDriveInfoPnl.h"
#include "CdboSpeedPnl.h"


CdboEraseCDDialog::CdboEraseCDDialog(QWidget *parent, const char *name ) :
	CdboEraseCDBase(true, parent,name),
	DCOPObject( "CdboDialogBase" ){
	eraseMode = true;
	init();
}
//=============================================
CdboEraseCDDialog::CdboEraseCDDialog(bool em, QWidget *parent, const char *name) :
	CdboEraseCDBase(em,parent,name),
	DCOPObject( "CdboDialogBase" ){
	eraseMode = em;
	if(eraseMode)
		init();
}
//=============================================
CdboFixateCDDialog::CdboFixateCDDialog(QWidget *parent, const char *name ) :
	CdboEraseCDDialog(false, parent, name),
	DCOPObject( "CdboDialogBase" ){
	eraseMode = false;
	init();
}
//=======================================
void CdboEraseCDDialog::init(){
	CdboDialogBase::init();
	
	///////////////////////////////////////////////////
	targetDevicePnl->setType(CdboDriveInfoPnl::Target);
	targetDevicePnl->setUsrBtnTitle(i18n("&Preview"));
	/////////////////////////////////////////////////
	if(eraseMode)
		dialogCntPnl->setStartBtnTitle(i18n("&Erase"));
	else
		dialogCntPnl->setStartBtnTitle(i18n("&Fixate"));
	/////////////////////////////////////////////////
	readOptions();
	targetDevicePnl->closeTray();

	// connecting to the Drive Info Panel
	connect( targetDevicePnl->getDetectBtn(), SIGNAL( clicked() ), this, SLOT( detectSlot() ) );
	connect( targetDevicePnl->getUsrBtn(), SIGNAL( clicked() ), this, SLOT( previewSlot() ) );
	// connecting to the Dialog Control Panel
	connect( dialogCntPnl->getStartBtn(), SIGNAL( clicked() ), this, SLOT( startBtnSlot() ) );
	connect( dialogCntPnl->getCancelBtn(), SIGNAL( clicked() ), this, SLOT( cancelSlot() ) );

	connect( dialogCntPnl->getSettingsBtn(), SIGNAL( clicked() ), this, SLOT( settingsSlot() ) );

	connect( dialogCntPnl->getQuitBtn(), SIGNAL( clicked() ), this, SLOT( quitSlot() ) );
	
	settingTypes = "cdboburnopt cdbodefaultopt";
}
//---------------------------------------------------------------------------------------------
CdboEraseCDDialog::~CdboEraseCDDialog(){
}
//---------------------------------------------------------------------------------------------
void CdboEraseCDDialog::saveOptions(){
	if(!config)
		return;
	if(eraseMode){
		config->setGroup("Erase Options");
		config->writeEntry("Force Blank", forceChk->isChecked());
		config->writeEntry("Type of Blank", blankTypeCmb->currentItem());
	}
	else{
		config->setGroup("Fixate options");
		config->writeEntry("Type of Fixate", blankTypeCmb->currentItem());
		config->writeEntry("Eject At End", forceChk->isChecked());
	}
	speedPnl->saveOptions();
	targetDevicePnl->saveOptions();
	config->sync();
}
//---------------------------------------------------------------------------------------------
void CdboEraseCDDialog::readOptions(){
	saveOptions();
	CdboDialogBase::readOptions();
	
	if(eraseMode){
		 config->setGroup("Erase Options");
		 forceChk->setChecked(config->readBoolEntry("Force Blank", true));
		 int iBuff = config->readNumEntry("Type of Blank", 0);
		 blankTypeCmb->setCurrentItem(iBuff);
	}
	else{
		config->setGroup("Fixate options");
		forceChk->setChecked(config->readBoolEntry("Eject At End", true));
		int iBuff = config->readNumEntry("Type of Fixate", 0);
		blankTypeCmb->setCurrentItem(iBuff);
	}
	speedPnl->readOptions();
	targetDevicePnl->readOptions();
	initBtns();
}
//---------------------------------------------------------------------------------------------
void CdboEraseCDDialog::initBtns(){
	if(targetDevicePnl->isReady()){
		blockBtns(false);
		dialogCntPnl->status(i18n("Ready"));
	}
	else{
		blockBtns(true);
		dialogCntPnl->getCancelBtn()->setEnabled(false);
		dialogCntPnl->status(i18n("No recording device."));
	}
}
//---------------------------------------------------------------------------------------------
void CdboEraseCDDialog::blockBtns(bool yes) {
	dialogCntPnl->getStartBtn()->setEnabled(!yes);
	dialogCntPnl->getCancelBtn()->setEnabled(yes);
	detailsFrm->setEnabled(!yes);
	targetDevicePnl->blockBtns(yes);
	
	if(doingSomething()){
		targetDevicePnl->getDetectBtn()->setEnabled(false);
		dialogCntPnl->getSettingsBtn()->setEnabled(false);
	}
	else{
		targetDevicePnl->getDetectBtn()->setEnabled(true);
		dialogCntPnl->getSettingsBtn()->setEnabled(true);
	}
}
//---------------------------------------------------------------------------------------------
QString* CdboEraseCDDialog::actionType(){
	int pos = blankTypeCmb->currentItem();
	if(eraseMode){
		switch(pos){
			case 0: mActionType = "fast"; break;
			case 1: mActionType =  "all"; break;
			case 2: mActionType =  "track"; break;
			case 3: mActionType =  "unreserve"; break;
			case 4: mActionType =  "trtail"; break;
			case 5: mActionType =  "unclose"; break;
			case 6: mActionType =  "session"; break;
		}
	}
	else{
		switch(pos){
			case 0: mActionType =  "burn"; break;
			case 1: mActionType =  "dummy"; break;
		}
	}
	return &mActionType;
}
//---------------------------------------------------------------------------------------------
void CdboEraseCDDialog::startBtnSlot(){
	clearOutput();
	QString type = "CdboEraseAction";
	if(!eraseMode){
		type = "CdboFixateAction";
		if(forceChk->isChecked())//this is realy eject :(
			forceCD = "TRUE";
		else
			forceCD = "FALSE";
	}
	else{
		if(forceChk->isChecked())
			forceCD = "TRUE";
		else
			forceCD = "FALSE";
		
	}
	
	if(!createAction(type))
		return;
	
	targetDevicePnl->closeTray();

	addActionParam("ACTION_TYPE", actionType());
	addActionParam("SPEED", speedPnl->speed());
	addActionParam("TARGET_DEVICE",  targetDevicePnl->scsiDevice());
	
	if(!eraseMode)
		addActionParam("EJECT_AFTER_FIXATE", &forceCD);
	else
		addActionParam("FORCE_BLANK", &forceCD);

	//debugCommands();
	launchCurrAction();
}

//------------------------------------------------------------------------------------------
void CdboEraseCDDialog::previewSlot(){
	if(!eraseMode){
		KApplication::startServiceByDesktopName("kscd", QString(""));
		return;
	}
	CdboPreviewDialog *pd = new CdboPreviewDialog((QWidget *)parent(), "pd");
	pd->open(targetDevicePnl->getDriveName());
}
