/***************************************************************************
                          CdboRipAudioDialog.cpp  -  description
                             -------------------
    begin                : Mon Feb 4 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtooltip.h>
#include <qdir.h>

#include <kurlrequester.h>
#include <kconfig.h>

#include "CdboRipAudioDialog.h"
#include "CdboRipAudioDialog.moc"
#include "CdboCddaView.h"
#include "CdboDriveInfoPnl.h"

#include "CdboConstants.h"

//---------------------------------------------------------------------------------------------
CdboRipAudioDialog::CdboRipAudioDialog(QWidget *parent, const char *name ) :
	CdboRipAudioBase(parent,name,true),
	DCOPObject( "CdboDialogBase" ){
	init();
}
//=======================================
void CdboRipAudioDialog::init(){
	CdboDialogBase::init();
	scaned = false;
	/////////////////////////////////////////////////
	srcDevicePnl->setType(CdboDriveInfoPnl::Source);
	srcDevicePnl->setUsrBtnTitle(i18n("Scan &Tracks"));
	////////////////////////////////////////////////
	dialogCntPnl->setStartBtnTitle(i18n("&Rip"));
	QToolTip::add(dialogCntPnl->getStartBtn(), i18n( "Start ripping" ) );

	readOptions();

	trackView->setHeaderClickEnabled(true);
	//trackView->setRenameable();
	trackView->showLongTitle();
	trackView->setShowSize();

	// connecting to the Src Drive Info Panel
	connect( srcDevicePnl->getDetectBtn(), SIGNAL( clicked() ), this, SLOT( detectSlot() ) );
	connect( srcDevicePnl->getUsrBtn(), SIGNAL( clicked() ), this, SLOT( scanSlot() ) );
	connect( srcDevicePnl, SIGNAL( trayEjected() ), trackView, SLOT( invalidate() ) );
	connect( srcDevicePnl, SIGNAL( selectedChaged() ), trackView, SLOT( invalidate() ) );

	// connecting to the Dialog Control Panel
	connect( dialogCntPnl->getStartBtn(), SIGNAL( clicked() ), this, SLOT( startBtnSlot() ) );
	connect( dialogCntPnl->getCancelBtn(), SIGNAL( clicked() ), this, SLOT( cancelSlot() ) );
	connect( dialogCntPnl->getSettingsBtn(), SIGNAL( clicked() ), this, SLOT( settingsSlot() ) );
	connect( dialogCntPnl->getQuitBtn(), SIGNAL( clicked() ), this, SLOT( quitSlot() ) );
	
	
	connect( fileFormatCmb, SIGNAL( activated(int) ), this, SLOT( typeChanged(int) ) );
	
	settingTypes = "audiocd cdbodefaultopt";

}
//---------------------------------------------------------------------------------------------
CdboRipAudioDialog::~CdboRipAudioDialog(){
}
//---------------------------------------------------------------------------------------------
void CdboRipAudioDialog::readOptions(){
	saveOptions();
	CdboDialogBase::readOptions();

	config->setGroup("Rip Options");

	QString tempDir = config->readEntry("Output Dir","");
	QDir dir(tempDir);
	if(tempDir.isEmpty() || !dir.exists())
		tempDir = QDir::homeDirPath();
	outputDirUr->setURL(tempDir);

	fileFormatCmb->setCurrentItem(config->readNumEntry("Type of Output", 0));

	srcDevicePnl->readOptions();
	initBtns();
}
//---------------------------------------------------------------------------------------------
void CdboRipAudioDialog::saveOptions(){
	if(!config)
		return;
	//////////////////////////////////////////////////////////
	config->setGroup("Rip Options");
	config->writeEntry("Type of Output", fileFormatCmb->currentItem());
	config->writeEntry("Output Dir",outputDirUr->url());

 	srcDevicePnl->saveOptions();
 	config->sync();
}
//---------------------------------------------------------------------------------------------
void CdboRipAudioDialog::initBtns(){
	if(srcDevicePnl->isReady()){
		srcDevicePnl->closeTray();
		blockBtns(false);
		dialogCntPnl->status(i18n("Ready"));
	}
	else{
		blockBtns(true);
		dialogCntPnl->getCancelBtn()->setEnabled(false);
		dialogCntPnl->status(i18n("No reading device configured."));
	}
}
//---------------------------------------------------------------------------------------------
void CdboRipAudioDialog::blockBtns(bool yes){
	dialogCntPnl->getStartBtn()->setEnabled(!yes);
	dialogCntPnl->getCancelBtn()->setEnabled(yes);
	trackView->setEnabled(!yes);
	detailsPnl->setEnabled(!yes);
	srcDevicePnl->blockBtns(yes);
	if(doingSomething()){
		srcDevicePnl->getDetectBtn()->setEnabled(false);
		dialogCntPnl->getSettingsBtn()->setEnabled(false);
	}
	else{
		srcDevicePnl->getDetectBtn()->setEnabled(true);
		dialogCntPnl->getSettingsBtn()->setEnabled(true);
	}
}
//---------------------------------------------------------------------------------------------
QString* CdboRipAudioDialog::actionType(){
	if(fileFormatCmb->currentItem()  == 0)
		mActionType = "ogg";
	else if(fileFormatCmb->currentItem()  == 1)
		mActionType = "mp3";
	else if(fileFormatCmb->currentItem()  == 2)
		mActionType = "wav";
	else
		mActionType = "cda";

	return &mActionType;
}

//---------------------------------------------------------------------------------------------
void CdboRipAudioDialog::startBtnSlot(){
	trackView->stop();
	clearOutput();

	QStringList pickedTracksList;
	if(!trackView->getPickedTracks(&pickedTracksList)){
		dialogCntPnl->output(Cdbo::StatusNotOkMessage,
			i18n("You must scan the device."));
		dialogCntPnl->output(Cdbo::StatusNotOkMessage,
			i18n("Pick the individual tracks you want to rip (read)."));
		return;
	}
	if(pickedTracksList.count() == 0){
		dialogCntPnl->output(Cdbo::ErrorMessage, i18n("No tracks chosen. There is nothing to rip."));
		return;
	}

	tempDir = outputDirUr->url();
	QDir dir(tempDir);
	if(tempDir.isEmpty() || !dir.exists()){
		dialogCntPnl->output(Cdbo::ErrorMessage, i18n("Invalid path for output directory."));
		return;
	}

	if(!createAction("CdboRipAction"))
		return;

	srcDevicePnl->closeTray();

	addActionParam("ACTION_TYPE", actionType());
	addActionParam("SOURCE_DEVICE",  srcDevicePnl->driveName());

	pickedTracks = pickedTracksList.join("\n");
	addActionParam("PICKED_TRACKS", &pickedTracks);
	addActionParam("TMP_DIRECTORY", &tempDir);

	launchCurrAction();

}
//------------------------------------------------------------------------------------------
void CdboRipAudioDialog::typeChanged(int){
	if(scaned)
		scanSlot();
}
//------------------------------------------------------------------------------------------
void CdboRipAudioDialog::scanSlot(){
	clearOutput();
	trackView->invalidate();
	scaned = true;
	if(!createAction("CdboScanAudioTracksAction")){
		return;
	}
	srcDevicePnl->closeTray();

	addActionParam("SOURCE_DEVICE",  srcDevicePnl->driveName());
	addActionParam("TRACKS_LIST", &pickedTracks);
	addActionParam("ACTION_TYPE", actionType());

	launchCurrAction();

}
//---------------------------------------------------------------------------------------------
void CdboRipAudioDialog::taskDone(bool success){
	if(success){
		if(isCurrAction("CdboScanAudioTracksAction"))
			trackView->load(pickedTracks);
	}
	CdboDialogBase::taskDone(success);
}
