/***************************************************************************
                          CdboAudioOpt.cpp  -  description
                             -------------------
    begin                : Sat May 5 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qspinbox.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcombobox.h>
#include <qlineedit.h>

#include <kapplication.h>
#include <kmessagebox.h>
#include <ksimpleconfig.h>
#include <kaboutdata.h>
#include <kcmodule.h>
#include <klocale.h>
#include <kglobal.h>
#include <dcopclient.h>

#include "CdboAudioOpt.h"
#include "CdboAudioOpt.moc"

CdboAudioOpt::CdboAudioOpt( QWidget* parent,  const char* name, WFlags  )
    		: CdboAudioOptBase( parent, name ){

	config = new KConfig("cdbakeovenrc");
    	load();
}
//--------------------------------------------------------------------------------
CdboAudioOpt::~CdboAudioOpt(){
}
//---------------------------------------------------------------------------------------------
void CdboAudioOpt::load(){
	config->setGroup("Audio Read Options");
	//cdrdao
	correctionModeSpn->setValue(config->readNumEntry("Correction Mode", 3));
	readRawChk->setChecked(config->readBoolEntry("Read Raw Toc", false));
	fastTocChk->setChecked(config->readBoolEntry("Fast Toc Lookup", true));
	outTypeCmb->setCurrentItem(config->readNumEntry("CD Type", 0));

	//cdda2wav
	echoChk->setChecked(config->readBoolEntry("Echo", false));
	paranoiaModeChk->setChecked(config->readBoolEntry("Paranoia Mode", false));
	transferSizeSpn->setValue(config->readNumEntry("Transfer Size", -1));
	cdromSpeedSpn->setValue(config->readNumEntry("CD-Rom Speed", -1));
	outTypeCmb->setCurrentItem(config->readNumEntry("CD Type", 0));

	//general
	cdromIndSpn->setCurrentItem(config->readNumEntry("CD-Rom Indianess", 0));
	outputIndSpn->setCurrentItem(config->readNumEntry("Output Indianess", 0));
	cddbLookupChk->setChecked(config->readBoolEntry("Perform CDDB Lookup", true));

}
//-----------------------------------------------------------------------------------------------
void CdboAudioOpt::save(){
	config->setGroup("Audio Read Options");
	//cdrdao
	config->writeEntry("Correction Mode", correctionModeSpn->value());
	config->writeEntry("Read Raw Toc", readRawChk->isChecked());
	config->writeEntry("Fast Toc Lookup", fastTocChk->isChecked());
	config->writeEntry("CD Type", outTypeCmb->currentItem());

	//cdda2wav
	config->writeEntry("Echo", echoChk->isChecked());
	config->writeEntry("Paranoia Mode", paranoiaModeChk->isChecked());
	config->writeEntry("CD Type", outTypeCmb->currentItem());
	config->writeEntry("Transfer Size", transferSizeSpn->value());
	config->writeEntry("CD-Rom Speed", cdromSpeedSpn->value());

	//general
	config->writeEntry("CD-Rom Indianess", cdromIndSpn->currentItem());
	config->writeEntry("Output Indianess", outputIndSpn->currentItem());
	config->writeEntry("Perform CDDB Lookup", cddbLookupChk->isChecked());

	config->sync();

	QByteArray data;
	if ( !kapp->dcopClient()->isAttached() )
		kapp->dcopClient()->attach();
	kapp->dcopClient()->send( "cdbakeoven", "CdboPluginManager", "updateSettings()", data );
	kapp->dcopClient()->send( "*", "CdboDialogBase", "readOptions()", data );
}
//-----------------------------------------------------------------------------------------------
void CdboAudioOpt::defaults(){
	config->deleteGroup("Audio Read Options");
	config->sync();
	load();
}
//=========================================
void CdboAudioOpt::slotChanged(){
   emit changed(true);
}
//---------------------------------------------------------------------------------------------
int CdboAudioOpt::buttons() {
	return KCModule::Default|KCModule::Apply|KCModule::Cancel|KCModule::Ok|KCModule::Help;
}
//---------------------------------------------------------------------------------------------
QString CdboAudioOpt::quickHelp() const{
	return I18N_NOOP("Audio Copy Settings");
}
//---------------------------------------------------------------------------------------------
const KAboutData* CdboAudioOpt::aboutData() const
{

	KAboutData *myAboutData = new KAboutData(I18N_NOOP("cdbakeoven"),
					I18N_NOOP("Audio copy options for CD Bake Oven - CD creation tool for KDE"),
                  0, 0, KAboutData::License_GPL,
                  "(c) 2001 - 2002 Alex Podolsky");

    myAboutData->addAuthor("Alex Podolsky", 0, "alexpod@users.sf.net");

    return myAboutData;
}

//---------------------------------------------------------------------------------------------
extern "C"{
    KCModule *create_cdboaudioopt(QWidget *parent, const char *name){
        KGlobal::locale()->insertCatalogue("cdbakeoven");
        return new CdboAudioOpt(parent, name);
    }

}
