/***************************************************************************
                          CdboDefaultOpt.cpp  -  description
                             -------------------
    begin                : Fri Jun 1 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qcheckbox.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qtimer.h>

#include <kcolorbutton.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kglobal.h>
#include <kconfig.h>
#include <dcopclient.h>
#include <kapplication.h>
#include <kactivelabel.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kprocess.h>

#include "CdboDefaultOpt.h"
#include "CdboDefaultOpt.moc"
#include "CdboToolsList.h"
#include "CdboTmpDirConfig.h"

#include "CdboConstants.h"

CdboDefaultOpt::CdboDefaultOpt( QWidget* parent,  const char* name, WFlags  )
    		: CdboDefaultOptBase( parent, name ){
			

	config = new KConfig("cdbakeovenrc");
	disconnect( launchWizardLbl, SIGNAL(linkClicked(const QString &)), 0, 0 );
	connect(launchWizardLbl, SIGNAL(linkClicked(const QString &)), this, SLOT(lauchWizardSlot(const QString&)));
	
	load();
}

/*
 *  Destroys the object and frees any allocated resources
 */
CdboDefaultOpt::~CdboDefaultOpt(){
	delete config;
}
//---------------------------------------------------------------------------------------------
void CdboDefaultOpt::load(){
	config->setGroup("Default Settings");
	maxSpeedSpn->setValue(config->readNumEntry("Max Write Speed", 20));
	comTimeOutSpn->setValue(config->readNumEntry("Default Command Time Out", 60));

	reopenFilesChk->setChecked(config->readBoolEntry("Reopen File Lists", true));
	scanDevicesChk->setChecked(config->readBoolEntry("Auto Scan Devices", true));
	scanScsiChk->setChecked(config->readBoolEntry("Auto Scan SCSI Devices", true));
	
	switch(config->readNumEntry("Level Of Output", Cdbo::StandartOutput)){
		case Cdbo::StandartOutput: stdOutRBtn->setChecked(true); stdOutSlot(); break;
		case Cdbo::CustomOutput: custoutRBtn->setChecked(true); customOutSlot(); break;
		default: quiteOutRBtn->setChecked(true); quiteOutSlot(); break;
	}

	outputLevelSpn->setValue(config->readNumEntry("Custom Level Of Output", 1));
	debugChk->setChecked(config->readBoolEntry("Show Debug Info", false));

	colorDoneBtn->setColor(config->readColorEntry("Process Done Color", &Qt::darkGreen));
	colorLeftBtn->setColor(config->readColorEntry("Process Left Color", &Qt::white));

	regFolderColorBtn->setColor(config->readColorEntry("Regular Folder Color", &Qt::white));
	importedFolderColorBtn->setColor(config->readColorEntry("Immutable Folder Color", &Qt::lightGray));
	regFileColorBtn->setColor(config->readColorEntry("Regular File Color", &Qt::white));
	importFileColorBtn->setColor(config->readColorEntry("Immutable File Color", &Qt::lightGray));
	disableDataColorsChk->setChecked(config->readBoolEntry("Disable Data Colors", false));
	disableDataColorsSlot();

	oggColorBtn->setColor(config->readColorEntry("Ogg Audio Color", &Qt::green));
	mp3ColorBtn->setColor(config->readColorEntry("MP3 Audio Color", &Qt::cyan));
	readyColorBtn->setColor(config->readColorEntry("Ready Audio Color", &Qt::lightGray));
	unknownColorBtn->setColor(config->readColorEntry("Unknown Audio Color", &Qt::red));
	disableAudioColorsChk->setChecked(config->readBoolEntry("Disable Audio Colors", false));
	disableAudioColorsSlot();
	useSameColorsChk->setChecked(config->readBoolEntry("Use Same Data Colors", false));
	useSameColorsSlot();

	config->setGroup("Notification Messages");
	saveCompilationChk->setChecked(config->readBoolEntry("Do Show Save List Warning", true));
	
	tmpDirConfig->readOptions();
	tmpDirConfig->enableErrorMessage();
	QTimer::singleShot(5, tmpDirConfig, SLOT(checkTmpSize()));
	QTimer::singleShot(5, toolsList, SLOT(loadSavedTools()));
}
//-----------------------------------------------------------------------------------------------
void CdboDefaultOpt::save(){
	config->setGroup("Default Settings");
	config->writeEntry("Max Write Speed", maxSpeedSpn->value());
	config->writeEntry("Default Command Time Out", comTimeOutSpn->value());

	config->writeEntry("Reopen File Lists", reopenFilesChk->isChecked());
	config->writeEntry("Auto Scan Devices", scanDevicesChk->isChecked());
	config->writeEntry("Auto Scan SCSI Devices", scanScsiChk->isChecked());

	if(stdOutRBtn->isChecked())
		config->writeEntry("Level Of Output", Cdbo::StandartOutput);
	else if(custoutRBtn->isChecked())
		config->writeEntry("Level Of Output", Cdbo::CustomOutput);
	else
		config->writeEntry("Level Of Output", Cdbo::QuiteOutput);

	config->writeEntry("Custom Level Of Output", outputLevelSpn->value());
	config->writeEntry("Show Debug Info", debugChk->isChecked());

	config->writeEntry("Process Done Color", colorDoneBtn->color());
	config->writeEntry("Process Left Color", colorLeftBtn->color());

	config->writeEntry("Regular Folder Color", regFolderColorBtn->color());
	config->writeEntry("Immutable Folder Color", importedFolderColorBtn->color());
	config->writeEntry("Regular File Color", regFileColorBtn->color());
	config->writeEntry("Immutable File Color", importFileColorBtn->color());
	config->writeEntry("Disable Data Colors", disableDataColorsChk->isChecked());

	config->writeEntry("Ogg Audio Color", oggColorBtn->color());
	config->writeEntry("MP3 Audio Color", mp3ColorBtn->color());
	config->writeEntry("Ready Audio Color", readyColorBtn->color());
	config->writeEntry("Unknown Audio Color", unknownColorBtn->color());
	config->writeEntry("Disable Audio Colors", disableAudioColorsChk->isChecked());

	config->writeEntry("Use Same Data Colors", useSameColorsChk->isChecked());


	config->setGroup("Notification Messages");
	config->writeEntry("Do Show Save List Warning", saveCompilationChk->isChecked());

	config->sync();
	
	tmpDirConfig->saveOptions();
	toolsList->saveOptions();
	
	QByteArray data;
	if ( !kapp->dcopClient()->isAttached() )
		kapp->dcopClient()->attach();
	kapp->dcopClient()->send( "cdbakeoven", "CdboPluginManager", "updateSettings()", data );
	kapp->dcopClient()->send( "*", "CdboDialogBase", "readOptions()", data );
}
//----------------------------------------------------------------------------------------------
void CdboDefaultOpt::defaults(){
	config->deleteGroup("Default Settings");
	config->deleteGroup("Notification Messages");
	load();
	tmpDirConfig->defaults();
	QTimer::singleShot(5, toolsList, SLOT(scanTools()));
}
//------------------------------------------------------------------------------------------
void CdboDefaultOpt::customOutSlot(){
	enableCustOut(true);
}
//------------------------------------------------------------------------------------------
void CdboDefaultOpt::quiteOutSlot(){
	enableCustOut(false);
}
//------------------------------------------------------------------------------------------
void CdboDefaultOpt::stdOutSlot(){
	enableCustOut(false);
}
//------------------------------------------------------------------------------------------
void CdboDefaultOpt::enableCustOut(bool yes){
	debugChk->setEnabled(yes);
	outputLbl->setEnabled(yes);
	outputLevelSpn->setEnabled(yes);
}
//---------------------------------------------------------------------------------------------
void CdboDefaultOpt::disableDataColorsSlot(){
	bool yes = !disableDataColorsChk->isChecked();
	regFolderColorBtn->setEnabled(yes);
	importedFolderColorBtn->setEnabled(yes);
	useSameColorsChk->setEnabled(yes);
	if(!yes){
		regFileColorBtn->setEnabled(false);
		importFileColorBtn->setEnabled(false);
	}
	else
		useSameColorsSlot();
}
//---------------------------------------------------------------------------------------------
void CdboDefaultOpt::disableAudioColorsSlot(){
	bool yes = !disableAudioColorsChk->isChecked();
	oggColorBtn->setEnabled(yes);
	mp3ColorBtn->setEnabled(yes);
	readyColorBtn->setEnabled(yes);
	unknownColorBtn->setEnabled(yes);
}
//---------------------------------------------------------------------------------------------
void CdboDefaultOpt::useSameColorsSlot(){
	bool yes = !useSameColorsChk->isChecked();
	regFileColorBtn->setEnabled(yes);
	importFileColorBtn->setEnabled(yes);
}
//---------------------------------------------------------------------------------------------
void CdboDefaultOpt::lauchWizardSlot(const QString&){
	KShellProcess *cdboProc = new KShellProcess();
	*cdboProc << "cdbosetuptool --launchWizard";
	connect(cdboProc, SIGNAL(processExited(KProcess*)), this, SLOT(kcmshellDone(KProcess*)));
	if(!cdboProc->start(KProcess::NotifyOnExit)){
		kdWarning() << "Failed to launch cdbosetuptool --launchWizard" << endl;
		delete cdboProc;
	}
}
//---------------------------------------------------------------------------------------------
void CdboDefaultOpt::kcmshellDone(KProcess *proc){
	delete proc;
}
//=========================================
void CdboDefaultOpt::slotChanged(){
	emit changed(true);
}
//---------------------------------------------------------------------------------------------
int CdboDefaultOpt::buttons() {
	return KCModule::Default|KCModule::Apply|KCModule::Cancel|KCModule::Ok|KCModule::Help;
}
//---------------------------------------------------------------------------------------------
QString CdboDefaultOpt::quickHelp() const{
	return I18N_NOOP("CD Bake Oven Default Settings.");
}
//---------------------------------------------------------------------------------------------
const KAboutData* CdboDefaultOpt::aboutData() const
{

	KAboutData *myAboutData = new KAboutData(I18N_NOOP("cdbakeoven"),
		I18N_NOOP("Default settings for CD Bake Oven - CD creation tool for KDE."),
		0, 0, KAboutData::License_GPL,
		"(c) 2001 - 2002 Alex Podolsky");

	myAboutData->addAuthor("Alex Podolsky", 0, "alexpod@users.sf.net");

	return myAboutData;
}

//---------------------------------------------------------------------------------------------
extern "C"{
	KCModule *create_cdbodefaultopt(QWidget *parent, const char *name){
		KGlobal::locale()->insertCatalogue("cdbakeoven");
		return new CdboDefaultOpt(parent, name);
	}

}
