/***************************************************************************
                          CdboToolsList.cpp  -  description
                             -------------------
    begin                : Fri Jul 14 2002
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qfileinfo.h>
#include <qpopupmenu.h>

#include <kstandarddirs.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <kdebug.h>

#include "CdboToolsList.h"
#include "CdboToolsList.moc"

KInstance *CdboToolsList::mInstance = 0;

CdboToolsList::CdboToolsList( QWidget* parent, const char* name, WFlags fl )
    : CdboToolsListBase( parent, name, fl ){
	
	toolsBrowseBtn->setPixmap( BarIcon("fileopen", KIcon::SizeSmall ) );
	
	toolMenu = new QPopupMenu();
	thpIndex = toolMenu -> insertItem(BarIcon("gohome"), i18n("Get Latest Version..."),this ,SLOT(launchToolHomepage()));
	toolMenu -> insertItem(BarIcon("reload"), i18n("Rescan Tools"),this ,SLOT(rescanTools()));
	toolMenu -> insertSeparator();
	toolMenu -> insertItem(BarIcon("txt"), i18n("Save this Information to a File"),this ,SLOT(dumpToolsSlot()));
	connect( toolsList, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
				this, SLOT(popupToolsMenuSlot(QListViewItem*, const QPoint&, int)));
   
}

/*
 *  Destroys the object and frees any allocated resources
 */
CdboToolsList::~CdboToolsList(){
	delete mInstance;
	mInstance = 0;
}
//======================================
KInstance *CdboToolsList::instance()
{
	if(!mInstance)
		mInstance = new KInstance("cdbakeoven");
	return mInstance;
}
//============================================
void CdboToolsList::initToolsToFixList(QStringList &list){
	if(toolsToFixList.count() == 0)
		return;
	for(unsigned int i = 0; i < toolsToFixList.count(); i++)
		list.append(toolsToFixList[i]);
}
//============================================
void CdboToolsList::saveOptions(){
	KConfig* config = new KConfig("cdbakeovenrc");
	config->setGroup("Wizard Detected");
	for(QListViewItem *item = toolsList->firstChild(); item; item = item->nextSibling())
		config->writeEntry("Path to " + item->text(0), item->text(2));
	
	config->sync();
	delete config;
}
//============================================
void CdboToolsList::loadSavedTools(){
	populateTools(true);
}
//============================================
void CdboToolsList::scanTools(){
	populateTools(false);
}
//============================================
void CdboToolsList::populateTools(bool loadPrevPath){
	toolsList->clear();
	
	KConfig* config = new KConfig("cdbakeovenrc");
	config->setGroup("Wizard Detected");
	QString path = QString::null;
	if(loadPrevPath)
		path = config->readEntry("Path to cdrecord", QString::null);
	initSingleTool("cdrecord", i18n("Records audio or data CDs"),
		"http://www.fokus.gmd.de/research/cc/glone/employees/joerg.schilling/private/cdrecord.html", path);
		
	path = QString::null;
	if(loadPrevPath)
		path = config->readEntry("Path to cdda2wav", QString::null);
	initSingleTool("cdda2wav", i18n("Reads audio CDs"),
		"http://www.fokus.gmd.de/research/cc/glone/employees/joerg.schilling/private/cdrecord.html", path);
	
	path = QString::null;
	if(loadPrevPath)
		path = config->readEntry("Path to readcd", QString::null);
	initSingleTool("readcd", i18n("Reads data CDs"),
		"http://www.fokus.gmd.de/research/cc/glone/employees/joerg.schilling/private/cdrecord.html", path);
	
	path = QString::null;
	if(loadPrevPath)
		path = config->readEntry("Path to cdrdao", QString::null);
	initSingleTool("cdrdao", i18n("Writes CDs in disc-at-once mode"),
		"http://cdrdao.sourceforge.net", path);
	
	path = QString::null;
	if(loadPrevPath)
		path = config->readEntry("Path to mkisofs", QString::null);
	initSingleTool("mkisofs", i18n("Creates hybrid ISO9660/JOLIET/HFS filesystem"),
		"http://www.fokus.gmd.de/research/cc/glone/employees/joerg.schilling/private/cdrecord.html", path);
	
	path = QString::null;
	if(loadPrevPath)
		path = config->readEntry("Path to ogg123", QString::null);
	initSingleTool("ogg123" , i18n("Decompression of Ogg Vorbis files"),
		"http://www.xiph.org/ogg/vorbis/index.html", path);
	
	path = QString::null;
	if(loadPrevPath)
		path = config->readEntry("Path to mpg123", QString::null);
	initSingleTool("mpg123", i18n("Decompression of MP3 files"),
		"http://www.mpg123.de", path);

	delete config;
}
//=====================================
void CdboToolsList::initSingleTool(const QString &name, const QString &description, const QString &homePage, const QString &path){

	QListViewItem *item = new QListViewItem(toolsList, 0);
	item->setText(0, name);
	item->setText(3, description);
	item->setText(4, homePage);
	checkTool(item, path);
}
//=====================================
void CdboToolsList::checkTool(QListViewItem * item, QString path){
	if(path.isNull())///if default was passed in
		path = KStandardDirs::findExe(item->text(0));

	if(path.isNull()){
		item->setPixmap(0, UserIcon("cross", KIcon::SizeSmall, instance()));
		//lets try to find it in some "could be" places
		path = item->text(0);
		QStringList possibleDirs =
			QStringList::split(",", "/opt/schily/bin/,/usr/bin/,/usr/local/bin/,/usr/local/sbin/");
		QFileInfo info;
		for(unsigned int i =0; i < possibleDirs.count(); i++){
			info.setFile(possibleDirs[i] + path);
			if(info.exists()){
				item->setText(1, info.group());
				item->setText(2, possibleDirs[i] + path);
				if(toolsToFixList.findIndex(possibleDirs[i] + path) == -1)
					toolsToFixList.append(possibleDirs[i] + path);
				return;
			}
		}
		item->setText(1, i18n("N/A"));
		item->setText(2, i18n("Not detected"));
		return;
	}
	else{
		QFileInfo info(path);
		item->setText(1, info.group());
		item->setText(2, path);

		if(info.isExecutable()){
			if(toolsToFixList.findIndex(path) != -1)
				toolsToFixList.remove(path);
			item->setPixmap(0, UserIcon("ok", KIcon::SizeSmall, instance()));
		}
		else{
			if(toolsToFixList.findIndex(path) == -1)
				toolsToFixList.append(path);
			item->setPixmap(0, UserIcon("cross", KIcon::SizeSmall, instance()));
		}
	}
}
//=====================================
void CdboToolsList::rescanTools(){
	QString buff;
	for(QListViewItem *i = toolsList->firstChild(); i; i = i->nextSibling()){
		buff = i->text(2);
		if(buff != i18n("Not detected"))
			checkTool(i, buff);
		else
			checkTool(i);
	}
}
//==============================================================
void CdboToolsList::popupToolsMenuSlot(QListViewItem *, const QPoint &point, int){
	if(toolsList->childCount() == 0)
		return;
	if(toolsList->selectedItem() == 0)
		toolMenu->setItemEnabled(thpIndex, false);
	else
		toolMenu->setItemEnabled(thpIndex, true);
	toolMenu->popup( point);
}
//============================================
void CdboToolsList::toolActivatedSlot(QListViewItem *item){
	toolsPathTxt->setText(item->text(2));
}
//============================================
void CdboToolsList::toolsApplySlot(){

	if(toolsPathTxt->text().isEmpty())
		return;
	QListViewItem * item = toolsList->selectedItem();
	if(!item)
		return;

	if(toolsPathTxt->text() == item->text(2))
		return;

	QFileInfo info(toolsPathTxt->text());
	if(!info.exists()){
		KMessageBox::detailedError(this, i18n("Invalid path entered."),
				i18n("Chosen path does not point to a valid file."), i18n("Invalid Path."));
		return;
	}

	checkTool(item, toolsPathTxt->text());
	emit modified();
}
//============================================
void CdboToolsList::toolsBrowseSlot(){
	KURL url;
	QString buff = toolsPathTxt->text();
	if(buff == i18n("Not detected")){
		url=KFileDialog::getOpenURL("/",
			i18n("*|All files"),
			this, i18n("Specify Tool Location"));
	}
	else{
		int i = buff.findRev("/");
		if(i > 0)
			buff = buff.left(i);

		if(!buff || buff == ""){
			url=KFileDialog::getOpenURL("/",
				i18n("*|All files"),
				this, i18n("Specify Tool Location"));
		}
		else{
			url=KFileDialog::getOpenURL(buff,
				i18n("*|All files"),
				this, i18n("Specify Tool Location"));
		}
	}
	if(url.isEmpty()){
		  return;
	}
	buff = url.path(0).stripWhiteSpace();
	toolsPathTxt->setText(buff);

}
//=====================================
void CdboToolsList::dumpToolsSlot(){

	if(!toolsList->firstChild())
		return;
    QString fileToOpen = KFileDialog::getSaveFileName(QDir::homeDirPath(),
				i18n("*|All files"),
				this,
				i18n("Save Output As..."));
	if(!fileToOpen || fileToOpen == "")
		return;
	QFile file(fileToOpen);

	if(file.exists())
		file.remove();
	if(!file.open(IO_ReadWrite))
		return;

	QTextStream stream(&file);

	stream << i18n("Detected tools configuration:");
	stream << "\n\n";

	for(int i = 0; i < toolsList->columns(); i++){
		stream << toolsList->columnText(i);
		stream << "\t";
	}
	stream << "\n\n\n";

	for(QListViewItem *i = toolsList->firstChild(); i; i = i->nextSibling()){
		stream << i->text(0);
		stream << "\t";
		stream << i->text(1);
		stream << "\t";
		stream << i->text(2);
		stream << "\t";
		stream << i->text(3);
		stream << "\n";
	}
	QDate date = QDate::currentDate();
	stream << "\n\nCD Bake Oven. ";
	stream << date.toString();
	stream << ".";
	file.close();
}
//============================================
void CdboToolsList::launchToolHomepage(){
	QString hp = toolsList->selectedItem()->text(4);
	KApplication::kApplication()->invokeBrowser(hp);
}



