/***************************************************************************
                          CdboAction.cpp  -  description
                             -------------------
    begin                : Wed Aug 14 2002
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdir.h>
#include <qfile.h>
#include <qtimer.h>

#include <kapplication.h>
#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klibloader.h>

#include "CdboAction.h"
#include "CdboAction.moc"

#include "CdboConstants.h"

CdboAction::CdboAction( QObject* user, const char* name) :
	QObject(user, name){
	
	doingSomething = false;
	unlockUser = true;
	autoDelete = true;
	showCanceledMess = true;
	config = 0;
}
//===================================================
CdboAction::~CdboAction(){
	if(config)
		delete config;
}
//===================================================
void CdboAction::performTask(CdboParamsDict* params){
	emitBlockBtns(true);

	prepareForProcess(params);

	currentProc = new KShellProcess();
	initProcess(currentProc, params);
	if(readyToLaunch)
		launchProcess(currentProc);
	
}
//===================================================
bool CdboAction::initProcessForPiping(KProcess *proc, CdboParamsDict* params, CdboAction *owner){
	prepareForProcess(params);

	connect(this, SIGNAL(processCancelled()), owner, SLOT(processCancelledByUser()));
	connect(this, SIGNAL(status(QString)), owner, SLOT(emitStatus(QString)));
	connect(this, SIGNAL(output(int,QString)), owner, SLOT(emitOutput(int,QString)));
	connect(this, SIGNAL(percentInfo(QString,QString)), owner, SLOT(emitPercentInfo(QString,QString)));
	connect(this, SIGNAL(percent(QString,int,QString)), owner, SLOT(emitPercent(QString,int,QString)));

	
	initProcess(proc, params);
	return readyToLaunch;

}
//===================================================
void CdboAction::prepareForProcess(CdboParamsDict* params){
	doingSomething = true;
	readyToLaunch = true;

	this->params = params;
	DEBUG_COMMANDS = false;
	boolParamVal("DEBUG_COMMANDS", DEBUG_COMMANDS, false);
	DEBUG_OUTPUT = false;
	boolParamVal("DEBUG_OUTPUT", DEBUG_OUTPUT, false);

	resetConfig();
}
//===================================================
void CdboAction::launchProcess(KProcess *proc){
	config->setGroup("Default Settings");
	proc->setWorkingDirectory(config->readEntry("Tmp Dir", "/tmp/"));
	QDir::setCurrent(config->readEntry("Tmp Dir", "/tmp/"));
	
	connect(proc, SIGNAL(processExited(KProcess *)),
		this, SLOT(processDoneSlot(KProcess *)));
	
	connect(proc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedMessageSlot(KProcess *, char *, int)));
	
	connect(proc, SIGNAL(receivedStderr (KProcess *, char *, int)),
		this, SLOT(receivedErrMessageSlot(KProcess *, char *, int )));

	debugProcess(proc);
	
	if(proc->start(KProcess::NotifyOnExit, (KProcess::Communication)(KProcess::Stdin|KProcess::Stdout|KProcess::Stderr)))
		processLaunched();
	else{
		delete proc;
		processFailedToLaunch();
	}
}
//---------------------------------------------------------------------------------------------
void CdboAction::processDoneSlot(KProcess *proc){
	if(proc->normalExit() && proc->exitStatus() == 0){
		delete proc;
		processSucceded();
		
	}
	else{
		delete proc;
		processFailed();
		
		qApp->processEvents();///////////////////////////
	}
}
//---------------------------------------------------------------------------------------------
void CdboAction::receivedMessageSlot(KProcess *, char *buffer, int buflen){	
	processOutput(Cdbo::InfoMessage, buffer, buflen);
}
//---------------------------------------------------------------------------------------------
void CdboAction::receivedErrMessageSlot(KProcess *, char *buffer, int buflen){
	processOutput(Cdbo::ProcessMessage, buffer, buflen);
}
//---------------------------------------------------------------------------------------------
void CdboAction::processOutput(int type, char *buffer, int buflen){
	if(buflen > 0){
		QString message(buffer);
		message = message.left(buflen);
		if(DEBUG_OUTPUT)
			kdDebug() << message << endl;

		QStringList list(QStringList::split("\n",message, false));
		for(int i = 0; i < (int)list.count(); i++){
			if(list[i].isEmpty())
				continue;
			message = list[i];
			if(filterOutput(type, message))
				emitOutput(type, message);
		}
 	}
}
//---------------------------------------------------------------------------------------------
void CdboAction::debugProcess(KProcess *proc){
	///////////////////////////////////////////////////////
	if(!DEBUG_COMMANDS)
		return;

	QCString command = "\n\n<<";
	QValueList<QCString> list = proc->args();
	for(int i = 0; i < (int)list.count(); i++)
		command = command + " " + list[i];
	kdDebug() << command << ">>\n\n";
}
//---------------------------------------------------------------------------------------------
void CdboAction::processLaunched(){
	//
}
//---------------------------------------------------------------------------------------------
void CdboAction::processFailedToLaunch(){
	emit failedToLaunchProcess();
	resetSettings();
	QTimer::singleShot(100, this, SLOT(emitActionDone()));
}
//---------------------------------------------------------------------------------------------
void CdboAction::emitActionDone() {
//kdDebug() << name() << " called emitActionDone()" << endl;
	if(autoDelete)
		emit actionDone(this);
}
//---------------------------------------------------------------------------------------------
void CdboAction::processSucceded(){
//kdDebug() << name() << " called CdboAction::processSucceded()>>>>>>>>>" << endl;
	emit processDone(true);
	resetSettings();
	QTimer::singleShot(100, this, SLOT(emitActionDone()));
}
//---------------------------------------------------------------------------------------------
void CdboAction::processFailed(){
//kdDebug() << name() << " called CdboAction::processFailed()>>>>>>>>>" << endl;
	emit processDone(false);
	resetSettings();
	QTimer::singleShot(100, this, SLOT(emitActionDone()));
}
//---------------------------------------------------------------------------------------------
void CdboAction::resetSettings(){
//kdDebug() << name() << " called CdboAction::resetSettings()>>>>>>>>>" << endl;
	currentProc = 0;
	readyToLaunch = false;
	canceled = false;
	if(unlockUser){
		emitStatus(i18n("Ready"));
		emitBlockBtns(false);
	}
	doingSomething = false;
}
//---------------------------------------------------------------------------------------------
bool CdboAction::tryToCancel(){
//kdDebug() << name() << " called CdboAction::tryToCancel()>>>>>>>>>" << endl;
	if(!doingSomething){
		processCancelledByUser();
		return true;
	}
	int result = KMessageBox::warningContinueCancel(0,
		cancelMessage(),
		i18n("Confirm abort"),
		QString(i18n("&Abort")));
	switch(result) {
		case KMessageBox::Continue:{
			cancelCurrentTask();
			return true;
		}
		case KMessageBox::Cancel:
			 return false;
	}
	return false;
}
//---------------------------------------------------------------------------------------------
void CdboAction::cancelCurrentTask(){
//kdDebug() << name() << " called CdboAction::cancelCurrentTask()>>>>>>>>>" << endl;
		canceled = true;
		killCurrentTask();
		processCancelledByUser();
}
//---------------------------------------------------------------------------------------------
void CdboAction::processCancelledByUser(){
//kdDebug() << name() << " called CdboAction::processCancelledByUser()>>>>>>>>>" << endl;
	emit processCancelled();
	resetSettings();
	if(showCanceledMess)
		emitOutput(Cdbo::StatusNotOkMessage, i18n("Process canceled."));
	//else
		//kdDebug() << name() << ": showCanceledMess = false" << endl;
	QTimer::singleShot(100, this, SLOT(emitActionDone()));
}
//---------------------------------------------------------------------------------------------
QString CdboAction::cancelMessage(){
	return i18n("Aborting now may damage the CD-RW media, "\
		"and/or even hang your system.");
}
//---------------------------------------------------------------------------------------------
void CdboAction::killCurrentTask(){
//kdDebug() << name() << " called CdboAction::killCurrentTask()>>>>>>>>>" << endl;
	if(currentProc){
		if(currentProc->isRunning()){
			currentProc->kill();
			delete currentProc;
		}
		currentProc = 0;
	}
	QTimer::singleShot(100, this, SLOT(emitActionDone()));
}
//---------------------------------------------------------------------------------------------
void CdboAction::internalError(QString errorText){
	emitOutput(Cdbo::ErrorMessage, i18n("Internal error occured:"));
	emitOutput(Cdbo::ErrorMessage, i18n(errorText.utf8()));
	emitOutput(Cdbo::ErrorMessage,
		i18n("Please submit this as a bug to the email address located in the Help->About dialog."));
	readyToLaunch = false;
	processFailed();
}
//---------------------------------------------------------------------------------------------
void CdboAction::resetConfig(){
	if(config)
		delete config;
	config = new KConfig("cdbakeovenrc");
}
//---------------------------------------------------------------------------------------------
void CdboAction::removeTmpImage(QString image, QString dirName){
	if(!image)
		return;
	QString filter = image.left(image.findRev(".")) + "*";
	filter = filter.right(filter.length() - filter.findRev("/") - 1);
	QDir tempDir(dirName, filter);

	QStringList flist = tempDir.entryList();

	for(int i = 0; i < (int)flist.count(); i++){
		removeOneFile(dirName + "/" + flist[i]);
	}
}
//---------------------------------------------------------------------------------------------
void CdboAction::removeOneFile(QString fileName, bool showError){
	if(!fileName)
		return;
	if(!QFile::remove(fileName) && showError)
		emitOutput(Cdbo::ErrorMessage, i18n("Failed to remove %1.").arg(fileName));
}
//---------------------------------------------------------------------------------------------
void CdboAction::removeOneDir(QString dirName, bool showError){
	if(!dirName)
		return;
	QDir dirToRem(dirName);
	if(!dirToRem.rmdir(dirName,true) && showError)
		emitOutput(Cdbo::ErrorMessage, i18n("Failed to remove %1.").arg(dirName));
}
//---------------------------------------------------------------------------------------------
bool CdboAction::requestReload(bool signalToProcess){
	emitOutput(Cdbo::StatusOkMessage, i18n("Requesting to reload medium."));
	KMessageBox::information(0, i18n("Please reload media in this drive and click 'OK'. Please make sure to close the trey."),
		i18n("Please reload"));
	if(signalToProcess)
		return sendSignalToProcess();
	else
		return true;
}
//---------------------------------------------------------------------------------------------
bool CdboAction::requestCDR(bool signalToProcess, const QString &driveName){
	emitOutput(Cdbo::StatusOkMessage, i18n("Requesting to insert recordable medium."));
	
	if(!driveName.isEmpty())
		eject(driveName);

	emitStatus(i18n("Waiting for CD-R(W)..."));
	int result = KMessageBox::questionYesNo(0,i18n("Read is complete. Please insert CD-R(W) into recording device and click 'OK'."),
		i18n("Please insert CD-R(W)"),
		KStdGuiItem::ok(), i18n("&Stop Here"));

	if(result == KMessageBox::No){
		cancelCurrentTask();
		return false;
	}
	if(!driveName.isEmpty())
		closeTray(driveName);
	if(signalToProcess)
		return sendSignalToProcess();
	else
		return true;
}
//---------------------------------------------------------------------------------------------
bool CdboAction::sendSignalToProcess(){
	if(currentProc->writeStdin("\n", 1)){
		kdDebug() << name() << ": Signal to process went through" << endl;
		return true;
	}
	else{
		internalError(i18n( "Failed to communicate with the process."));
		kdDebug() << name() << ": Signal to process failed" << endl;
		return false;
	}
}
//---------------------------------------------------------------------------------------------
void CdboAction::eject(const QString &driveName){
	stillEjecting = true;
	KShellProcess* ejectProc = new KShellProcess();
	*ejectProc << "eject" << driveName;
	connect(ejectProc, SIGNAL(processExited(KProcess *)),
     			this, SLOT(ejectProcComplited(KProcess *)));

	if(!ejectProc->start(KProcess::NotifyOnExit)){
		kdDebug() << name() << ": Failed to eject " << driveName << endl;
		delete ejectProc;
		return;
	}
	while(stillEjecting)
		qApp->processEvents(1);
}
//---------------------------------------------------------------------------------------------
void CdboAction::closeTray(const QString &driveName){
	stillEjecting = true;
	KShellProcess* ejectProc = new KShellProcess();
	*ejectProc << "eject" << "-t" << driveName;
	connect(ejectProc, SIGNAL(processExited(KProcess *)),
     			this, SLOT(ejectProcComplited(KProcess *)));

	if(!ejectProc->start(KProcess::NotifyOnExit)){
		kdDebug() << name() << ": Failed to close tray " << driveName << endl;
		delete ejectProc;
		return;
	}
	while(stillEjecting)
		qApp->processEvents(1);
}
//---------------------------------------------------------------------------------------------
void CdboAction::ejectProcComplited(KProcess *ejectProc){
	delete ejectProc;
	stillEjecting = false;
}
//---------------------------------------------------------------------------------------------
void CdboAction::enableTimeout(){
	config->setGroup("Default Settings");
	timeOutId = startTimer(config->readNumEntry("Default Command Time Out", 60) * 1000);
	watchTime = true;
}
//---------------------------------------------------------------------------------------------
void CdboAction::timerEvent(QTimerEvent *event){
	if(event->timerId() == timeOutId && watchTime){
		processFailedToLaunch();
	}
}
//---------------------------------------------------------------------------------------------
void CdboAction::resetTimeout(){
	if(!watchTime)
		return;
	killTimer(timeOutId);
	watchTime = false;
}
//---------------------------------------------------------------------------------------------
QString CdboAction::timeStamp(){
	QDate today = QDate::currentDate();
	QTime now = QTime::currentTime();
	return QString::number(today.month()) +
		"_" + QString::number(today.day()) +
		"_" + QString::number(now.hour()) +
		"_" + QString::number(now.minute()) +
		"_" + QString::number(now.second());
}
//---------------------------------------------------------------------------------------------
QString CdboAction::timeString(long posSecs){
	int posSeconds = (int)posSecs % 60;
	int posMinutes = (posSecs - posSeconds) / 60;

	QString result;
	result.sprintf("%.2d:%.2d", posMinutes, posSeconds);
	return result;
}
//-----------------------------------------------
QString CdboAction::paramVal(QString paramName, bool showError){
	if(!params){
		if(showError)
			internalError(i18n("No Parameters."));
		return QString::null;
	}
	QString *param = dynamic_cast<QString *>((*params)[paramName]);
	if(param == NULL || param->isNull()){
		if(showError)
			internalError(i18n("Failed to get '%1' for: %2.").arg(paramName).arg(name()));
		return QString::null;
	}
	QString val = *param;
	return val;
}
//-----------------------------------------------
void CdboAction::setParamVal(QString paramName, QString *paramValue){
	if(!params)
		return;
	params->remove(paramName);
	params->insert(paramName, paramValue);
}
//-----------------------------------------------
bool CdboAction::boolParamVal(QString paramName, bool &value, bool showError){
	if(!params)
		return false;
	QString param = paramVal(paramName, showError);
	if(param.isEmpty() && !param.isNull()){
		if(showError)
			internalError(i18n("Failed to get '%1' for: %2.").arg(paramName).arg(name()));
		return false;
	}
	 value = (param == "TRUE");
	 return true;
}
//-----------------------------------------------
bool CdboAction::listParamVal(QString paramName, QStringList &list, QString breakFlag, bool showError){
	QString param = paramVal(paramName, showError);
	if(param.isEmpty() && !param.isNull()){
		if(showError)
			internalError(i18n("Failed to get '%1' for: %2.").arg(paramName).arg(name()));
		return false;
	}
	list = QStringList::split(breakFlag, param);
	return true;
}
//-----------------------------------------------
bool CdboAction::intParamVal(QString paramName, int &num, bool showError){
	QString param = paramVal(paramName, showError);
	if(param.isEmpty() && !param.isNull()){
		if(showError)
			internalError(i18n("Failed to get '%1' for: %2.").arg(paramName).arg(name()));
		return false;
	}
		
	bool ok;
	num = param.toInt(&ok);
	if(!ok){
		if(showError)
			internalError(i18n("Failed to get '%1' for: %2.").arg(paramName).arg(name()));
		return false;
	}
	return true;
}
////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
CdboJobBasedAction::CdboJobBasedAction( QObject* d, const char* name) :
	CdboAction(d, name){
	curJob = 0;
}
//-----------------------------------------------
CdboJobBasedAction::~CdboJobBasedAction(){
}
//---------------------------------------------------------------------------------------------
void CdboJobBasedAction::performTask(CdboParamsDict* params){
	doingSomething = true;
	readyToLaunch = true;
	emitBlockBtns(true);

	this->params = params;
	resetConfig();
	currentProc = 0;
	launchJob(params);
}
//---------------------------------------------------------------------------------------------
void CdboJobBasedAction::resetSettings(){
	curJob = 0;
	CdboAction::resetSettings();
}
//---------------------------------------------------------------------------------------------
void CdboJobBasedAction::killCurrentTask(){
	if(curJob){
		curJob->kill();
	}
	CdboAction::killCurrentTask();
}


////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
CdboActionHandler::CdboActionHandler(KLibFactory* factory, QObject *user, const char *name) :
	CdboAction(user, name){
	slaveAction = 0;
	parentFactory = factory;
	slaveQueue.setAutoDelete(false);
}
CdboActionHandler::~CdboActionHandler(){
	
}
//===================================================
void CdboActionHandler::prepareForProcess(CdboParamsDict* params){
	CdboAction::prepareForProcess(params);
	
	if(!intParamVal("NUMBER_OF_COPIES", copiesWanted, false))
		copiesWanted = 1;
	copiesDone = 1;
}
//---------------------------------------------------------------------------------------------
void CdboActionHandler::resetSettings(){
//kdDebug() << name() << " called CdboActionHandler::resetSettings()>>>>>>>>>" << endl;
	removeAllSlaves();
	slaveAction = 0;
	CdboAction::resetSettings();
}
//---------------------------------------------------------------------------------------------
bool CdboActionHandler::createAction(QString type, KLibFactory* factory){
	if(!factory)
		factory = parentFactory;
		
	if(!factory){
		internalError(i18n("Failed to create action of type '%1', factory is NULL.").arg(type));
		return false;
	}
	QObject *obj = factory->create(parent(), type.latin1(), type.latin1());
	if(!obj){
		internalError(i18n("Failed to create object of type '%1'.").arg(type));
		return false;
	}
	if ( !obj->inherits ( "CdboAction" ) ){
		internalError(i18n("Failed to create action of type '%1'.").arg(type));
		return false;
	}
	slaveAction = static_cast<CdboAction* >(obj);
	
	slaveAction->disconnect( this );//in case the action is reused

	connect( slaveAction, SIGNAL( processDone(bool) ), this, SLOT( slaveActionDone(bool) ) );
	connect( slaveAction, SIGNAL( failedToLaunchProcess() ), this, SLOT( slaveActionFailedToLaunch()) );
	connect( slaveAction, SIGNAL( processCancelled() ), this, SLOT( slaveActionCancelled()) );

	connect( slaveAction, SIGNAL( blockBtns(bool) ), this, SLOT( emitBlockBtns(bool)));
	connect( slaveAction, SIGNAL( status(QString) ), this, SLOT( emitStatus(QString)));
	connect( slaveAction, SIGNAL( output(int, QString) ),
				this, SLOT( emitOutput(int, QString)));
	connect( slaveAction, SIGNAL( percent(QString,int,QString) ),
				this, SLOT( emitPercent(QString,int,QString) ));
	connect( slaveAction, SIGNAL( percentInfo(QString,QString) ),
				this, SLOT( emitPercentInfo(QString,QString) ));

	slaveQueue.enqueue(slaveAction);
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboActionHandler::begin(){
	if(slaveQueue.isEmpty()){
		kdDebug() << name() << ": slaveQueue is empty, " << this->name() << " done." << endl;
		processSucceded();
		QTimer::singleShot(100, this, SLOT(emitActionDone()));
		return;
	}
	else{
		slaveAction = slaveQueue.dequeue();
		kdDebug() << name() << ":Executing first action: " << slaveAction->name() << endl;
			kdDebug() << name() << ": (" << slaveQueue.count() << ") more to go. " << endl;
		QTimer::singleShot(1, this, SLOT(lunchSlaveAction()));
		return;
	}
}
//---------------------------------------------------------------------------------------------
void CdboActionHandler::slaveActionDone(bool success){
	if(success){
		if(slaveQueue.isEmpty()){
		
			if(copiesDone < copiesWanted){
				kdDebug() << name() << ": copiesDone < copiesWanted" << endl;
				slaveAction->setAutoDelete(false);

				int result = KMessageBox::questionYesNo(0,i18n("%1 of %2 requested copies are done. " 
					"Please insert next CD-R(W) and click 'OK'.").arg(QString::number(copiesDone)).arg(QString::number(copiesWanted)),
					i18n("Please insert next CD-R(W)"),
					KStdGuiItem::ok(), i18n("&Stop Here"));

				if(result == KMessageBox::Yes){
					copiesDone++;
					if(!createAction(slaveAction->name())){
						processFailed();
						return;
					}
					slaveAction = slaveQueue.dequeue();
					kdDebug() << name() << ": Repeating last action:" << slaveAction->name() << endl;
					QTimer::singleShot(1, this, SLOT(lunchSlaveAction()));
					return;
				}
				else
					copiesDone = copiesWanted;
			}
			if(copiesDone == copiesWanted)
				slaveAction->setAutoDelete(true);
		
			kdDebug() << name() << ": slaveQueue is empty, " << this->name() << " done." << endl;
			processSucceded();
			QTimer::singleShot(100, this, SLOT(emitActionDone()));
			return;
		}
		else{
			slaveAction = slaveQueue.dequeue();
			kdDebug() << name() << ": Executing next action: " << slaveAction->name() << endl;
			kdDebug() << name() << ": (" << slaveQueue.count() << ") more to go. " << endl;
			QTimer::singleShot(1, this, SLOT(lunchSlaveAction()));// I don't get it, why is this needed
			return;
		}
	}
	else{
		if(slaveAction)
			kdDebug() << name() << ": " << slaveAction->name() << " failed." << endl;
		processFailed();
		QTimer::singleShot(100, this, SLOT(emitActionDone()));// I don't get it, why is this needed
	}
}
//---------------------------------------------------------------------------------------------
void CdboActionHandler::lunchSlaveAction(){
	slaveAction->performTask(params);
}
//---------------------------------------------------------------------------------------------
void CdboActionHandler::slaveActionFailedToLaunch(){
	processFailedToLaunch();
	QTimer::singleShot(100, this, SLOT(emitActionDone()));// I don't get it, why is this needed
}
//---------------------------------------------------------------------------------------------
void CdboActionHandler::slaveActionCancelled(){
//kdDebug() << name() << " called slaveActionCancelled()>>>>>>>>>" << endl;

	slaveAction->setAutoDelete(true);
	slaveAction->showCanceledMess = false;
	processCancelledByUser();
}
//---------------------------------------------------------------------------------------------
void CdboActionHandler::killCurrentTask(){
//kdDebug() << name() << " called CdboActionHandler::killCurrentTask()>>>>>>>>>" << endl;
	if(slaveAction){
		slaveAction->setAutoDelete(true);
		slaveAction->killCurrentTask();
	}
	removeAllSlaves();
	slaveAction = 0;
	CdboAction::killCurrentTask();
	QTimer::singleShot(100, this, SLOT(emitActionDone()));// I don't get it, why is this needed
}
//---------------------------------------------------------------------------------------------
void CdboActionHandler::removeAllSlaves(){
	while(!slaveQueue.isEmpty()){
		CdboAction* action = slaveQueue.dequeue();
		if(action){
			slaveAction->setAutoDelete(true);
			action->emitActionDone();
		}
	}
	slaveQueue.clear();
}


