/***************************************************************************
                          CdboCDRom.h  -  description
                             -------------------
    begin                : Tue Sep 24 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sf.net

   ** Inspired by small CDROM handling class by Harald Hoyer **
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#ifndef CDBO_CDROM_H
#define CDBO_CDROM_H

#include <qobject.h>

#include "CdboConstants.h"


class CdboCDRom : public QObject  {
	Q_OBJECT
	
public:
	CdboCDRom(QString path, QObject* parent = 0, const char* name = 0);
	~CdboCDRom();
	
	bool isValid();
	bool isDiskOK();

	void startMonitoringState();
	void stopMonitoringState();
	bool isMonitoringState() { return (eTimeId != -1); }

	int monitorStateInterval() { return testInterval; }
	void setMonitorStateInterval(int i) {
		testInterval = i;
		startMonitoringState();
	}

	Cdbo::MediaStatus testMedia();
	Cdbo::MediaStatus currentMediaStatus() { return currMediaStatus; }
	Cdbo::MediaType currentMediaType() { return currMediaType; }
	
	QString devPath() { return path; }
	
	//void setDevName(QString devName) { name = devName; }
	//QString devName() { return name; }
	
	bool allowsEject() { return allowEject; }
	void setAllowEject(bool ae) { allowEject = ae; }

	bool isMounted();

	int getFD() { return cdromfd; }

	
public slots:
	void checkState();
	bool mount();
	bool umount();
	
	bool eject();
	void closeTray();
	
signals:
	void stateChanged(CdboCDRom&);

protected slots:
 	virtual void timerEvent(QTimerEvent *event);
  
protected:
	int open();
	bool isOpen();
	int close();
	bool hasMediaChanged();

private:
	QString path;
	//QString name;
	QString mountPath;
	
	int cdromfd;//id
	int testInterval;
	int eTimeId;
	
	Cdbo::MediaStatus currMediaStatus;
	Cdbo::MediaType currMediaType;
	
	bool allowEject;
};

#endif

