/***************************************************************************
                          CdboMainView.cpp  -  description
                             -------------------
    begin                : Sat Oct 19 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/ 

#include <qdir.h>
#include <qtabwidget.h>
#include <qwidgetstack.h>
#include <qtimer.h>

//#include <kapplication.h>
#include <kstandarddirs.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kmimetype.h>
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "CdboMainView.h"
#include "CdboMainView.moc"
#include "CdboShell.h"
#include "CdboViewPart.h"
#include "CdboPluginManager.h"



//=====================================
CdboMainView::CdboMainView(CdboShell *parent, const char *name) : QSplitter(parent, name){

	this->setOrientation(QSplitter::Vertical);
	active = 0;
	parts.setAutoDelete(true);
	config = 0;
	untitledCount = 0;
	((CdboShell* )parentWidget())->updateProgress(20);
	initView();
	
	readOptions();
}
//=====================================
CdboMainView::~CdboMainView(){
	parts.clear();
	if(config)
		delete config;
}
//=====================================
void CdboMainView::readOptions(){
	if(config)
		delete config;
	config = new KConfig("cdbakeovenrc");
	config->setGroup("General Options");
	setSizes(config->readIntListEntry("Sizes in CdboMainView"));
}
//=====================================
void CdboMainView::saveOptions(){
	if(config)
		delete config;
	config = new KConfig("cdbakeovenrc");
	config->setGroup("General Options");
	
	config->writeEntry("Sizes in CdboMainView", sizes());
	
	config->sync();
}
//=====================================
void CdboMainView::initView(){
	((CdboShell* )parentWidget())->updateProgress(30);
	((CdboShell* )parentWidget())->slotSetStatusBarText(i18n("Loading file view..."));
	
	topViewHolder = new QWidgetStack(this, "topViewHolder");
	
	((CdboShell* )parentWidget())->updateProgress(50);
	stackButtomViewHolder = new QWidgetStack(this, "stackButtomViewHolder");
	
	tabButtomViewHolder = new QTabWidget(stackButtomViewHolder, "tabButtomViewHolder");
	connect(tabButtomViewHolder, SIGNAL(currentChanged(QWidget *)), this , SLOT(currentVisibleChanged(QWidget *)));
	stackButtomViewHolder->addWidget(tabButtomViewHolder);
	
	infoBrws = new CdboHTMLPart(tabButtomViewHolder);
	stackButtomViewHolder->addWidget(infoBrws->widget());
	stackButtomViewHolder->raiseWidget(infoBrws->widget());
	
	QTimer::singleShot(1, this, SLOT(loadButtomMessage()));
	
	((CdboShell* )parentWidget())->updateProgress(60);
	connect(infoBrws, SIGNAL(execRequested(const QString &)), 
		CdboPluginManager::self() , SLOT(activateHelper(const QString &)));
	
}
//=====================================
void CdboMainView::loadButtomMessage(){
	QStringList empty;
	reloadButtomMessage(empty, empty);
}
//=====================================
void CdboMainView::reloadButtomMessage(const QStringList &types, const QStringList &infos){
	QString message = "<html><head><title></title></head><body>"
		"<font size=+1><center><i><b>%1</b></i></center><br><br>"
		"<table><tr><td>"
		"<img src=\"%1\"></td>"
		"<td><table><tr><td colspan=\"2\">%1<br><br>%1<br></td></tr>";
	QString available = i18n("No Plugins Loaded.");
	
	if(types.count() > 0){
		available = i18n("Currently Available Types:");
		QString infoRow = "<tr><td width=\"20%\">* <A HREF=\"exec:/%1\">%1</A> </td><td> %1 </td></tr>";
		for(uint i = 0; i < types.count(); i++)
			message += infoRow.arg(types[i]).arg(types[i]).arg(infos[i]);
	}
	message += "</table></td></tr></table></font></body></html>";
		
	QString greeting = i18n("Welcome to CD Bake Oven");
	QString description = i18n("Please begin by choosing the type for your new CD or continue existing Konqueror session.");
		
	QString image = locate("data", "cdbakeoven/toolbar/cdimage.png");
	
	
	infoBrws->begin();
	infoBrws->write(message.arg(greeting).arg(image).arg(description).arg(available));
	infoBrws->end();
}
//=====================================
void CdboMainView::currentVisibleChanged(QWidget *widget){
	widget->setFocus();
}
//=====================================
QWidget* CdboMainView::partsHolder(){
	return tabButtomViewHolder;
}
//=====================================
QWidget* CdboMainView::topHolder(){ 
	return topViewHolder; 
}
//=====================================
void CdboMainView::setTop(KParts::Part* topPart){
	top = topPart;
	topViewHolder->addWidget(topPart->widget());
	topViewHolder->raiseWidget(topPart->widget());
}
//=====================================
KParts::Part* CdboMainView::topPart() {
	return top;
}
//=====================================
void CdboMainView::addPart(KParts::Part* newPart){
	CdboViewPart *newPartCdbo = static_cast<CdboViewPart *>(newPart);
	if(newPartCdbo){
		if(newPartCdbo->caption() == ""){
			QString caption = i18n("Untitled %1").arg(QString::number(++untitledCount));
			newPartCdbo->setCaption(caption);
			newPartCdbo->setShortCaption(caption);
		}
		connect(newPartCdbo, SIGNAL(shortCaptionChanged(KParts::Part*)), this, SLOT(updateTab(KParts::Part*)));
		connect(newPartCdbo, SIGNAL(captionChanged(KParts::Part*)), this, SLOT(updateCaption(KParts::Part*)));
	}
	
	addWidget(newPart);
	
	active = newPart;
	active->widget()->show();
	parts.append(active);
	
}
//=====================================
void CdboMainView::addWidget(KParts::Part* newPart){
	if(parts.count() == 0){
		stackButtomViewHolder->addWidget(newPart->widget());
		stackButtomViewHolder->raiseWidget(newPart->widget());
		return;
	}
	if(parts.count() == 1){
		stackButtomViewHolder->removeWidget(active->widget());
		addWidgetToTabButtom(active);
	}
	
	addWidgetToTabButtom(newPart);
}
//=====================================
void CdboMainView::addWidgetToTabButtom(KParts::Part* newPart){
	tabButtomViewHolder->addTab(newPart->widget(), "");
	updateTab(newPart);
	tabButtomViewHolder->showPage(newPart->widget());
	
	if(tabButtomViewHolder != stackButtomViewHolder->visibleWidget())
		stackButtomViewHolder->raiseWidget(tabButtomViewHolder);
}
//=====================================
void CdboMainView::updateTab(KParts::Part* newPart){
	QString shortCaption = "", icon = "";
	CdboViewPart *newPartCdbo = static_cast<CdboViewPart *>(newPart);
	if(newPartCdbo){
		shortCaption = newPartCdbo->shortCaption();
		icon = newPartCdbo->icon();
	}
	else{
		KParts::ReadOnlyPart *newPartRO = static_cast<KParts::ReadOnlyPart *>(newPart);
		if(newPartRO){
			QFileInfo info(newPartRO->url().path());
			shortCaption = info.fileName();
			icon = KMimeType::iconForURL(newPartRO->url().path());
		}
	}
	
	if(icon == "")
		tabButtomViewHolder->changeTab(newPart->widget(), shortCaption);
	else
		tabButtomViewHolder->changeTab(newPart->widget(), SmallIcon(icon), shortCaption);
}   
//=====================================
void CdboMainView::updateCaption(KParts::Part *newPart){
	if(newPart != active)
		return;

	QString caption;
	bool modified = false;
	CdboViewPart *newPartCdbo = static_cast<CdboViewPart *>(newPart);
	if(newPartCdbo){
		caption = newPartCdbo->caption();
		modified = newPartCdbo->isModified();
	}
	else{
		KParts::ReadOnlyPart *newPartRO = static_cast<KParts::ReadOnlyPart *>(newPart);
		if(newPartRO){
			caption = newPartRO->url().prettyURL();
		}
		KParts::ReadWritePart *newPartRW = static_cast<KParts::ReadWritePart *>(newPart);
		if(newPartRW){
			modified = newPartRW->isModified();
		}
	}
	
	((CdboShell* )parentWidget())->setCaption(caption, modified);
}      
//=====================================
bool CdboMainView::slotCloseView(){
	QString url;
	return slotCloseView(url);
}
//=====================================
bool CdboMainView::slotCloseView(QString &lastUrl){
	if(active){
		if(1 < parts.count())
			tabButtomViewHolder->showPage(active->widget());
		
		KParts::ReadOnlyPart *activeRO = static_cast<KParts::ReadOnlyPart *>(active);
		if(activeRO){
			if(!activeRO->closeURL())
				return false;
			if(!activeRO->url().isEmpty())
				lastUrl = activeRO->url().path();
			else
				lastUrl = QString::null;
		}
		parts.remove(active);
		if(parts.count() == 0){
			active = 0;
			((CdboShell* )parentWidget())->actionCollection()->action( "file_close" )->setEnabled(false);
			stackButtomViewHolder->raiseWidget(infoBrws->widget());
			((CdboShell* )parentWidget())->newType(i18n("Not Selected"));
			top->widget()->setFocus();
		}
		else if(parts.count() == 1){
			QWidget *previouse = tabButtomViewHolder->currentPage();
			tabButtomViewHolder->removePage(previouse);
			stackButtomViewHolder->addWidget(previouse);
			stackButtomViewHolder->raiseWidget(previouse);
			previouse->setFocus();
		}
		else{
			tabButtomViewHolder->currentPage()->setFocus();
		}
	}
	return true;
}
//=====================================
bool CdboMainView::queryClose(){
	QStringList lastUrls;
	return queryClose(lastUrls);
}
//=====================================
bool CdboMainView::queryClose(QStringList &lastUrls){
	QString lastUrl;
	while(0 < parts.count()){
		active = parts.at(0);
		if(!slotCloseView(lastUrl)){
			return false;
		}
		if(!lastUrl.isNull())
			lastUrls.append(lastUrl);
	}
	return true;
}
//================================================================
void CdboMainView::updateSettings(){
	for(uint i = 0; i < parts.count(); i++){
		CdboViewPart *part = static_cast<CdboViewPart *>(parts.at(i));
		if(part)
			part->readOptions();
	}
}
//=====================================
void CdboMainView::activeChanged( KParts::Part *newActive ){
	if(top == newActive){
		((CdboShell* )parentWidget())->actionCollection()->action( "file_close" )->setEnabled(false);
		updateCaption(active);
		return;
	}
	CdboViewPart *newActiveCdbo = static_cast<CdboViewPart *>(newActive);
	if(newActiveCdbo){
		active->disconnect(((CdboShell* )parentWidget()));
		top->disconnect(SIGNAL(addToCD(KURL::List&)));
		((CdboShell* )parentWidget())->newType(newActiveCdbo->type());
		((CdboShell* )parentWidget())->slotSetStatusBarText(newActiveCdbo->lastStatus());
		connect(newActiveCdbo, SIGNAL(haveStatus(const QString &)),((CdboShell* )parentWidget()) , SLOT(slotSetStatusBarText(const QString &)));
		connect(top, SIGNAL(addToCD(KURL::List&)), newActiveCdbo, SLOT(addItems(KURL::List &)));
	}
	else{
		((CdboShell* )parentWidget())->newType(i18n("Preview"));
	}
	
	active = newActive;
	updateCaption(newActive);
	((CdboShell* )parentWidget())->actionCollection()->action( "file_close" )->setEnabled(true);
}



//=====================================
//=====================================
//=====================================





CdboHTMLPart::CdboHTMLPart(QWidget *parentWidget, const char *widgetname, 
		QObject *parent, const char *name, GUIProfile prof) : 
		KHTMLPart(parentWidget, widgetname, parent, name, prof){

}
//=====================================
CdboHTMLPart::~CdboHTMLPart(){
}
//=====================================
void CdboHTMLPart::urlSelected( const QString &url, int button, int state, const QString &target, KParts::URLArgs _args ){
	KURL u( url );
	if (u.protocol() == "exec"){
		execRequested(url.mid(6));
		return;
	}

	KHTMLPart::urlSelected( url, button, state, target, _args );
}
