/***************************************************************************
                          cdboplugins.h  -  description
                             -------------------
    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdir.h>

#include <kmessagebox.h>
#include <kgenericfactory.h>
#include <kurl.h>
#include <kshortcut.h>
#include <kprocess.h>
#include <kdebug.h>
#include <krun.h>
#include <kiconloader.h>
#include <kstdaction.h>
#include <kinstance.h>
#include <kpopupmenu.h>

#include "cdboplugins.h"
#include "cdboplugins.moc"

#include "CdboEraseCDDialog.h"
#include "CdboCopyCDDialog.h"
#include "CdboDataCDDialog.h"
#include "CdboAudioCDDialog.h"
#include "CdboRipAudioDialog.h"
#include "CdboSelectDriveDialog.h"
#include "CdboComOutDialog.h"
#include "CdboPluginManager.h"

K_EXPORT_COMPONENT_FACTORY( cdboplugins, KGenericFactory<CdboPlugins> );

CdboPlugins::CdboPlugins(QObject* parent, const char* name, const QStringList& )
	:KParts::Plugin(parent, name){

	new KAction( i18n( "CD-Recording Setup Wizard" ), "wizard", KShortcut::null(), this,
		 SLOT(launchWizardSlot()), actionCollection(), "show_wizard");

	
	new KAction(i18n("CD-R(W) Information"), 0, this, 
		SLOT(loadCDInfoSlot()), actionCollection(), "loadCDInfoAction");
		
	new KAction(i18n("Device Information"), 0,this,
		SLOT(loadDeviceInfoSlot()), actionCollection(), "loadDeviceInfoAction");

	new KAction(i18n("Unlock device"), 0,this,
		SLOT(unlockCDSlot()), actionCollection(), "unlockCDAction");

	new KAction(i18n("Copy CD"), UserIcon("cdcopySm"), KShortcut(CTRL+Key_C), this,
		SLOT(popupCdboCopyDialog()), actionCollection(), "showCopyDialogAction");

	KStdAction::preferences(this, SLOT(popupSettingsDialog()), actionCollection(),
						"options_configure");

	new KAction(i18n("Erase CD-RW"), QIconSet( UserIcon("eraseSm"), UserIcon("erase")),  KShortcut(CTRL+Key_E), this,
		SLOT(popupCdboEraseDialog()), actionCollection(), "showEraseDialogAction");

	new KAction(i18n("Fixate Unfixated CD-R(W)"), KShortcut(CTRL+Key_F), this,
		SLOT(popupCdboFixateCdDialog()), actionCollection(), "showFixateDialogAction");

	new KAction(i18n("Rip Audio CD"),QIconSet( SmallIcon("cdaudio_unmount"), BarIcon("cdaudio_unmount")),
		KShortcut(CTRL+Key_R), this, SLOT(popupCdboRipAudioDialog()), actionCollection(), "showRipAudioDialogAction");
		
	newcdMenuAction = new KActionMenu( i18n( "&Konqueror Session" ), "cdtrack", actionCollection(), "newcd_menu");

	connect(newcdMenuAction->popupMenu(), SIGNAL(activated(int)), this, SLOT(slotLoadNewcd(int)));
		
	registerExt();
	newcdMenuAction->popupMenu()->insertItem( "newcd:/New Data CD");
	//plugNewcdAction();

	checkFirstRun();
}
//===========================================
QWidget* CdboPlugins::parentWidget() const{
   // Don't use dynamic_cast in a plugin
	if ( !parent() || !parent()->inherits ( "QWidget" ) ){
		return 0L;
	}
	return static_cast<QWidget*>( parent() );
}
//================================
void CdboPlugins::plugNewcdAction() {
	unplugActionList( "newcd_menu" );
	QPtrList<KAction> lst;
	lst.append( newcdMenuAction );
	plugActionList( "newcd_menu", lst );
}
//==========================================
void CdboPlugins::slotLoadNewcd(int pos){
	QString newcdUrl = newcdMenuAction->popupMenu()->text(pos);
	KURL url;
	url.setPath(newcdUrl);
	CdboPluginManager::self()->loadPart("libcdbonewcdview", "CdboNewcdViewFactory", url, false, "cdtrack", i18n("Konqueror Session"));
}
//==========================================
void CdboPlugins::registerExt(){
	QStringList extLst;
	extLst << "iso" << "ISO" << "raw" << "RAW" << "toc" << "TOC";
	
	QStringList prettyExtLst;
	prettyExtLst << "iso" << "raw" << "toc";
	
	CdboPluginManager::self()->registerExt(this, extLst, prettyExtLst);
}
//================================
void CdboPlugins::openURL(const KURL& url){
	QFileInfo info(url.path());
	if(info.extension(false).lower() == "toc"){
		CdboAudioCDDialog *ncd = new CdboAudioCDDialog(parentWidget(),"newCdD");
		ncd->setTocFile(url.path());
		ncd->exec();
		delete ncd;
	}
	else{
		CdboDataCDDialog *ncd = new CdboDataCDDialog(parentWidget(),"newCdD");
		ncd->setISOImage(url.path());
		ncd->exec();
		delete ncd;
	}
}
//==========================================
void CdboPlugins::loadCDInfoSlot(){
	CdboSelectDriveDialog *driveInfoDlg = new CdboSelectDriveDialog(parentWidget(), "propDlg" );
	if(!driveInfoDlg->isReady())
		KMessageBox::sorry(parentWidget(), i18n("No usable devices found."));
	else{
		CdboComOutDialog *cod = new CdboComOutDialog(parentWidget(), "cod");
		cod->setCaption(i18n("CD-R(W) Detailed Info - CD Bake Oven"));
		connect( driveInfoDlg, SIGNAL( loadScsi(QString) ), cod , SLOT( getCDDetailes(QString) ) );
		driveInfoDlg->exec();
	}
}
//==========================================
void CdboPlugins::unlockCDSlot(){
	CdboSelectDriveDialog *driveInfoDlg = new CdboSelectDriveDialog(parentWidget(), "propDlg" );
	if(!driveInfoDlg->isReady())
		KMessageBox::sorry(parentWidget(), i18n("No usable devices found."));
	else{
		CdboComOutDialog *cod = new CdboComOutDialog(parentWidget(), "cod");
		cod->setCaption(i18n("Unlock CD-R(W) - CD Bake Oven"));
		connect( driveInfoDlg, SIGNAL( loadScsi(QString) ), cod , SLOT( unlockCD(QString) ) );
		driveInfoDlg->exec();
	}
}
//==========================================
void CdboPlugins::loadDeviceInfoSlot(){
	CdboSelectDriveDialog *driveInfoDlg = new CdboSelectDriveDialog(parentWidget(), "propDlg" );
	if(!driveInfoDlg->isReady())
		KMessageBox::sorry(parentWidget(), i18n("No usable devices found."));
	else{
		CdboComOutDialog *cod = new CdboComOutDialog(parentWidget(), "cod");
		cod->setCaption(i18n("Device Detailed Info - CD Bake Oven"));
		connect( driveInfoDlg, SIGNAL( loadScsi(QString) ), cod , SLOT( getScsiDetailes(QString) ) );
		driveInfoDlg->exec();
	}
}
//===========================================
void CdboPlugins::popupSettingsDialog(){
	launchCommand("kcmshell cdbodefaultopt cdbocdromdevices kcmdf cdboburnopt cdboisoopt cdboaudioopt audiocd");
}
//===========================================
void CdboPlugins::popupCdboRipAudioDialog(){
	CdboRipAudioDialog *rad = new CdboRipAudioDialog(parentWidget(),"rippAudioD");
	rad->exec();
	delete rad;
}
//===========================================
void CdboPlugins::popupCdboEraseDialog(){
	CdboEraseCDDialog *ed = new CdboEraseCDDialog(parentWidget(),"eraseD");
	ed->exec();
	delete ed;
}
//===========================================
void CdboPlugins::popupCdboFixateCdDialog(){
	CdboFixateCDDialog *ed = new CdboFixateCDDialog(parentWidget(),"fixateD");
	ed->exec();
	delete ed;
}
//===========================================
void CdboPlugins::popupCdboCopyDialog(){
	CdboCopyCDDialog *ccd = new CdboCopyCDDialog(parentWidget(),"copyD");
	ccd->exec();
	delete ccd;
}
//===========================================
void CdboPlugins::checkFirstRun(){
	KConfig *config = new KConfig("cdbakeoven_wizard");
	config->setGroup("General Options");
	if(!config->readBoolEntry("Launched First Time Wizard", false)){
		launchWizardSlot();
		config->writeEntry("Launched First Time Wizard", true);
		config->sync();
	}
	delete config;

}
//===========================================
void CdboPlugins::launchWizardSlot(){
	launchCommand("cdbosetuptool --launchWizard");
}
//===========================================
void CdboPlugins::launchCommand(const QString& command){
	KShellProcess *cdboProc = new KShellProcess();
	*cdboProc << command;
	kdDebug() << "Ready to launch " << command << endl;
	connect(cdboProc, SIGNAL(processExited(KProcess*)), this, SLOT(kcmshellDone(KProcess*)));
	if(!cdboProc->start(KProcess::NotifyOnExit)){
		kdWarning() << "Failed to launch " << command << endl;
		delete cdboProc;
	}
}
//===========================================
void CdboPlugins::kcmshellDone(KProcess *proc){
	delete proc;
}

