/***************************************************************************
                          CdboAudioViewPart..cpp  -  description
                             -------------------
    begin                : Sun Nov 18 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/ 


#include <qevent.h>
#include <qlayout.h>

#include <kaboutdata.h>
#include <kaction.h>
#include <kinstance.h>
#include <klocale.h>
#include <kxmlguifactory.h>
#include <kglobal.h>
#include <kdebug.h>
#include <kurldrag.h>
#include <kconfig.h>
#include <kio/netaccess.h>

#include "CdboAudioViewPart.h"
#include "CdboAudioViewPart.moc"

#include "CdboAudioEstimate.h"
#include "CdboPlayerPnl.h"
#include "CdboAudioListView.h"
#include "CdboViewWidgetBase.h"
#include "CdboAudioCDDialog.h"

#include "CdboVersion.h"


K_EXPORT_COMPONENT_FACTORY( libcdboaudioview, CdboAudioViewFactory );

//===================================
CdboAudioViewPart::CdboAudioViewPart(QWidget *parentWidget, const char *widgetName,
                            QObject *parent, const char *name, const QStringList& list)
	: CdboViewPart(parentWidget, widgetName, parent, name, list){
	
	settingTypes = "cdboburnopt cdbodefaultopt";
	
	setInstance( CdboAudioViewFactory::instance() );
	new CdboAudioViewBrowserExtension( this );
	
	estimate = new CdboAudioEstimate(viewBase()->getEstimateHolder(), "estimate");
	viewBase()->plugEstimate(estimate);
	
	
	QWidget* holder = new QWidget(viewBase()->getListViewHolder(), "holder");
	QVBoxLayout* layout = new QVBoxLayout( holder );
	layout->setSpacing( 0 );
	layout->setMargin( 0 );
		 
	listView = new CdboAudioListView(holder, "listView");
	listView->setEstimate(estimate);
	
	layout->addWidget( listView );

	player = new CdboPlayerPnl( holder, "CdboAudioViewPart Player" );
	player->setMinimumSize( QSize( 0, 27 ) );
	layout->addWidget( player );
	
	viewBase()->plugView(holder);
	viewBase()->setFocusProxy(listView);
	
	
	connect(  listView, SIGNAL(dropped (QDropEvent *, QListViewItem *)),
		this, SLOT(dropOcurred(QDropEvent *, QListViewItem *)));
	connect(  listView, SIGNAL(addItems(KURL::List &)),
		this, SLOT(addItems(KURL::List &)));
	connect(listView, SIGNAL(limitReached()), 
		this, SLOT(emitLimitReached()));
	connect(listView, SIGNAL(haveStatus(const QString &)), 
		this, SLOT(emitStatus(const QString &)));
	
	connect(listView, SIGNAL(processStarted()), 
		viewBase(), SLOT(setWaitCursor()));
	connect(listView, SIGNAL(processDone()), 
		viewBase(), SLOT(resetCursor()));
	connect(listView, SIGNAL(processDone()), 
		this, SLOT(resetStatus()));
	connect(  listView, SIGNAL(modified()),
						this, SLOT(setModified()));
						
	connect(  listView, SIGNAL(selectedURL(const KURL&)),
			player, SLOT(playURL(const KURL&)));
	connect(  listView, SIGNAL(selectedURLs(KURL::List &)),
			player, SLOT(playURLs(KURL::List &)));
	connect(  player, SIGNAL(emptyPlayRequested()),
			listView, SLOT(previewSlot()));
		
	setupActions();
	readOptions();
	
	setXMLFile( "cdboaudioviewpartui.rc" );

}
//===================================
CdboAudioViewPart::~CdboAudioViewPart(){
	saveOptions();
}
//===================================
void CdboAudioViewPart::setupActions(){
	player->setupActions(actionCollection());
	listView->plugMenuAction(actionCollection()->action("newCDProperties" ));
	listView->plugMenuAction(actionCollection()->action("burnThisCD" ));
	listView->plugMenuAction(actionCollection()->action("separator" ));
	listView->plugMenuAction(actionCollection()->action("player_menu" ));
	
	actionCollection()->addDocCollection(listView->getActionCollection());
}
//===================================
void CdboAudioViewPart::readOptions(){
	estimate->readOptions();
	player->readOptions();
	listView->updateSettings();
}
//===================================
void CdboAudioViewPart::saveOptions(){
	estimate->saveOptions();
	player->saveOptions();
}
//===================================
void CdboAudioViewPart::createCD(){
	CdboAudioCDDialog *ncd = new CdboAudioCDDialog(listView->getFileList(), viewBase(), "ncd" ) ;
	ncd->exec();
	delete ncd;
	listView->updateSettings();
}
//=================================
bool CdboAudioViewPart::openFile(){
	listView->updateSettings();
	KConfig *file = new KConfig(m_file);
	file->setGroup("CD Bake Oven - CD File List");
	
	QStringList fList = file->readListEntry("Files List");
	delete file;
	if(!fList.isEmpty()){
		KURL::List urls( fList );
		addItems(urls);
	}
	setModified(false);
	return true;
}
//=================================
void CdboAudioViewPart::saveToFile(KConfig *file){
	//QString name = top->text(0).right(top->text(0).length() - top->text(0).findRev("/", -2) - 1);
	//file->writeEntry("Name of CD", name);
	QStringList *fList = listView->getFileList();
	file->writeEntry("Files List", *fList);
}
//=================================
bool CdboAudioViewPart::isEmpty(){
	return (listView->childCount() == 0);
}
//=================================
void CdboAudioViewPart::dropOcurred(QDropEvent * e, QListViewItem *){
	e->accept();
	viewBase()->setFocus();
	KURL::List urls;
	KURLDrag::decode( e, urls );
	if(urls.count() > 0)
		addItems( urls );
}
//====================================
void CdboAudioViewPart::addItems(KURL::List &urls){
	for(int i = 0; i < (int)urls.count(); i++){
		if(!urls[i].isLocalFile()){//FIXME: handle this better
			localUrlsOnlyError(urls[i].prettyURL());
			urls.remove(urls[i]);
			i--;
		}
		else{
			if(!canAccept(urls[i])){
				break;
			}
		}
	}
}
//=====================================
bool CdboAudioViewPart::canAccept(const KURL &url) {
	bool stopHere;
	if(!listView->canAccept(url.path(), stopHere)){
		invalidPathError(url.path());
		return false;
	}
	else
		return !stopHere;
}
//===================================
KAboutData* CdboAudioViewPart::createAboutData(){
	return new KAboutData( "cdboaudioviewpart", I18N_NOOP("CD Bake Oven"),
		CDBO_VERSION, I18N_NOOP("Thank you for using CD Bake Oven - CD Burning facility for KDE"),
		KAboutData::License_GPL,
		I18N_NOOP("(c) 2001-2002, Alexy Podolsky"));
}

//===================================
//===================================
//===================================
//===================================
//===================================
CdboAudioViewBrowserExtension::CdboAudioViewBrowserExtension( CdboAudioViewPart *p )
		: KParts::BrowserExtension( p, "CdboAudioViewBrowserExtension" ){
	
	KGlobal::locale()->insertCatalogue("cdbakeoven");
	
}
//===================================
CdboAudioViewBrowserExtension::~CdboAudioViewBrowserExtension(){
}

