/***************************************************************************
                          CdboDataEstimate.cpp  -  description
                             -------------------
    begin                : Fri Jul 27 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qcombobox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlcdnumber.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include <klocale.h>
#include <kconfig.h>


#include "CdboDataEstimate.h"
#include "CdboDataEstimate.moc"

/* 
 *  Constructs a CdboDataEstimate which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
CdboDataEstimate::CdboDataEstimate( QWidget* parent,  const char* name, WFlags fl )
   : QWidget( parent, name, fl ), total(0), wasted(0), max(0), wastedMB(false), usedMB(false)
  {
    	
    	if ( !name )
		setName( "CdboDataEstimate" );
	
     setCaption( i18n( "Form1" ) );
     cdboDataEstimateLayout = new QGridLayout( this );
     cdboDataEstimateLayout->setSpacing( 2 );
     cdboDataEstimateLayout->setMargin( 4 );

     estFrm = new QGroupBox( this, "estFrm" );
     estFrm->setTitle( i18n( "Estimate" ) );
     estFrm->setColumnLayout(0, Qt::Vertical );
     estFrm->layout()->setSpacing( 0 );
     estFrm->layout()->setMargin( 0 );
     estFrmLayout = new QGridLayout( estFrm->layout() );
     estFrmLayout->setAlignment( Qt::AlignTop );
     estFrmLayout->setSpacing( 3 );
     estFrmLayout->setMargin( 9 );

     recalcEstBtn = new QPushButton( estFrm, "recalcEstBtn" );
    recalcEstBtn->setMinimumSize( QSize( 120, 30 ) );
    recalcEstBtn->setMaximumSize( QSize( 120, 30 ) );
    recalcEstBtn->setText( i18n( "&Recalculate" ) );
    QToolTip::add( recalcEstBtn, i18n( "Recalculate estimate." ) );
    estFrmLayout->addWidget( recalcEstBtn, 6, 1 );
    
    QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    estFrmLayout->addItem( spacer_3, 5, 1 );

    statsLbl = new QLabel(estFrm, "statsLbl" );
     statsLbl->setAlignment(AlignHCenter);

    estFrmLayout->addWidget( statsLbl, 3, 1 );
    
    stats2Lbl = new QLabel(estFrm, "stats2Lbl" );
    stats2Lbl->setAlignment(AlignHCenter);
    estFrmLayout->addWidget( stats2Lbl, 4, 1 );
	 
     capacityPnl = new QGroupBox( estFrm, "GroupBox5" );
     capacityPnl->setTitle( i18n( "CD Capacity" ) );
     capacityPnl->setColumnLayout(0, Qt::Vertical );
     capacityPnl->layout()->setSpacing( 0 );
     capacityPnl->layout()->setMargin( 0 );
     capacityPnlLayout = new QHBoxLayout( capacityPnl->layout() );
     capacityPnlLayout->setAlignment( Qt::AlignTop );
     capacityPnlLayout->setSpacing( 6 );
     capacityPnlLayout->setMargin( 11 );

     capacityCmb = new QComboBox( FALSE, capacityPnl, "capacityCmb" );
     capacityCmb->insertItem( i18n( "50 MB (5.5 min)" ) );
     capacityCmb->insertItem( i18n( "185 MB (21 min)" ) );
     capacityCmb->insertItem( i18n( "202 MB (23 min)" ) );
     
     capacityCmb->insertItem( i18n( "650 MB (74 min)" ) );
     capacityCmb->insertItem( i18n( "700 MB (80 min)" ) );
	  capacityCmb->insertItem( i18n( "790 MB (90 min)" ) );
     capacityCmb->insertItem( i18n( "850 MB (100 min)" ) );
     QToolTip::add(  capacityCmb, i18n( "Choose CD-R(W) capacity" ) );
     capacityPnlLayout->addWidget( capacityCmb );

     estFrmLayout->addMultiCellWidget( capacityPnl, 0, 0, 0, 2 );

     usedPnl = new QGroupBox( estFrm, "usedPnl" );
     usedPnl->setTitle( i18n( "Used" ) );
     usedPnl->setColumnLayout(0, Qt::Vertical );
     usedPnl->layout()->setSpacing( 0 );
     usedPnl->layout()->setMargin( 0 );
     usedPnlLayout = new QHBoxLayout( usedPnl->layout() );
     usedPnlLayout->setAlignment( Qt::AlignTop );
     usedPnlLayout->setSpacing( 6 );
     usedPnlLayout->setMargin( 11 );

     usedLCD = new QLCDNumber( usedPnl, "usedLCD" );
     usedLCD->setMargin( 0 );
     usedLCD->setNumDigits( 6 );
     usedLCD->setMode( QLCDNumber::DEC );
     usedLCD->setSegmentStyle( QLCDNumber::Filled );
     usedLCD->setProperty( "intValue", 0 );
     QToolTip::add(  usedLCD, i18n( "Total used space" ) );
     usedPnlLayout->addWidget( usedLCD, 1 );

     usedTypeCmb = new QComboBox( FALSE, usedPnl, "usedTypeCmb" );
     usedTypeCmb->insertItem( i18n( "MB" ) );
     usedTypeCmb->insertItem( i18n( "KB" ) );
     usedPnlLayout->addWidget( usedTypeCmb );
     estFrmLayout->addMultiCellWidget( usedPnl, 1, 1, 0, 2 );



     wastedPnl = new QGroupBox( estFrm, "wastedPnl" );
     wastedPnl->setTitle( i18n( "Wasted" ) );
     wastedPnl->setColumnLayout(0, Qt::Vertical );
     wastedPnl->layout()->setSpacing( 0 );
     wastedPnl->layout()->setMargin( 0 );
     wastedPnlLayout = new QHBoxLayout( wastedPnl->layout() );
     wastedPnlLayout->setAlignment( Qt::AlignTop );
     wastedPnlLayout->setSpacing( 6 );
     wastedPnlLayout->setMargin( 11 );

     wastedLCD = new QLCDNumber( wastedPnl, "wastedLCD" );
     wastedLCD->setMargin( 0 );
     wastedLCD->setNumDigits( 6 );
     wastedLCD->setMode( QLCDNumber::DEC );
     wastedLCD->setSegmentStyle( QLCDNumber::Filled );
     wastedLCD->setProperty( "intValue", 0 );
     QToolTip::add(  wastedLCD, i18n( "Total wasted space" ) );
     wastedPnlLayout->addWidget( wastedLCD, 1 );

     wastedTypeCmb = new QComboBox( FALSE, wastedPnl, "wastedTypeCmb" );
     wastedTypeCmb->insertItem( i18n( "MB" ) );
     wastedTypeCmb->insertItem( i18n( "KB" ) );
     wastedPnlLayout->addWidget( wastedTypeCmb );


     
     estFrmLayout->addMultiCellWidget( wastedPnl, 2, 2, 0, 2 );

     cdboDataEstimateLayout->addWidget( estFrm, 0, 0 );
    	
     readOptions();
     reset();
     showStats(false);
    updateStats(0, 0);
	///////////////////////////////////////

     // signals and slots connections
     //connect( recalcEstBtn, SIGNAL( clicked() ), this, SLOT( recalculate() ) );
     connect( wastedTypeCmb, SIGNAL( activated(int) ), this, SLOT( wastedTypeChangedSlot(int) ) );
     connect( usedTypeCmb, SIGNAL( activated(int) ), this, SLOT( usedTypeChangedSlot(int) ) );
     connect( capacityCmb, SIGNAL( activated(int) ), this, SLOT( capacityChangedSlot(int) ) );
}

CdboDataEstimate::~CdboDataEstimate(){
}
//---------------------------------------------------------------------------------------------
void CdboDataEstimate::readOptions(KConfig* config){
	bool defaultConfig = false;
	if(!config){
		config = new KConfig("cdbakeovenrc");
		defaultConfig = true;
	}
	config->setGroup("Estimate");
	int  iBuff = config->readNumEntry("Capacity Type", 3);
	capacityCmb->setCurrentItem(iBuff);
	capacityChangedSlot(iBuff);

	iBuff = config->readNumEntry("Used Type", 0);
	usedTypeCmb->setCurrentItem(iBuff);
	usedTypeChangedSlot(iBuff);

	iBuff = config->readNumEntry("Wasted Type", 0);
	wastedTypeCmb->setCurrentItem(iBuff);
	wastedTypeChangedSlot(iBuff);
	
	if(defaultConfig)
		delete config;
}
//-----------------------------------------------------------------------------------------------
void CdboDataEstimate::saveOptions(KConfig* config){
	bool defaultConfig = false;
	if(!config){
		config = new KConfig("cdbakeovenrc");
		defaultConfig = true;
	}
	config->setGroup("Estimate");		
	config->writeEntry("Capacity Type", capacityCmb->currentItem());
	config->writeEntry("Used Type", usedTypeCmb->currentItem());
	config->writeEntry("Wasted Type", wastedTypeCmb->currentItem());
	
	config->sync();
	if(defaultConfig)
		delete config;
}
//===================================
void CdboDataEstimate::capacityChangedSlot(int index){
	int buff = max;
	switch(index){
		case 0: max = 51200; break;
		case 1: max = 189440; break;
		case 2: max = 206848; break;
		case 3: max = 665600; break;
		case 4: max = 716800; break;
		case 5: max = 808960; break;
		case 6: max = 896000; break;
	}
	if(max != buff)
		emit recalculate();
}
//=======================================
bool CdboDataEstimate::canAdd(int buff){
	if( (buff + total)  > max)
		return false;
	return true;
}
//=======================================
bool CdboDataEstimate::add(int buff){
	if( !canAdd(buff))
		return false;
	
	total += buff;
	wasted -= buff;
	if(wasted < 0)
		wasted = 0;
	synchUp();
	return true;
}
//===============================
void CdboDataEstimate::subtract(int buff){
	if(buff == 0) return;
	total -= buff;
	if(total < 0)
		total = 0;
	wasted += buff;	
	synchUp();
}
//===============================
void CdboDataEstimate::synchUp(){
	QString num;
	if(wastedMB){	
		num.setNum( ((double)wasted) / 1024, 'f', 1);
		wastedLCD->display(num);
	}
	else
		wastedLCD->display(wasted);
		
	//if(total == 0) return;

	if(usedMB){	
		num.setNum( ((double)total) / 1024, 'f', 1);
		usedLCD->display(num);
	}
	else
		usedLCD->display(total);
		
}
//===============================
void CdboDataEstimate::reset(){
	total = 0;
	wasted = max;
	synchUp();
}
//===============================
void CdboDataEstimate::usedTypeChangedSlot(int index){
    switch(index){
    		case 0: usedMB = true; break;
    		case 1: usedMB = false; break;
   	 }
    synchUp();
}
//=======================================
void CdboDataEstimate::wastedTypeChangedSlot(int index){
	switch(index){
    		case 0: wastedMB = true; break;
    		case 1: wastedMB = false; break;
	 }
	synchUp();
}
//=======================================
void CdboDataEstimate::updateStats(int foldersTot, int filesTot){
    statsLbl->setText(i18n("%1 Items on list:").arg(QString::number(foldersTot+filesTot)));
    stats2Lbl->setText(i18n("%1 Folders, %2 Files").arg(QString::number(foldersTot)).arg(QString::number(filesTot)));
}
//=======================================
void CdboDataEstimate::showStats(bool show){
	if(show){
		statsLbl->show();
		stats2Lbl->show();
	}
	else{
		statsLbl->hide();
		stats2Lbl->hide();
	}
}



