/***************************************************************************
                          CdboDataFilesView.cpp  -  description
                             -------------------
    begin                : Sat Dec 1 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

// FIXME This class must be broken into independant handler and views in order to support multiple view types as in kdiroperator 

// KDE includes
#include <qdir.h>
#include <qstringlist.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qfileinfo.h>
#include <qheader.h>
#include <qdragobject.h>
#include <qcheckbox.h>
#include <qevent.h>
#include <qlist.h>

// KDE includes
#include <kurl.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kopenwith.h>
#include <kio/netaccess.h>
#include <kmessagebox.h>
#include <kmimetype.h>
#include <kcombobox.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kstdaccel.h>
#include <kdebug.h>

// CDBO includes
#include "CdboDataFilesView.h"
#include "CdboDataFilesView.moc"

#include "CdboDataFolder.h"

//===================================================
class BasicFolderItem : public QListViewItem  {
public:
	BasicFolderItem(QListView * parent);
	virtual QString key ( int column, bool ascending ) const;
	virtual void paintCell(QPainter *p, const QColorGroup &cg, int col, int width, int align);
	
	CdboDataFolder *i;

protected:
	virtual void okRename(int col);
};

//===================================================
class BasicFileItem : public QListViewItem  {
public:
	BasicFileItem(QListView * parent);
	virtual QString key ( int column, bool ascending ) const;
	virtual void paintCell(QPainter *p, const QColorGroup &cg, int col, int width, int align);
	
	CdboFileEntry *i;

protected:
	virtual void okRename(int col);
};

//===================================================
CdboDataFilesView::CdboDataFilesView( QWidget *parent, const char *name) : KListView(parent, name){

	addColumn(   i18n("Name")  );
	addColumn(   i18n("Size") );
	addColumn(  i18n( "Original Location") );

	setColumnAlignment(1, AlignRight);

	setColumnWidth(0,180);
	setColumnWidth(1,65);
	setColumnWidth(2,450);

	setDragEnabled(true);
	setAcceptDrops(true);
	setDropVisualizer(false);
	setFullWidth(true);
	//backStack.setAutoDelete(true);
	//forwardStack.setAutoDelete(true);

	setSelectionMode(QListView::Extended);

	parentFolder = 0;
	parentFolderPtr = 0;
	ignoreReload = false;
	reloadNav = true;
	config = 0;
	
//FIXME: SINGLE OR DOUBLE CLICK??

	connect(  this, SIGNAL(executed(QListViewItem *)),
						this, SLOT(slotSelectFile(QListViewItem *)));

	/*connect(  this, SIGNAL(clicked(QListViewItem *)),
						this, SLOT(slotSelectFileConst(QListViewItem *)));*/
	/*connect(  this, SIGNAL(selectionChanged(QListViewItem *)),
						this, SLOT(slotSelectFile(QListViewItem *)));*/
	
//FIXME: SINGLE OR DOUBLE CLICK??

	connect(	this, SIGNAL(returnPressed(QListViewItem *)),
						this, SLOT(slotSelectFile(QListViewItem *)));
	
	connect(	this, SIGNAL(markSelected()),
						this, SLOT(createSelectedLists()));
	connect( this, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
					 this, SLOT(popupMenuSlot(QListViewItem*, const QPoint&, int)));

	//setSorting(0,false);
	updateSettings();
}
//===================================================
CdboDataFilesView::~CdboDataFilesView(){
}
//===================================================
void CdboDataFilesView::updateSettings(){
	if(config)
		delete config;
	config = new KConfig("cdbakeovenrc");
	/*
	
	config->setGroup("Default Settings");
	showHidden = config->readBoolEntry("Show Hidden Files On Fake File List", true);
	
	*/
	viewport()->update();
}
//===================================
void CdboDataFilesView::setupActions(KActionCollection* ac){
	
	actionCollection = ac;
	actionMenu =new KActionMenu( i18n("CdboDataFilesView Menu"), actionCollection, "CdboDataFilesView menu");
	
	
	KAction* a = new KAction( i18n( "Delete" ), "editdelete", KShortcut(Key_Delete), this,
                  SLOT( remove() ), actionCollection, "delete_in_files_view" );
    
	actionMenu->insert(a);
	a = new KAction( i18n("Preview With..."), KShortcut::null(),
		this, SLOT( openWithSlot() ), actionCollection, "preview_with" );
	actionMenu->insert(a);
	
	//popupMenu -> insertItem( i18n("Rename File"),  this ,SLOT(startRenameSlot()), Key_F2);
	a = KStdAction::up(this, SLOT(goUp()), actionCollection, "up");
	a->setEnabled(false);
	a = KStdAction::forward(this, SLOT(forward()), actionCollection, "forward");
	a->setEnabled(false);
	a = KStdAction::back(this, SLOT(back()), actionCollection, "back");
	a->setEnabled(false);
	
	a = KStdAction::redisplay( this, SLOT(reloadListSlot()), actionCollection, "reload_listview" );
	a->setText( i18n("Reload") );
	a->setShortcut( KStdAccel::shortcut( KStdAccel::Reload ));
	
	actionMenu->insert(actionCollection->action("separator"));
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::popupMenuSlot(QListViewItem*/* item*/, const QPoint& point, int){
	
	createSelectedLists();
	
	if(selectedEntriesList.count() == 0)
		actionCollection->action("preview_with")->setEnabled(false);
	else
		actionCollection->action("preview_with")->setEnabled(true);
		
	if(selectedFoldersList.count() == 0 && selectedEntriesList.count() == 0){
		emit onFolder(parentFolder);
		actionCollection->action("delete_in_files_view")->setEnabled(false);
	}
	else
		actionCollection->action("delete_in_files_view")->setEnabled(true);
	
	emit menuRequested();
	actionMenu->popup( point);
}
//===================================
void CdboDataFilesView::plugMenuAction(KAction* action){
	actionMenu->insert(action);
}
//===================================
void CdboDataFilesView::prependMenuAction(KAction* action){
	actionMenu->insert(action, 0);
}
//===================================
void CdboDataFilesView::loadFiles( CdboDataFolder *item ){
	if(ignoreReload)
		return;
	clear();
//BEGIN: //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	CdboFileEntry *thisEntry;
	BasicFileItem *newItem;
	int ii = 1;

	for (thisEntry=item->getFirstCdboFileEntry(); thisEntry; thisEntry=item->getNextCdboFileEntry(), ii++){
		newItem = new BasicFileItem(this);
		newItem->setText(0, thisEntry->name);

		newItem->setPixmap(0,BarIcon(KMimeType::iconForURL(thisEntry->path), KIcon::SizeSmall ));
		newItem->setText(1, KIO::convertSize(thisEntry->size));
		newItem->setText(2, " " + thisEntry->path);
		if(thisEntry->immutable)
			newItem->setText(2, i18n(" Imported from previous session"));
		newItem->i = thisEntry;
	}
	BasicFolderItem *folderItem;
	for (CdboDataFolder *i = (CdboDataFolder *)item->firstChild(); ; i=(CdboDataFolder *)i->nextSibling()){
		if(!i)
			break;
		folderItem = new BasicFolderItem(this);
		folderItem->setText(0, i->getName());
		folderItem->setPixmap(0,i->getSmallIcon());
		folderItem->setText(1, KIO::convertSize(i->getSize()));
		folderItem->setText(2, "");
		folderItem->i = i;
	}
	
//END: //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	if(parentFolder != item){
		reloadNavActions( parentFolderPtr? parentFolderPtr->item(): 0, item );
	}
	parentFolderPtr = item->getHolder();
	parentFolder = item;
}
//===================================
void CdboDataFilesView::reloadNavActions(CdboDataFolder *prevParent, CdboDataFolder *newParent){
	if(reloadNav){
		if(prevParent)
			backStack.push(prevParent->getHolder());
		forwardStack.clear();
	}
	else
		reloadNav = true;
	
	if(newParent->isTop()) 
		actionCollection->action( "up" )->setEnabled(false);
	else
		actionCollection->action( "up" )->setEnabled(true);

	actionCollection->action( "forward" )->setEnabled( !forwardStack.isEmpty() );
	actionCollection->action( "back" )->setEnabled( !backStack.isEmpty() );
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::goUp(){
	emit selectFolder(parentFolder->parent());
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::back(){	
	CdboDataFolderHolder* folderPtr;
	do{
		if ( backStack.isEmpty() ){
			actionCollection->action( "back" )->setEnabled(false);
			return;
		}
		folderPtr = backStack.pop();
		if(!folderPtr->item())
			delete folderPtr;
		else
			break;
	} while(true);

	reloadNav = false;
	forwardStack.push(parentFolder->getHolder());
	emit selectFolder(folderPtr->item());
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::forward(){
	
	CdboDataFolderHolder* folderPtr;
	do{
		if ( forwardStack.isEmpty() ){
			actionCollection->action( "back" )->setEnabled(false);
			return;
		}
		folderPtr = forwardStack.pop();
		if(!folderPtr->item())
			delete folderPtr;
		else
			break;
	} while(true);
	
	reloadNav = false;
	backStack.push(parentFolder->getHolder());
	emit selectFolder(folderPtr->item());
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::setIgnoreReload(bool doIt){
	ignoreReload = doIt; 
	if(!doIt) 
		reloadListSlot();
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::reloadListSlot(){
	reloadNav = false;
	loadFiles(parentFolder);
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::slotSelectFile(QListViewItem * item){
	BasicFolderItem *folderItem = dynamic_cast<BasicFolderItem *>(item);
	if(!folderItem)
		return;
	else{
		parentFolder->setOpen(true);
		emit selectFolder(folderItem->i);
	}
}
//=============================================
void CdboDataFilesView:: currentModifiedSlot(){
	emit modified();
	reloadListSlot();
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::openWithSlot(){
	QListViewItem *fItem = currentItem();
  	if (!fItem) 
		return;

	clearSelection();
	setSelected(fItem, true);
	
	QString sp = fItem->text(2).stripWhiteSpace();

	QStringList list;
	list.append(sp);

	KFileOpenWithHandler *kfowh = new KFileOpenWithHandler();

	kfowh -> displayOpenWithDialog( KURL::List( list ) );
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::startRenameSlot(){
	QListViewItem *fItem = currentItem();
  	if (!fItem) return;

	fItem->startRename(0);
}
//---------------------------------------------------------------------------------
void CdboDataFilesView::remove(){
	ignoreReload = true;
	bool loopCanceled = false;
	
	createSelectedLists();
	CdboDataFoldersList*  folders = selectedFolders();
	if(folders->count() > 0){
		for (CdboDataFolder *entry=folders->first(); entry; entry=folders->next()){
			if(entry->isImmutable()){
				int result = KMessageBox::warningYesNo(this,
					i18n("You cannot remove this folder. "\
					"It was imported from the previous session"),
					i18n("Cannot remove %1").arg(entry->getName()),
					i18n("Skip"),
					i18n("Cancel"));
				switch(result){
					case KMessageBox::Yes:
						break;
					case KMessageBox::No:
						loopCanceled = true;
						break;
				}
			}
			else{
				parentFolder->subtractFromSize(entry->getSize());
				delete entry;
			}
			if(loopCanceled)
				break;
		}
	}
	
	CdboFileEntriesList* files = selectedFiles();
	if(files->count() > 0 && !loopCanceled){
		for (CdboFileEntry *entry=files->first(); entry; entry=files->next()){
			if(!parentFolder->removeEntry(entry)){
				int result = KMessageBox::warningYesNo(this,
					i18n("You cannot remove this item. "\
					"It was imported from the previous session"),
					i18n("Cannot Remove %1").arg(entry->name),
					i18n("Skip"),
					i18n("Cancel"));
				switch(result){
					case KMessageBox::Yes:
						break;
					case KMessageBox::No:
						loopCanceled = true;
						break;
				}
			}
			if(loopCanceled)
				break;
		}
	}

	ignoreReload = false;
	reloadListSlot();
	emit modified();
}
//=============================================
QDragObject *CdboDataFilesView::dragObject(){
	QListViewItem *item = currentItem();
	if ( !item || item->text(2) == i18n(" Imported from previous session"))
		return 0;

	BasicFolderItem *folderItem = dynamic_cast<BasicFolderItem *>(item);
	if(folderItem && folderItem->i->isImmutable())
		return 0;

	emit markSelected();
	
	QDragObject *d = new QTextDrag("This package came from your friendly CdboDataFilesView", (QWidget *)this);
	const QPixmap *p = currentItem()->pixmap(0);
	d->setPixmap(*p);
	
	return d;
}
//=============================================
void CdboDataFilesView::createSelectedLists(){

	QList<QListViewItem>selectedList = selectedItems();
	selectedFoldersList.clear();
	selectedEntriesList.clear();

	BasicFolderItem *folderItem;
	for(QListViewItem *item = selectedList.first(); item; item = selectedList.next()){
		folderItem = dynamic_cast<BasicFolderItem *>(item);
		if(!folderItem)
			selectedEntriesList.append(((BasicFileItem *)item)->i);
		else
			selectedFoldersList.append(folderItem->i);
	}
}
//===================================
bool CdboDataFilesView::acceptDrag (QDropEvent* event) const{
	if(QTextDrag::canDecode(event)){

		if(event->source() != 0){
			QString buff("QLineEdit");
			if(buff.contains((event->source())->className())){
				return false;
			}
		}
		if(!parentFolder)
			return true;
		QListViewItem *selected = itemAt(event->pos());
		if(lastSelected == selected && selected == selectedItem())
			return true;

		if(selected){
		//cerr << "itemAt() = " << iiii->text(0) << "\n";
			selected->listView()->clearSelection();
			((CdboDataFilesView *)selected->listView())->lastSelected = selected;
			BasicFolderItem *item = dynamic_cast<BasicFolderItem *>(selected);
			if(!item){
				parentFolder->listView()->ensureItemVisible(parentFolder);
				emit ((CdboDataFilesView *)selected->listView())->onFolder(parentFolder);
				if(event->source() == (CdboDataFilesView *)this){
					return false;
				}
			}
			else{
				selected->listView()->setSelected(selected, true);
				emit ((CdboDataFilesView *)selected->listView())->onFolder(item->i);
				parentFolder->listView()->ensureItemVisible(item->i);
			}
		}
		else{
			selected = firstChild();
			if(selected){
				selected->listView()->clearSelection();
				if(event->source() == (CdboDataFilesView *)this){
					return false;
				}
				
				parentFolder->listView()->ensureItemVisible(parentFolder);
				emit ((CdboDataFilesView *)selected->listView())->onFolder(parentFolder);
			}
		}
		return true;
	}
	else
		return false;
}
//---------------------------------------------------------------------------------
/*void CdboDataFilesView::checkShowHidden(){
	if(cnt){
		showHidden = cnt->getShowHiddenChk()->isChecked();
		loadDir(dirPath);
		////////////////////////////////////////////////////////////////////////////////////////
		config->setGroup("Default Settings");
		config->writeEntry("Show Hidden Files On Fake File List", showHidden);
	}
}*/



//===================================================
BasicFolderItem::BasicFolderItem(QListView * parent): QListViewItem(parent){
	//setRenameEnabled(0,true);
}
//===================================================
QString BasicFolderItem::key ( int column, bool ) const{
	if(column == 1){
		QString tmpString;
		tmpString.sprintf( "%08d", (int)i->getSize() );
		return tmpString;
	}
	return  QString(".1") + text(column);
}
//===================================================
void BasicFolderItem::paintCell(QPainter *p, const QColorGroup &cg, int col, int width, int align){
	KConfig* config = ((CdboDataFilesView*)listView())->config;
	config->setGroup("Default Settings");
	if(config->readBoolEntry("Disable Data Colors", false)){
		 QListViewItem::paintCell(p, cg, col, width, align);
		 return;
	}
	QColor color =
		(i->isImmutable() == true)? config->readColorEntry("Immutable Folder Color", &(cg.base())):
				config->readColorEntry("Regular Folder Color", &(cg.base()));
    QColorGroup mycg(cg);
    mycg.setBrush(QColorGroup::Base, color);

    QListViewItem::paintCell(p, mycg, col, width, align);
}
//===================================================
void BasicFolderItem::okRename(int col){
	QListViewItem::okRename(col);
	if(i->getName() == text(col))
		return;

	if(text(col).isEmpty()){
		KMessageBox::sorry(listView(),
			i18n("Name can not be empty. Please choose another name."));
		setText(col, i->getName());
		startRename(col);
		return;

	}
	if(text(col).find('/') != -1){
		KMessageBox::sorry(listView(),
			i18n("Entered name contains invalid character. Please choose another name."));
		setText(col, i->getName());
		startRename(col);
		return;

	}
	if(((CdboDataFilesView*)listView())->parentFolder->containsEntry(text(col))){
		KMessageBox::sorry(listView(),
			i18n("Item with the same name already exists in this folder. "\
			"Please choose another name."));
		setText(col, i->getName());
		startRename(col);
		return;
	}
	i->setName(text(col));
	((CdboDataFilesView*)listView())->emitModified();
}


//===================================================
BasicFileItem::BasicFileItem(QListView * parent): QListViewItem(parent){
	//setHeight(30);
	//setRenameEnabled(0,true);
}
//===================================================
QString BasicFileItem::key ( int column, bool ) const{
	if(column == 1){
		QString tmpString;
		tmpString.sprintf( "%08d", i->size );
		return tmpString;
	}
	return text(column);
}
//===================================================
void BasicFileItem::paintCell(QPainter *p, const QColorGroup &cg, int col, int width, int align){
	KConfig* config = ((CdboDataFilesView*)listView())->config;
	config->setGroup("Default Settings");
	if(config->readBoolEntry("Disable Data Colors", false)){
		 QListViewItem::paintCell(p, cg, col, width, align);
		 return;
	}
	QString type = "File";
	if(config->readBoolEntry("Use Same Data Colors", false))
		type = "Folder";
	QColor color =
		(i->immutable == true)? config->readColorEntry("Immutable " + type + " Color", &(cg.base())):
				config->readColorEntry("Regular " + type + " Color", &(cg.base()));
    QColorGroup mycg(cg);
    mycg.setBrush(QColorGroup::Base, color);

    QListViewItem::paintCell(p, mycg, col, width, align);
}
//===================================================
void BasicFileItem::okRename(int col){
	QListViewItem::okRename(col);

	if(i->name == text(col))
		return;

	if(text(col).isEmpty()){
		KMessageBox::sorry(listView(),
			i18n("Name can not be empty. Please choose another name."));
		setText(col, i->name);
		startRename(col);
		return;

	}
	if(text(col).find('/') != -1){
		KMessageBox::sorry(listView(),
			i18n("Entered name contains invalid character. Please choose another name."));
		setText(col, i->name);
		startRename(col);
		return;

	}
	if(((CdboDataFilesView*)listView())->parentFolder->containsEntry(text(col))){
		KMessageBox::sorry(listView(),
			i18n("Item with the same name already exists in this folder. "\
			"Please choose another name."));
		setText(col, i->name);
		startRename(col);
		return;
	}
	i->name = text(col);
	((CdboDataFilesView*)listView())->emitModified();
}


