/***************************************************************************
                          CdboDataFolder.h  -  description
                             -------------------
    begin                : Sun Dec 2 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/
#ifndef CDBODATAFOLDER_H
#define CDBODATAFOLDER_H

#include <klistview.h>

#include <qptrlist.h>
#include <qguardedptr.h>

class QPixmap;
class QProgressDialog;
class KConfig;
class CdboDataFoldersView;
class CdboDataFolderHolder;

struct CdboFileEntry{
	QString path;
	QString name;
	int size;
	bool immutable;
	int visibility;
};

typedef QList<CdboFileEntry> CdboFileEntriesList;

typedef QGuardedPtr<CdboDataFolderHolder> CdboDataFolderPtr;

class CdboDataFolder : public KListViewItem  {

public:
	enum FakePermissions {None_Visible=0, RR_Visible=1, Joi_Visible=2, HFS_Visible=4};
	typedef QString (CdboDataFolder::*permFunk)(QString);
public:
	CdboDataFolder(CdboDataFoldersView *parent, QString name = "");
	CdboDataFolder(CdboDataFolder *parent, QString name = "", bool immutable = false);
	CdboDataFolder(CdboDataFolder *fakeFolderItem, CdboDataFolder *parent, QProgressDialog *progress);
	CdboDataFolder(CdboDataFolder *fakeFolderItem, CdboDataFoldersView *parent, QProgressDialog *progress);
	~CdboDataFolder();

	void setOpen( bool );
	void setup();
	bool addEntry(QString path);
	void addEntry(QString path, QString name, unsigned long size, bool immutable = false);
	void addEntry(CdboFileEntry *entry);
	bool removeEntry(QString name);
	bool removeEntry(CdboFileEntry *thisEntry);
	bool readPath( QString path, bool includeHidden = true );
	void load(QProgressDialog *progress, QStringList *list);
	void load(QStringList *list);
	bool containsEntry(QString name);
	CdboFileEntry* getEntry(QString name);
	
	CdboDataFolderPtr getHolder();
	
	CdboDataFoldersView* parentView(){ return (CdboDataFoldersView*)listView(); }
	
	bool hasChild(QString name);
	CdboDataFolder* getChild(QString name);
	
	inline unsigned long getSize() { return size;}
	void addToSize(unsigned long size);
	void subtractFromSize(unsigned long size);
	
	inline bool isTop() { return top; }
	inline bool isImmutable() { return immutable; }
	
	inline CdboFileEntry* getFirstCdboFileEntry(){ return entries.first(); }
	inline CdboFileEntry* getNextCdboFileEntry(){ return entries.next(); }
	inline CdboFileEntry* getLastCdboFileEntry(){ return entries.last(); }
	inline CdboFileEntry* getPrevCdboFileEntry(){ return entries.prev(); }
	inline int totalEntries(){ return entries.count(); }
	
	inline void setName(QString newName) { setText( 0, newName ); }
	inline QString getName() { return text(0);}
	
	QPixmap getIcon();
	QPixmap getSmallIcon();
	
	QString getPathToTop();
	void generateMappings(QProgressDialog *progress, QTextStream &pathSpec, QTextStream &hideHFS, QTextStream &hideJoi, QTextStream &hideRR);
			
	void persist(QProgressDialog *progress, KConfig *file);

	virtual void paintCell(QPainter *p, const QColorGroup &cg,
			   int col, int width, int align);

protected:
	virtual void okRename(int col);
	void init();
	
private:
	bool opened;
	bool includeHidden;
	
	CdboFileEntriesList entries;


	unsigned long size;
	bool top;
	bool immutable;
	
	CdboDataFolderPtr holder;
};

typedef QList<CdboDataFolder> CdboDataFoldersList;

class CdboDataFolderHolder : public QObject {
public:
	CdboDataFolderHolder( QObject* parent = 0, const char* name = 0 ):QObject(parent, name){ m_item=0; }
	
	void setItem(CdboDataFolder* item){ m_item = item; }
	void itemGone() {m_item = 0;}
	
	CdboDataFolder* item() { return m_item; }

private:
	CdboDataFolder* m_item;

};

#endif
