/***************************************************************************
                          CdboDataFoldersView.cpp  -  description
                             -------------------
    begin                : Sat Dec 1 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdir.h>
#include <qheader.h>
#include <qdragobject.h>
#include <qprogressdialog.h>
#include <qregexp.h>
#include <qpushbutton.h>

#include <klocale.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kconfig.h>
#include <klineeditdlg.h>
#include <kstdaction.h>
#include <kstdaccel.h>
#include <kdebug.h>

#include "CdboDataFoldersView.h"
#include "CdboDataFoldersView.moc"

#include "CdboDataEstimate.h"

using namespace KIO;

CdboDataFoldersView::CdboDataFoldersView(QWidget *parent, const char *name):
 	KListView( parent, name ){

 	doListern = false;
	setDragEnabled(true);
	setAcceptDrops(true);
	setDropVisualizer(true);
	//setDropHighlighter(true);

	setRootIsDecorated( true );
	header()->hide();
	setSorting( 0 );
	setFrameStyle( Panel | Sunken );
	setLineWidth( 2 );
	addColumn( i18n("Name") );

	dicts.setAutoDelete(true);
	jobs.setAutoDelete(false);
	top = 0;
	config = 0;
	totalFolders = 0;
	totalFiles = 0;
	
//FIXME: SINGLE OR DOUBLE CLICK??

	connect(  this, SIGNAL(executed(QListViewItem *)),
						this, SLOT(slotSelectFile(QListViewItem *)));

	/*connect(  this, SIGNAL(clicked(QListViewItem *)),
						this, SLOT(slotSelectFileConst(QListViewItem *)));*/
	/*connect(  this, SIGNAL(selectionChanged(QListViewItem *)),
						this, SLOT(slotSelectFile(QListViewItem *)));*/
	
//FIXME: SINGLE OR DOUBLE CLICK??
	
	connect(	this, SIGNAL(returnPressed(QListViewItem *)),
						this, SLOT(slotSelectFile(QListViewItem *)));

	connect( this, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
					 this, SLOT(slotMenu(QListViewItem*, const QPoint&, int)));


	updateSettings();
	parentForDrop = top;
}
//==========================================
CdboDataFoldersView::~CdboDataFoldersView(){
}
//==========================================
void CdboDataFoldersView::setupActions(KActionCollection* ac){
	
	actionCollection = ac;

	actionMenu =new KActionMenu( i18n("CdboDataFoldersView Menu"), actionCollection, "CdboDataFoldersView menu");
	
	KAction* a = new KAction( i18n("New Directory..."), KShortcut::null(),
		this, SLOT( mkdir() ), actionCollection, "mkdir" );
	a->setIcon( QString::fromLatin1("folder_new") );
	actionMenu->insert(a);
	
	a = new KAction( i18n( "Delete" ), "editdelete", KShortcut::null(), this,
                  SLOT( remove() ), actionCollection, "delete_in_folders_view" );
     
	actionMenu->insert(a);
	
	actionMenu->insert(actionCollection->action("separator"));
		
	
	new KAction( i18n("Remove All"), KShortcut::null(), this, SLOT( removeAll() ), 
		actionCollection, "delete_all" );
		
	KAction* reloadAction = KStdAction::redisplay( this, SLOT(resetSize()), actionCollection, "reload" );
	reloadAction->setText( i18n("Reload") );
	reloadAction->setShortcut( KStdAccel::shortcut( KStdAccel::Reload ));
	
	new KAction(i18n("Rename"), KShortcut(Key_F2),
		this, SLOT(startRenameSlot()), actionCollection, "stop_load_action");
	
	new KAction(i18n("Stop"), "stop", KShortcut(Key_Escape),
		this, SLOT(cancelOperation()), actionCollection, "stop_load_action");
	actionCollection->action( "stop_load_action" )->setEnabled(false);
	
}
//==========================================
void CdboDataFoldersView::slotMenu(QListViewItem */*item*/, const QPoint &point, int){
	if(top->childCount() == 0 || !selectedItem() || selectedItem() == top)
		actionCollection->action("delete_in_folders_view")->setEnabled(false);
	else
		actionCollection->action("delete_in_folders_view")->setEnabled(true);
		
	emit menuRequested();
	actionMenu->popup( point);

}
//==========================================
void CdboDataFoldersView::plugMenuAction(KAction* action){
	actionMenu->insert(action);
}
//==========================================
void CdboDataFoldersView::updateSettings(){
	//config->sync();
	if(config)
 		delete config;
	config = new KConfig("cdbakeovenrc");
	config->setGroup("ISO Settings");
	QString fn = config->readEntry("Custom ISO Name", "cdboImage_<DATE>.iso");
	config->setGroup("Default Settings");
	if(!top)
		top = new CdboDataFolder(this, fn);
	top->setPixmap(0,BarIcon("cdrom_unmount", KIcon::SizeSmall ));
	top->setText(0, fn);
	viewport()->update();
	
	
	/*
	config->setGroup("Default Settings");
	showHidden = config->readBoolEntry("Show Hidden Items in Fake Folders List", true);///what ta hell??
	
	*/
}
//====================================
void CdboDataFoldersView::setEstimate(CdboDataEstimate *e) {
	if(!e) return;

	estimate = e;
	connect(estimate->getRecalculateBtn(), SIGNAL(clicked ()),
						this, SLOT(resetSize()));
	connect(estimate, SIGNAL(recalculate ()),
						this, SLOT(resetSize()));

}
//=====================================
QString CdboDataFoldersView::getPathToTop(CdboDataFolder *item){
	if(item == top){
		return "/";
	}

	CdboDataFolder *tItem = item;
	QString path = item->getName() + "/";

	do {
		tItem = (CdboDataFolder *)tItem->parent();
		if(tItem == top){
			return "/" + path;
		}
		path = tItem->getName() + "/" + path;
	}while(tItem != 0);
	return path;
}
//=====================================
bool CdboDataFoldersView::isParentOf(QListViewItem *i1, QListViewItem *i2){
	if(i2 == top)
		return false;

	QListViewItem *parentOf_i2 = i2->parent();
	while(parentOf_i2 != top) {
		if(parentOf_i2 == i1){
			return true;
		}
		parentOf_i2 = parentOf_i2->parent();
	}

	return false;
}
//=====================================
void CdboDataFoldersView::slotSelectFile(QListViewItem *item){
	CdboDataFolder *ftf = dynamic_cast<CdboDataFolder *>(item);
	if (!ftf){
		return;
	}
	ftf->setOpen(!ftf->isOpen());

	emit folderClicked(ftf);///////
	
	parentForDrop = ftf;
	setSelected(item, true);
	ensureItemVisible(item);
}
//=====================================
void CdboDataFoldersView::slotSelectFileConst(QListViewItem *item){
	CdboDataFolder *ftf = dynamic_cast<CdboDataFolder *>(item);
	if (!ftf)
		return;//ftf = top;

	emit folderClicked(ftf);
	parentForDrop = ftf;
	setSelected(item, true);
}
//============================================
bool CdboDataFoldersView::canFit(int size, QString fname){
	if(!estimate->add(size)){
		emit rejectedSize(fname);
		return false;
	}
	return true;
}
//==============================================//works??
void CdboDataFoldersView::slotShowHidden(){
	showHidden = !showHidden;
	QListViewItem *item;
	for(item = firstChild(); item; item = item->nextSibling()){
		CdboDataFolder * i = dynamic_cast<CdboDataFolder *>(item);
		if(i){
//	  		i->setShowHidden(showHidden);
//	  		i->reloadList();
			i->setOpen(false);
		}
	}
	config->setGroup("Default Settings");
	config->writeEntry("Show Hidden Items in Fake Folders List", showHidden);
}
//=====================================
void CdboDataFoldersView::setParentForDrop(CdboDataFolder *item){
	parentForDrop = item;
	setSelected(item, true);
}
//=====================================
void CdboDataFoldersView::currentModifiedSlot(){
	emit modified();
	emit folderClicked(parentForDrop);
}
//=====================================
void CdboDataFoldersView:: mkdir(){
	CdboDataFolder *ftf = dynamic_cast<CdboDataFolder *>(currentItem());
	if (!ftf){
		ftf = top;
		top->setSelected(true);
	}
	bool done = false, ok;
	QString name = "Directory";
	do{
		name = KLineEditDlg::getText(i18n("New Directory"), name, &ok, this);
		if(!ok)
			return;
		name = name.simplifyWhiteSpace();
		if(name.isEmpty()){
			int result = KMessageBox::warningYesNo(this,
					i18n("New folder name cannot be empty. "\
					"Would you like to choose another name?"),
					i18n("No Name"),
					KStdGuiItem::yes(),
					KStdGuiItem::cancel());

			switch(result){
				case KMessageBox::Yes:
					done = false;
					break;
				case KMessageBox::No:
					return;
			}
		}
		else if(ftf->containsEntry(name)){
			int result = KMessageBox::warningYesNo(this,
					i18n("An item with the same name already exists in this folder. "\
					"Would you like to choose another name?"),
					i18n("%1 Already Exists").arg(name),
					KStdGuiItem::yes(),
					KStdGuiItem::cancel());

			switch(result){
				case KMessageBox::Yes:
					done = false;
					break;
				case KMessageBox::No:
					return;
			}
		}
		else
			done = true;
	}
	while( !done );

	CdboDataFolder *newItem = new CdboDataFolder(ftf, name );
	ensureItemVisible(newItem);
    	//setSelected(newItem, true);

	currentModifiedSlot();
	estimate->updateStats(totalFolders - 1, totalFiles);
}
//=====================================
bool CdboDataFoldersView::remove(){
	CdboDataFolder *item = dynamic_cast<CdboDataFolder *>(currentItem());
	if (!item || item == top)
		return false;
	 if(item->isImmutable()){
		int result = KMessageBox::warningYesNo(this,
					i18n("You cannot remove this folder. "\
					"It was imported from the previous session"),
					i18n("Cannot Remove %1").arg(item->getName()),
					i18n("&Skip"),
					KStdGuiItem::cancel());

		switch(result){
			case KMessageBox::Yes:
					return true;
			case KMessageBox::No:
				return false;
		}
	}
	CdboDataFolder *v = (CdboDataFolder *)item->parent();
	v->subtractFromSize(item->getSize());
	delete item;
	resetSize();

	slotSelectFileConst(v);
	emit modified();
	return true;
}
//=====================================
void CdboDataFoldersView::removeAll(){
	clear();
	top = 0;
	updateSettings();
	parentForDrop = top;
	resetSize();

	currentModifiedSlot();
}
//=====================================
void CdboDataFoldersView::resetSize(){
	estimate->reset();

	estimate->updateStats(totalFolders - 1, totalFiles);
	if(top->getSize() > 0.0)
		estimate->add(top->getSize() /1024);
}
//=====================================FIXME: rename needed??
void CdboDataFoldersView::startRenameSlot(){
	QListViewItem *fItem = currentItem();
	if (!fItem) return;

	fItem->startRename(0);
}
//=====================================
void CdboDataFoldersView::showProgress(const QString& labelText, const QString& cancelButtonText, int totalSteps){
	progress = new QProgressDialog(labelText, cancelButtonText , totalSteps, this, "progress", true );
	progress->setCaption( i18n("Please wait..."));
	progress->setProgress(0);
}
//=====================================
void CdboDataFoldersView::updateProgress(int amount){
	progress->setProgress(amount);
}
//=====================================
void CdboDataFoldersView::hideProgress(){
	delete progress;
}
//=========================================================
void CdboDataFoldersView::contentsDragEnterEvent(QDragEnterEvent *e){
	KListView::contentsDragEnterEvent(e);
	doListern = true;
}
//=========================================================
void CdboDataFoldersView::contentsDragLeaveEvent(QDragLeaveEvent *e){
	KListView::contentsDragLeaveEvent(e);
	doListern = false;
}
//=========================================================
QRect CdboDataFoldersView::drawDropVisualizer (QPainter *p, QListViewItem *parent, QListViewItem *){
	if(doListern){
		parentForDrop = dynamic_cast<CdboDataFolder *> (parent);
		if(!parentForDrop)
			parentForDrop = top;
		setSelected(parentForDrop, true);
		ensureItemVisible(parentForDrop);
	}
	return KListView::drawDropVisualizer(p, parentForDrop, parentForDrop);
}
//============================================================
void CdboDataFoldersView::startDrag(){
	dragedItem = (CdboDataFolder *)currentItem();
	KListView::startDrag();
}
//=============================================
QDragObject *CdboDataFoldersView::dragObject(){
	CdboDataFolder *cItem = dynamic_cast<CdboDataFolder *> (currentItem());

	if ( !cItem || cItem == top || cItem->isImmutable())
		return 0;

	QDragObject *d = new QTextDrag("This package should go from CdboDataFoldersView to CdboDataFoldersView",
		(QWidget *)this);

	const QPixmap *p = cItem->pixmap(0);
	d->setPixmap(*p);
	return d;
}
//===================================
bool CdboDataFoldersView::acceptDrag (QDropEvent* event) const{
	if(QTextDrag::canDecode(event)){
		if(event->source() != 0){
			QString buff("QLineEdit");
			if(buff.contains((event->source())->className())){
				return false;
			}
		}
		return true;
	}
	else
		return false;
}
//=====================================
bool CdboDataFoldersView::canAccept(QString path, bool &stopHere) {
	stopHere = false;
	dragedItem = 0;

	QFileInfo d( path );
	if(!d.exists() || !d.isReadable()){
		return false;
	}
	if(d.isDir()){
		loadingSession = false;
		CdboDataFolder *newItem = new CdboDataFolder(parentForDrop, d.fileName() );
		addDir(newItem, path, i18n("Processing directory. Please wait..."));
		return true;
	}

	int size = (int)d.size();
	if(!canFit(size / 1024, d.fileName())){
		resetSize();
		stopHere = true;
	}
	else{
		parentForDrop->addEntry(path, d.fileName(), size);
		currentModifiedSlot();
		estimate->updateStats(totalFolders - 1, totalFiles);
	}
	return true;
}
//=====================================
void CdboDataFoldersView::importSession(QString pathToDrive) {
	QFileInfo d( pathToDrive );
	if(!d.exists() || !d.isReadable() || !d.isDir())
		return;

	loadingSession = true;
	addDir(top, pathToDrive, i18n("Loading session from CD. Please wait..."));
}
//=====================================
void CdboDataFoldersView::addDir(CdboDataFolder *parentItem, const QString &dirPath, const QString &status) {
	SourceAndPositionMap *dirAndPosMap = new SourceAndPositionMap();
	dirAndPosMap->insert(dirPath + "/", parentItem);
	dicts.insert(dirPath + "/", dirAndPosMap);

	ensureItemVisible(parentItem);
	emit haveStatus(status);
	emit processStarted();
	actionCollection->action( "stop_load_action" )->setEnabled(true);
	estimate->getRecalculateBtn()->setEnabled(false);

	KURL url(dirPath);
	KIO::ListJob * listJob = listRecursive( url, false);

	listJob->setName((dirPath + "/").utf8());
	jobs.append(listJob);

	connect( listJob, SIGNAL(entries( KIO::Job *, const KIO::UDSEntryList& )),
				SLOT( slotEntries( KIO::Job*, const KIO::UDSEntryList& )));

	connect(listJob, SIGNAL(result ( KIO::Job *)),
		this, SLOT(slotResult(KIO::Job *) ));
}
//=====================================
void CdboDataFoldersView::slotEntries( KIO::Job* job, const KIO::UDSEntryList & list ){ 
	QString jobName = QString::fromUtf8(job->name());
	SourceAndPositionMap *dirAndPosMap = dicts[jobName];
	KIO::UDSEntryListConstIterator it = list.begin();
	KIO::UDSEntryListConstIterator end = list.end();
	//QString url = "";
	bool noGood = false;
	bool dir = false;
	QString name;
	long size = 0;
	CdboDataFolder* newItem;
	CdboDataFolder* parItem;
	estimate->updateStats(totalFolders - 1, totalFiles);

	for (; it != end; ++it) {
		KIO::UDSEntry::ConstIterator it2 = (*it).begin();
		for( ; it2 != (*it).end(); it2++ ) {
			switch( (*it2).m_uds ) {
				case KIO::UDS_LINK_DEST:{///FIXME:: handle links
					noGood = true;
					break;
				}
				case KIO::UDS_FILE_TYPE:{
					if((*it2).m_long == 16384){
						dir = true;
					}
					else{
						dir = false;
					}
					break;
				}
				case KIO::UDS_NAME:{
					if((*it2).m_str == "." || (*it2).m_str == ".."){
						noGood = true;
						break;
					}
					name = (*it2).m_str;
					break;
				}
				case KIO::UDS_SIZE:{
					if(!dir){
						size = (*it2).m_long;
					}
					break;
				}
				default:
					break;
			}
			if(noGood)
				break;
		}
		if(!noGood){
			if(!dir){
				if(!canFit(size / 1024, jobName)){
					killProcess(job);
					return;
				}
				int k = name.findRev("/", -1);

				parItem = (* dirAndPosMap)[name.left(k)];
				if(!parItem){
					parItem = (* dirAndPosMap)[jobName];
				}

				parItem->addEntry(jobName + name, name.right(name.length() - k -1), size, loadingSession);

			}
			else{
				int k = name.findRev("/", -1);

				parItem = (* dirAndPosMap)[name.left(k)];
				if(!parItem){
					parItem = (* dirAndPosMap)[jobName];
				}
				newItem = new CdboDataFolder(parItem, name.right(name.length() - k -1), loadingSession);

				dirAndPosMap->insert(name, newItem);
				}
		}
		noGood = false;
	}
}
//=====================================
void CdboDataFoldersView::slotResult( KIO::Job * job){
	dicts.remove(QString::fromUtf8(job->name()));
	jobs.remove(job);
	currentModifiedSlot();
	
	if(jobs.isEmpty()){
		emit processDone();
		resetSize();
		actionCollection->action( "stop_load_action" )->setEnabled(false);
		estimate->getRecalculateBtn()->setEnabled(true);
	}
}
//=====================================
void CdboDataFoldersView::cancelOperation(){
	KIO::Job *jobToKill;
	while(!jobs.isEmpty()){
		jobToKill = jobs.at(0);
		killProcess(jobToKill);
	}
}
//=====================================
void CdboDataFoldersView::killProcess(KIO::Job *jobToKill){
	
	if(!jobToKill)
		return;
	QString jobName = jobToKill->name();
	jobToKill->kill();
	
	
	if(!loadingSession){
		SourceAndPositionMap *dirAndPosMap = dicts[jobName];
		CdboDataFolder *item = (* dirAndPosMap)[jobName];
		parentForDrop->subtractFromSize(item->getSize());
		delete item;
	}
	else
		removeAll();

	dicts.remove(jobName);
	
	jobs.remove(jobToKill);
	if(jobs.isEmpty()){
		emit processDone();
		actionCollection->action( "stop_load_action" )->setEnabled(false);
		estimate->getRecalculateBtn()->setEnabled(true);
	}
	resetSize();
	currentModifiedSlot();
}
//============================================
bool CdboDataFoldersView::copyRow(CdboDataFolder *item, CdboDataFolder *parent){
	CdboDataFolder *newFolder;
	for(CdboDataFolder *i=(CdboDataFolder *)item->firstChild(); ; i=(CdboDataFolder *)i->nextSibling()){
		if(!i)
			return true;

		if ( progress->wasCancelled() )
			return false;

		newFolder = new CdboDataFolder(i, parent, progress);

		if(i->childCount() > 0){
			if(!copyRow(i, newFolder))
				return false;
		}
	}
	estimate->updateStats(totalFolders - 1, totalFiles);
	return true;
}




//============================================
bool CdboDataFoldersView::generateMappingsString(){
	showProgress(i18n( "Generating file mappings, please wait..."), i18n("&Cancel"), (int)(top->getSize()/1000));
	progress->setProgress(1 / 1000);
	
	config->setGroup("Default Settings");
	QString baseName = config->readEntry("Tmp Dir", "/tmp/");
	baseName += config->readEntry("Custom ISO Name", "cdboImage_<DATE>.iso");
	
	if(baseName.contains("<DATE>")){// it will not be same name as image, but that's ok
		QDate today = QDate::currentDate();
		QTime now = QTime::currentTime();
		QString time = QString::number(today.month()) +
			"_" + QString::number(today.day()) +
			"_" + QString::number(now.hour()) +
			"_" + QString::number(now.minute()) +
			"_" + QString::number(now.second());
		baseName.replace(QRegExp("<DATE>"), time);
	}
	///////////////////////////////////////////////////////////////////////////////////
	pathspecName = baseName + ".pathspec";
	hidejoiName = baseName + ".hidejoi";
	hidehfsName = baseName + ".hidehfs";
	hiderrName = baseName + ".hiderr";
	
	QFile pathspec(pathspecName);
	if(pathspec.exists())
		pathspec.remove();
	if(!pathspec.open(IO_ReadWrite)){
		KMessageBox::error(this,
					i18n("Failed to create %1, please make verify your working directory settings.").arg(pathspecName));
		return false;
	}
	QTextStream pathSpec(&pathspec);
	///////////////////////////////////////////////////////////////////////////////////
	QFile hidejoi(hidejoiName);
	if(hidejoi.exists())
		hidejoi.remove();
	if(!hidejoi.open(IO_ReadWrite)){
		KMessageBox::error(this,
					i18n("Failed to create %1, please make verify your working directory settings.").arg(hidejoiName));
		return false;
	}
	QTextStream hideJoi(&hidejoi);
	///////////////////////////////////////////////////////////////////////////////////
	QFile hidehfs(hidehfsName);
	if(hidehfs.exists())
		hidehfs.remove();
	if(!hidehfs.open(IO_ReadWrite)){
		KMessageBox::error(this,
					i18n("Failed to create %1, please make verify your working directory settings.").arg(hidehfsName));
		return false;
	}
	QTextStream hideHFS(&hidehfs);
	///////////////////////////////////////////////////////////////////////////////////
	QFile hiderr(hiderrName);
	if(hiderr.exists())
		hiderr.remove();
	if(!hiderr.open(IO_ReadWrite)){
		KMessageBox::error(this,
					i18n("Failed to create %1, please make verify your working directory settings.").arg(hiderrName));
		return false;
	}
	QTextStream hideRR(&hiderr);
	///////////////////////////////////////////////////////////////////////////////////
	
	
	top->generateMappings(progress, pathSpec, hideHFS, hideJoi, hideRR);
	
	for(CdboDataFolder *i=(CdboDataFolder *)top->firstChild(); ; i=(CdboDataFolder *)i->nextSibling()){
		if(!i)
			break;
		if(i->childCount() > 0){
			if(!readRow(i, pathSpec, hideHFS, hideJoi, hideRR)){
				delete progress;
				return false;
			}
		}
		i->generateMappings(progress, pathSpec, hideHFS, hideJoi, hideRR);
	}


	pathspec.close();
	hidejoi.close();
	hidehfs.close();
	hiderr.close();

	progress->setProgress( (int)(top->getSize()/1000));
	delete progress;
	
	return true;
}
//============================================
bool CdboDataFoldersView::readRow(CdboDataFolder *item, QTextStream &pathSpec, QTextStream &hideHFS, QTextStream &hideJoi, QTextStream &hideRR){
	CdboDataFolder *i;
	for(i=(CdboDataFolder *)item->firstChild(); ; i=(CdboDataFolder *)i->nextSibling()){
		if(!i){

			return true;
		}
		if ( progress->wasCancelled() )
			return false;

		if(i->childCount() > 0){
			if(!readRow(i, pathSpec, hideHFS, hideJoi, hideRR))
				return false;

		}
		i->generateMappings(progress, pathSpec, hideHFS, hideJoi, hideRR);
	}
	return true;
}
//============================================
void CdboDataFoldersView::postCreateCleanUp(){
	if(QFile::exists(pathspecName))
		QFile::remove(pathspecName);
	if(QFile::exists(hidejoiName))
		QFile::remove(hidejoiName);
	if(QFile::exists(hidehfsName))
		QFile::remove(hidehfsName);
	if(QFile::exists(hiderrName))
		QFile::remove(hiderrName);
}
//============================================
bool CdboDataFoldersView::persist(KConfig *file){

	showProgress(i18n( "Saving file mappings, please wait..."), i18n("&Cancel"), (int)(top->getSize()/1000));
	progress->setProgress(1 / 1000);

	top->persist(progress, file);
	CdboDataFolder *i;
	for(i=(CdboDataFolder *)top->firstChild(); ; i=(CdboDataFolder *)i->nextSibling()){
		if(!i)
			break;
		if(i->childCount() > 0){
			if(!persistRow(i, file)){
				delete progress;
				return false;
			}
		}
		i->persist(progress, file);
	}
	progress->setProgress( (int)(top->getSize()/1000));

	delete progress;
	return true;
}
//============================================
bool CdboDataFoldersView::persistRow(CdboDataFolder *item, KConfig *file){
	CdboDataFolder *i;
	for(i=(CdboDataFolder *)item->firstChild(); ; i=(CdboDataFolder *)i->nextSibling()){
		if(!i){

			return true;
		}
		if ( progress->wasCancelled() )
			return false;

		if(i->childCount() > 0){
			if(!persistRow(i, file))
				return false;
		}
		i->persist(progress, file);
	}
	return true;
}
//============================================
bool CdboDataFoldersView::loadFile(KConfig *file){
	file->setGroup("CD Bake Oven - CD File List");
	top->setText(0, file->readEntry("Name of CD", top->text(0)));
	//unsigned long size = file->readUnsignedLongNumEntry("Total Size");

	file->setGroup("top");
	QStringList list = file->readListEntry("Entries", ';');

	top->load(&list);

	list = file->readListEntry("Children", ';');
	if(list.count() > 0){
		bool immutable = false;
		QString buff = "";
		CdboDataFolder *item;
		QStringList list2;
		for(int i = 0; i < (int)list.count() ; i++){
			file->setGroup(list[i]);
			buff = file->readEntry("Name", "");
			if(file->readEntry("Immutable", "false") == "true")
				immutable = true;
			else
				immutable = false;
			item = new CdboDataFolder(top, buff, immutable);
			list2 = file->readListEntry("Entries", ';');
			item->load(&list2);

			list2 = file->readListEntry("Children", ';');
			if(list2.count() > 0){
				if(!loadRow(item, file, &list2)){
					return false;
				}
			}
		}
	}
	slotSelectFile(top);
	resetSize();
	return true;
}
//============================================
bool CdboDataFoldersView::loadRow(CdboDataFolder *item, KConfig *file, QStringList *list){
	bool immutable = false;
	QString buff = "";
	CdboDataFolder *newItem;
	QStringList list2;
	for(int i = 0; i < (int)list->count() ; i++){
		file->setGroup((*list)[i]);
		buff = file->readEntry("Name", "");
		if(file->readEntry("Immutable", "false") == "true")
			immutable = true;
		else
			immutable = false;
		newItem = new CdboDataFolder(item, buff, immutable);
		list2 = file->readListEntry("Entries", ';');
		newItem->load(&list2);

		list2 = file->readListEntry("Children", ';');
		if(list2.count() > 0){
			if(!loadRow(newItem, file, &list2)){
				return false;
			}
		}
	}
	estimate->updateStats(totalFolders - 1, totalFiles);
	return true;
}
