/***************************************************************************
                          CdboDataFoldersView.h  -  description
                             -------------------
    begin                : Sat Dec 1 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#ifndef CDBODATAFOLDERSVIEW_H
#define CDBODATAFOLDERSVIEW_H

#include <klistview.h>
#include <kaction.h>
#include <kio/job.h>

#include <qdict.h>

#include "CdboDataFolder.h"

class QProgressDialog;
class KConfig;
class CdboDataEstimate;

typedef QDict<CdboDataFolder> SourceAndPositionMap;

class CdboDataFoldersView : public KListView  {
friend class CdboDataFolder;

	Q_OBJECT
public:
	CdboDataFoldersView(QWidget *parent, const char *name);
	~CdboDataFoldersView();
	
	void setupActions(KActionCollection* ac);
	void plugMenuAction(KAction* action);
	
	void setEstimate(CdboDataEstimate *e);
	
	bool isParentOf(QListViewItem *i1, QListViewItem *i2);
	QString getPathToTop(CdboDataFolder *item);
	CdboDataFolder * getDragedItem() {return dragedItem; }
	CdboDataFolder * getParentForDrop() {return parentForDrop; }
	
	inline unsigned long getTotalSize(){ return top->getSize(); }
	bool canFit(int size, QString fname);
	bool canAccept(QString path, bool &stopHere);
	
	bool generateMappingsString();
	QString  getPathspecName() { return pathspecName;}
	QString  getHiderrName() { return hiderrName;}
	QString  getHidejoiName() { return hidejoiName;}
	QString  getHidehfsName() { return hidehfsName;}
	void postCreateCleanUp();
	
	bool persist(KConfig *file);
	bool loadFile(KConfig *file);
	bool copyRow(CdboDataFolder *item, CdboDataFolder *parent);
	
	
	int totalFolders;
	int totalFiles;

public slots:
	void mkdir();
	bool remove();
	void removeAll();
	void resetSize();
	void startRenameSlot();
	void cancelOperation();
	void currentModifiedSlot();
	
	void slotShowHidden();//works???
	
	void importSession(QString);
	void updateSettings();
	
	void slotSelectFile(QListViewItem *);
	void slotSelectFileConst(QListViewItem *);
	void setParentForDrop(CdboDataFolder *);
	void setDragedItem(QListViewItem * item) { dragedItem = (CdboDataFolder *)item; }
	
	void showProgress(const QString& labelText, const QString& cancelButtonText, int totalSteps);
	void updateProgress(int amount);
	QProgressDialog* getProgressDlg(){ return progress; } 
	void hideProgress();

signals:
	void folderClicked(CdboDataFolder *);
	void processStarted();
	void processDone();
	void modified();
	
	void menuRequested();
	
	void haveStatus(const QString &);
	void rejectedSize(const QString &);

protected:
	void addDir(CdboDataFolder *parentItem, const QString &dirPath, const QString &status = "");
	bool readRow(CdboDataFolder *item);
	bool readRow(CdboDataFolder *item, QTextStream &pathSpec, QTextStream &hideHFS, QTextStream &hideJoi, QTextStream &hideRR);
	bool persistRow(CdboDataFolder *item, KConfig *file);
	bool loadRow(CdboDataFolder *item, KConfig *file, QStringList *list);
	
	virtual bool acceptDrag (QDropEvent* event) const;
	virtual void contentsDragEnterEvent(QDragEnterEvent *);
	virtual void contentsDragLeaveEvent(QDragLeaveEvent *);
	virtual QRect drawDropVisualizer (QPainter *, QListViewItem *, QListViewItem *);
	
protected slots:
	virtual QDragObject *dragObject();
	void slotMenu(QListViewItem *, const QPoint &, int);
	
	void slotEntries( KIO::Job * , const KIO::UDSEntryList & );
	void slotResult( KIO::Job *);
	void killProcess(KIO::Job *);
	
	virtual void startDrag();
	
private:
	KActionCollection* actionCollection;
	KActionMenu* actionMenu;
	
	CdboDataFolder *top;
	
	QDict <SourceAndPositionMap>dicts;
	QPtrList <KIO::Job>jobs;
	
	QString pathspecName;
	QString hiderrName;
	QString hidejoiName;
	QString hidehfsName;
	
	KConfig *config;
	CdboDataEstimate *estimate;
	
	
	QProgressDialog *progress;
	CdboDataFolder *parentForDrop;
	CdboDataFolder *dragedItem;
	
	bool doListern;
	bool loadingSession;
	
	bool showHidden;
};

#endif
