/***************************************************************************
                          CdboViewWidgetBase.cpp  -  description
                             -------------------
    begin                : Sat Oct 21 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/ 

#include <qlayout.h>

#include <klocale.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kstdaction.h>
#include <kaction.h>
#include <kaction.h>

#include "CdboViewWidgetBase.h"
#include "CdboViewWidgetBase.moc"

CdboViewWidgetBase::CdboViewWidgetBase(QWidget *parent, const char *name) :
			QFrame(parent, name){

	mainLayout = new QGridLayout(this);
	
	listViewHolder = new QFrame(this, "listViewHolder");
	mainLayout->addWidget(listViewHolder, 1, 0);

	estimateHldr = new CdboExpandPnl(this, "estimateHldr");
	mainLayout->addWidget(estimateHldr, 1, 1);

	mainLayout->setColStretch(0, 5);
	mainLayout->setRowStretch(1, 10);
	
	setupActions();
	
}
//===================================
CdboViewWidgetBase::~CdboViewWidgetBase(){
}
//===================================
void CdboViewWidgetBase::setupActions(){
	actionCollection = new KActionCollection( this, "CdboViewWidgetBase::actionCollection" );
	showEstimateAction = new KToggleAction(i18n("Show Estimate"), KShortcut::null(),
			this, SLOT(showEstimate()), getActionCollection(), "show_estimate_action");
}
//===================================
void CdboViewWidgetBase::readOptions(KConfig* config){
	bool defaultConfig = false;
	if(!config){
		config = new KConfig("cdbakeovenrc");
		defaultConfig = true;
	}
	
	config->setGroup("Settings of " + QString(name()));
	
	showEstimateAction->setChecked(config->readBoolEntry("Show Estimate", true));
	showEstimate();
	
	
	if(defaultConfig)
		delete config;
}
//===================================
void CdboViewWidgetBase::saveOptions(KConfig* config){
	bool defaultConfig = false;
	if(!config){
		config = new KConfig("cdbakeovenrc");
		defaultConfig = true;
	}
	config->setGroup("Settings of " + QString(name()));
	config->writeEntry("Show Estimate", showEstimateAction->isChecked());
	config->sync();
	
	if(defaultConfig)
		delete config;
}
//==============================
void CdboViewWidgetBase::showEstimate(){
	if(!showEstimateAction->isChecked())
		estimateHldr->hide();
	else
		estimateHldr->show();
}
//===================================
void CdboViewWidgetBase::plugView(QWidget *listView){
	QHBoxLayout* listViewHolderLayout = new QHBoxLayout( listViewHolder );
	listViewHolderLayout->setSpacing( 0 );
	listViewHolderLayout->setMargin( 0 );
	
	listViewHolderLayout->addWidget( listView );
}
//===================================
void CdboViewWidgetBase::plugEstimate(QWidget *estimate, bool show){
	estimateHldr->plugWidget(estimate);
	showEstimateAction->setChecked(show);
	showEstimate();
}
//===================================
void CdboViewWidgetBase::resetCursor(){
	unsetCursor();
}
//===================================
void CdboViewWidgetBase::setWaitCursor(){
	setCursor( waitCursor );
}

